/**
 * @(#)FileProperties.java 1.0 03/02/07
 *
 * Copyright (2003) Mediterranean
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Boston, MA 02111.
 *
 *
 */

package mediaManager.video;

import java.io.*;

abstract class FileProperties {

    /* The resolution (width x heigth) */
    private String _videoResolution = "";
    /* The video codec */
    private String _videoCodec = "";
    /* The video rate (fps) */
    private int _videoFps = -1;
    /* The video bit rate (kbps) */
    private int _videoBitRate = -1;
    /* The video duration (seconds) */
    private int _duration = -1;
    /* The audio codec */
    private String _audioCodec = "";
    /* The audio rate (Hz) */
    private int _audioSampleRate = -1;
    /* The audio bit rate (kbps) */
    private int _audioBitRate = -1;
    /* The audio channels */
    private int _audioChannels = -1;
    private int _width = -1;
    private int _height = -1;

    /**
     * Returns the resolution.
     */
    public String getVideoResolution() {
        return _videoResolution;
    }

    /**
     * Returns the video codec.
     */
    public String getVideoCodec() {
        return _videoCodec;
    }

    public int getVideoFps() {
        return _videoFps;
    }

    /**
     * Returns the video bit rate.
     */
    public int getVideoBitRate() {
        return _videoBitRate;
    }

    /**
     * Returns the duration.
     */
    public int getDuration() {
        return _duration;
    }

    /**
     * Returns the audio codec.
     */
    public String getAudioCodec() {
        return _audioCodec;
    }

    /**
     * Returns the audio rate.
     */
    public int getAudioSampleRate() {
        return _audioSampleRate;
    }

    /**
     * Returns the audio bit rate.
     */
    public int getAudioBitRate() {
        return _audioBitRate;
    }

    /**
     * Returns the audio channels.
     */
    public int getAudioChannels() {
        return _audioChannels;
    }

    public void setVideoWidth(int width) {
        _width = width;
    }
    
    public void setVideoHeight(int height) {
        _height = height;
    }

    public int getVideoWidth() {
        return _width;
    }

    public int getVideoHeight() {
        return _height;
    }

    /**
     * Sets the resolution.
     */
    public void setVideoResolution(String videoResolution) {
        _videoResolution = videoResolution;
    }

    /**
     * Sets the video codec (vids handler).
     */
    public void setVideoCodec(String videoCodec) {
        _videoCodec = videoCodec;
    }

    /**
     * Sets the video rate.
     */
    public void setVideoFps(int fps) {
        _videoFps = fps;
    }

    /**
     * Sets the video bit rate.
     */
    public void setVideoBitRate(int videoBitRate) {
        _videoBitRate = videoBitRate;
    }

    /**
     * Sets the duration.
     */
    public void setDuration(int duration) {
        _duration = duration;
    }

    /**
     * Sets the audio codec (auds handler).
     */
    public void setAudioCodec(String audioCodec) {
        _audioCodec = audioCodec;
    }

    /**
     * Sets the audio rate.
     */
    public void setAudioRate(int audioSampleRate) {
        _audioSampleRate = audioSampleRate;
    }

    /**
     * Sets the audio bit rate.
     */
    public void setAudioBitRate(int audioBitRate) {
        _audioBitRate = audioBitRate;
    }

    /**
     * Sets the audio channels.
     */
    public void setAudioChannels(int audioChannels) {
        _audioChannels = audioChannels;
    }

    /**
     * Processes a file from the given DataInputStream.
     */
    public void process(DataInputStream dataStream) throws Exception {
    }

    /**
     * Reads an unsigned byte and returns its int representation.
     */
    public int readUnsignedByte(DataInputStream dataStream)
        throws Exception {
        int data = dataStream.readUnsignedByte();
        if (data == -1) {
            throw new Exception("Unexpected end of stream.");
        }
        return data;
    }

    /**
     * Reads n unsigned bytes and returns it in an int[n].
     */
    public int[] readUnsignedBytes(DataInputStream dataStream, int n)
        throws Exception {
        int[] data = new int[n];
        for (int i = 0; i < data.length; i++) {
            data[i] = readUnsignedByte(dataStream);
        }
        return data;
    }

    /**
     * Reads an unsigned 16-bit integer.
     */
    public int readUnsignedInt16(DataInputStream dataStream)
        throws Exception {
        return (
            readUnsignedByte(dataStream) | (readUnsignedByte(dataStream) << 8));
    }

    /**
     * Reads an unsigned 32-bit integer.
     */
    public int readUnsignedInt32(DataInputStream dataStream)
        throws Exception {
        return (
            readUnsignedInt16(dataStream)
                | (readUnsignedInt16(dataStream) << 16));
    }

    /**
     * Discards n bytes.
     */
    public void skipBytes(DataInputStream dataStream, int n)
        throws Exception {
        readUnsignedBytes(dataStream, n);
    }
}
