package mediaManager.video;

import java.io.*;
import java.util.*;

public class FilePropertiesMPEG extends FileProperties {

    private final int[] PICTURE_START_CODE = { 0x00, 0x00, 0x01, 0x00 };
    private final int[] USER_DATA_START_CODE = { 0x00, 0x00, 0x01, 0xb2 };
    private final int[] SEQUENCE_HEADER_CODE = { 0x00, 0x00, 0x01, 0xb3 };
    private final int[] SEQUENCE_END_CODE = { 0x00, 0x00, 0x01, 0xb7 };
    private final int[] GROUP_START_CODE = { 0x00, 0x00, 0x01, 0xb8 };

    /**
     * Processes a file from the given DataInputStream.
     **/
    public void process(DataInputStream dataStream) throws Exception {
        System.out.println("Start processing MPEG file.");
        dataStream.reset();
        int[] code;

        boolean found = false;
        do {
            code = getNextStartCode(dataStream);
            if (Arrays.equals(code, SEQUENCE_HEADER_CODE)) {
                processSequenceHeader(dataStream);
                found = true;
            }
        } while (!Arrays.equals(code, SEQUENCE_END_CODE) && !found);
    }

    /**
     * Scans the dataStrem until it finds a four-byte sequence starting
     * with 0x00 0x00 0x01 followed by 0x?? and then returns it.
     **/
    private int[] getNextStartCode(DataInputStream dataStream) throws Exception {
        
        int i;

        int[] data = new int[4];
        for (i = 0; i < 3; i++) {
            data[i] = readUnsignedByte(dataStream);
        }
        /* Loops until a 0x00 0x00 0x01 0x?? sequence has been found... */
        boolean found = false;
        do {
            data[i] = readUnsignedByte(dataStream);
            switch (i) {
                case (0) :
                    if (data[1] == 0 && data[2] == 0 && data[3] == 1) {
                        found = true;
                        break;
                    }
                    i = 1;
                    break;
                case (1) :
                    if (data[2] == 0 && data[3] == 0 && data[0] == 1) {
                        found = true;
                        break;
                    }
                    i = 2;
                    break;
                case (2) :
                    if (data[3] == 0 && data[0] == 0 && data[1] == 1) {
                        found = true;
                        break;
                    }
                    i = 3;
                    break;
                case (3) :
                    if (data[0] == 0 && data[1] == 0 && data[2] == 1) {
                        found = true;
                        break;
                    }
                    i = 0;
                    break;
            }
        } while (!found);

        data[3] = data[i];
        data[0] = 0;
        data[1] = 0;
        data[2] = 1;

        return data;
    }

    /**
     * Processes the header sequence to obtain the properties...
     **/
    private void processSequenceHeader(DataInputStream dataStream)
        throws Exception {
        setVideoCodec("MPEG");
        setAudioCodec("MPEG");
        int[] data = readUnsignedBytes(dataStream, 7);

        int width = (data[0] << 4) + ((data[1] >> 4) & 0x0f);
        int height = (data[2] + ((data[1] & 0x0f) << 8));
        setVideoResolution(width + " x " + height);
        setVideoWidth(width);
        setVideoHeight(height);

        switch (data[3] & 0x0f) {
            case (0) :
                throw new Exception("MPEG file corrupted (invalid frame rate).");
            case (1) :
            case (2) :
                setVideoFps(24);
                break;
            case (3) :
                setVideoFps(25);
                break;
            case (4) :
            case (5) :
                setVideoFps(30);
                break;
            case (6) :
                setVideoFps(50);
                break;
            case (7) :
            case (8) :
                setVideoFps(60);
                break;
        }

        setVideoBitRate(Math.round((float) (((data[4] << 10)
                        + (data[5] << 2)
                        + ((data[6] >> 6) & 0x03))
                        * 400) / 1000F));
    }
}