/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;

import rjw.dc.core.Repository;
import rjw.flex.settings.Settings;

/**
 * Creates and opens files using the JFileChooser class from Java.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class BrowserDialog extends JFrame {

    private final JFileChooser browser = new JFileChooser();
    private String extension = null;

    /**
     * Initializes this dialog
     */
    public BrowserDialog(String sTitle) {
        browser.setBackground(Settings.getValueAsColor(Repository.settings.stColor));
        browser.setMultiSelectionEnabled(false);
        this.setTitle(sTitle);
    }
    
    public BrowserDialog(String sTitle, String extension) {
        this(sTitle);
        this.extension = extension;
    }

    /**
     * Opens a dialog to select a directory and returns it (or null)
     *
     * @return (File)   the selected file (can be null)
     */
    public File showSelectDirectoryDialog(Component c, File file) {
        if (file != null) {
            browser.setSelectedFile(file);
        }
        browser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        browser.showOpenDialog(c);
        return browser.getSelectedFile();
    }

    /**
     * Opens a save file dialog and returns the selected file or null
     *
     * @return (File)   the selected file (can be null)
     */
    public File showCreateFileDialog(Component c) {
        browser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        browser.showSaveDialog(c);
        File file = browser.getSelectedFile();
        
        if (file == null) {
            return null;
        } else if (extension != null){
            String s = file.toString();
            if (!s.endsWith(extension)) {
                s = s + "." + extension;
            }
            try {
                file.delete();
            } catch (Exception ignore) {}
            file = new File(s);
            
        }
        return file;
    }

    /**
     * Returns the selected file (can be null)
     *
     * @return File the selected file (can be null)
     */
    public File showOpenFileDialog(Component c, File file) {
        if (file != null) {
            browser.setSelectedFile(file);
        }
        browser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        browser.showOpenDialog(c);
        return browser.getSelectedFile();
    }
}
