/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JLabel;
import javax.swing.JPanel;

import rjw.dc.core.DataCrow;
import rjw.dc.messages.Messages;
import rjw.dc.util.SwingUtil;

/**
 * The godfather of all panels
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataCrowPanel extends JPanel {

    public DataCrowPanel() {
        this.addMouseListener(new ActiveFrame());   
    }

    protected JLabel labelStatus = new JLabel();

    /**
     * Sets the status text at the bottom of the screen
     *
     * @param text  display String
     */
    public void setStatus(String text) {
        labelStatus.setText(text);
    }
    
    public static class ActiveFrame implements MouseListener {
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {
            SwingUtil.setRootFrame(DataCrow.mainFrame);
        }
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {} 
                      
    }    
    
    /**
     * Applies the background color to the sub panels
     *
     * @param color     background color
     */
    public void setBackgrounds(Color color) {
        this.setBackground(color);

        try {
            setCompBackColors(this.getComponents(), color);
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applies the background color to all the components
     *
     * @param comps     the components on which the color is applied
     * @param color     background color
     */
    protected void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
                comps[i].getParent().setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }    
}
