/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Dialog, used to asked confirmation of the settings for exporting CD
 * or Hard Disk information to a file. The dialog shows the selected
 * the settings
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ExportCDDialog extends JDialog {

    private JPanel              panelVerificatie;
    private JPanel              panelActions;

    private JLabel              labelLocation;
    private JLabel              labelAskPerDir;
    private JLabel              labelTarget;
    private JLabel              labelCDName;

    private JLabel              labelLocationValue;
    private JLabel              labelAskPerDirValue;
    private JLabel              labelTargetValue;
    private JLabel              labelCDNameValue;
    private JLabel              labelToFile;
    private JLabel              labelToFileValue;
    private JLabel              labelToTable;
    private JLabel              labelToTableValue;

    private JButton             buttonYes;
    private JButton             buttonNo;

    /** Object which holds the results of the pressed button */
    public static Object        result          = null;

    /**
     * Initializes and builds the dialog, then it shown. Modal is used
     *
     * @param fromLocation  the drive letter
     * @param askPerDir     confirmation per directory?
     * @param toFile        absolute path of the export file
     * @param CDName        the location name, used for grouping data
     */
    public ExportCDDialog(  String fromLocation,
                            String askPerDir,
                            String sTarget,
                            String CDName,
                            String toFile,
                            String toTable) {
        buildPanel();

        labelLocation.setText(Resources.getText(DCLabels.lblFromLocation));
        labelLocationValue.setText(fromLocation);

        labelAskPerDir.setText(Resources.getText(DCLabels.lblAskPerDir));
        labelAskPerDirValue.setText(askPerDir);

        labelTarget.setText(Resources.getText(DCLabels.lblToFile));
        labelTargetValue.setText(sTarget);

        labelCDName.setText(Resources.getText(DCLabels.lblLocationName));
        labelCDNameValue.setText(CDName);

        labelToFile.setText(Resources.getText(DCLabels.lblExportToFile));
        labelToFileValue.setText(toFile);
        labelToTable.setText(Resources.getText(DCLabels.lblExportToTable));
        labelToTableValue.setText(toTable);

        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        this.pack();
        this.setModal(true);
        this.setVisible(true);
    }

    /**
     * Closes and disposes the dialog
     */
    public void close() {
        this.setVisible(false);
        this.setModal(false);
        this.dispose();
    }

    /**
     * Creates the dialog
     */
    private void buildPanel() {
        panelVerificatie = new JPanel(false);
        panelActions     = new JPanel(false);

        // ************************************************************************
        // VERIFICATIE PANEL
        // Set layout style
        GridBagLayout gridbVerificatie    = new GridBagLayout();
        panelVerificatie.setLayout(gridbVerificatie);
        panelVerificatie.setBackground(Settings.getValueAsColor(
                                       Repository.settings.stColor));

        // Initialize components
        labelLocation        = ComponentFactory.getLabel("", 200);
        labelLocationValue   = ComponentFactory.getLabel("", 150);
        labelAskPerDir       = ComponentFactory.getLabel("", 200);
        labelAskPerDirValue  = ComponentFactory.getLabel("", 150);
        labelTarget          = ComponentFactory.getLabel("", 200);
        labelTargetValue     = ComponentFactory.getLabel("", 150);
        labelCDName          = ComponentFactory.getLabel("", 200);
        labelCDNameValue     = ComponentFactory.getLabel("", 150);
        labelToFile          = ComponentFactory.getLabel("", 200);
        labelToFileValue     = ComponentFactory.getLabel("", 150);
        labelToTable         = ComponentFactory.getLabel("", 200);
        labelToTableValue    = ComponentFactory.getLabel("", 150);

        // Build the panel
        panelVerificatie.add(labelLocation,        new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelLocationValue,   new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelAskPerDir,       new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelAskPerDirValue,  new GridBagConstraints( 1, 1, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelTarget,          new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelTargetValue,     new GridBagConstraints( 1, 2, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelCDName,          new GridBagConstraints( 0, 3, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelCDNameValue,     new GridBagConstraints( 1, 3, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelToFile,          new GridBagConstraints( 0, 4, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelToFileValue,     new GridBagConstraints( 1, 4, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelToTable,         new GridBagConstraints( 0, 5, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));
        panelVerificatie.add(labelToTableValue,    new GridBagConstraints( 1, 5, 1, 1, 0.0, 0.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                   new Insets(0, 10, 0, 10), 0, 0));


        // ************************************************************************
        // ACTIONS PANEL
        // Set layout style
        GridBagLayout gridbActions    = new GridBagLayout();
        panelActions.setLayout(gridbActions);

        // Initialize components
        buttonYes       = ComponentFactory.getButton(Resources.getText(DCLabels.lblYes), 100);
        buttonNo        = ComponentFactory.getButton(Resources.getText(DCLabels.lblNo),  100);
        buttonNo.addActionListener(new NoButtonAction());
        buttonYes.addActionListener(new YesButtonAction());

        // Build the panel
        panelActions.add(buttonYes,            new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets(0, 10, 0, 5), 0, 0));
        panelActions.add(buttonNo,             new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets(0, 0, 0, 10), 0, 0));
        panelActions.setBackground(Settings.getValueAsColor(
                                   Repository.settings.stColor));

        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        this.getContentPane().setLayout(new GridBagLayout());

        // Initialize components
        this.setResizable(false);
        this.setBounds(100,100,100,100);
        this.setTitle(Resources.getText(DCLabels.lblVerification));

        // Build panel
        this.getContentPane().add(panelVerificatie,  new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                     new Insets(0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelActions,      new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                     new Insets(10, 0, 10, 0), 0, 0));
        this.getContentPane().setBackground(Settings.getValueAsColor(
                                            Repository.settings.stColor));
    }


    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    private class YesButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(true);
            close();
        }
    }

    private class NoButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(false);
            close();
        }
    }

}


