package rjw.dc.console;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.TableColumn;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

public class FieldSelectorDialog extends JDialog {
    
    private final JList leftList;
    private final JList rightList;
    private final JButton buttonLeft;
    private final JButton buttonRight;
    private final JButton buttonCancel;
    private final JButton buttonOK;
    
    private final TableColumn[] columns;
    
    private static boolean success = false;
    
    public FieldSelectorDialog(Vector visible, Vector hidden) {

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setSizeSetting();
            }
        });
        
        leftList = ComponentFactory.getList(visible);
        rightList = ComponentFactory.getList(hidden);
        buttonRight = ComponentFactory.getButton(">>");
        buttonLeft = ComponentFactory.getButton("<<");
        buttonCancel = ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
        buttonOK = ComponentFactory.getButton(Resources.getText(DCLabels.lblOK));
        
        this.columns = new TableColumn[visible.size() + hidden.size()];
        
        int counter = 0;
        for (Enumeration enum = visible.elements(); enum.hasMoreElements(); ) {
            columns[counter++] = (TableColumn) enum.nextElement();
        }

        for (Enumeration enum = hidden.elements(); enum.hasMoreElements();) {
            columns[counter++] = (TableColumn) enum.nextElement();
        }

        buildDialog();
        init(visible, hidden);
    }
    
    public void init(Vector visible, Vector hidden) {
        String[] headersVisible = new String[visible.size()];
        String[] headersHidden = new String[hidden.size()];
        
        int counter = 0;
        for (Enumeration enum = visible.elements(); enum.hasMoreElements(); ) {
            headersVisible[counter++] = ((TableColumn) enum.nextElement()).getHeaderValue().toString();   
        }
        
        counter = 0;
        for (Enumeration enum = hidden.elements(); enum.hasMoreElements(); ) {
            headersHidden[counter++] = ((TableColumn) enum.nextElement()).getHeaderValue().toString();   
        }
        
        leftList.setListData(headersVisible);
        rightList.setListData(headersHidden);
    }

    public static boolean isSuccessful() {
        return success;   
    }

    public Vector getVisibleFields() {
        Vector cols = new Vector();
        for (int i = 0; i < leftList.getModel().getSize(); i++) {
            Object header = leftList.getModel().getElementAt(i);
            for (int j = 0; j < columns.length; j++) {
                if (header.equals(columns[j].getHeaderValue())) {
                    cols.addElement(columns[j]);
                }   
            }
        }
        return cols;
    }
    
    public Vector getHiddenFields() {
        Vector cols = new Vector();
        for (int i = 0; i < rightList.getModel().getSize(); i++) {
            Object header = rightList.getModel().getElementAt(i);
            
            for (int j = 0; j < columns.length; j++) {
                if (header.equals(columns[j].getHeaderValue())) {
                    cols.addElement(columns[j]);
                }   
            }
        }
        return cols;
    }
    
    public void buildDialog() {
        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        this.getContentPane().setBackground(color);

        JPanel panelLeft = new JPanel(false);
        panelLeft.setBackground(color);
        JPanel panelAction = new JPanel(false);
        panelAction.setBackground(color);
        JPanel panelRight = new JPanel(false);
        panelRight.setBackground(color);
        JPanel panelConfirm = new JPanel(false);
        panelConfirm.setBackground(color);
            
        // The left part
        panelLeft.setLayout(new GridBagLayout());        
        JScrollPane scrollLeft = new JScrollPane(leftList);
        panelLeft.add(scrollLeft,     new GridBagConstraints( 0, 0, 1, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
    
        // The actionPanel
        panelAction.setLayout(new GridBagLayout());    
        buttonRight.addActionListener(new MoveToRightAction());
        buttonLeft.addActionListener(new MoveToLeftAction()); 
        
        panelAction.add(buttonRight,  new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
        panelAction.add(buttonLeft,   new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
    
        // The right part
        panelRight.setLayout(new GridBagLayout());        
        JScrollPane scrollRight = new JScrollPane(rightList);
        
        panelRight.add(scrollRight,   new GridBagConstraints( 0, 0, 1, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                                      new Insets(5, 5, 5, 5), 0, 0));                         

        // The confirmation panel
        panelConfirm.setLayout(new GridBagLayout());  
        buttonOK.addActionListener(new OkAction());
        buttonCancel.addActionListener(new CancelAction());
        panelConfirm.add(buttonOK,      new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));                         
        panelConfirm.add(buttonCancel,  new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));  
    
        // Main panel            
        panelLeft.setBorder(    ComponentFactory.getBorder(
                                Resources.getText(DCLabels.lblVisibleFields)));
        panelRight.setBorder(   ComponentFactory.getBorder(
                                Resources.getText(DCLabels.lblHiddenFields)));
                                      
        this.getContentPane().setLayout(new GridBagLayout());
        this.getContentPane().add(panelLeft,   new GridBagConstraints( 0, 0, 1, 1, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(panelAction, new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(20, 10, 10, 10), 0, 0)); 
        this.getContentPane().add(panelRight,  new GridBagConstraints( 2, 0, 1, 1, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        this.getContentPane().add(panelConfirm,new GridBagConstraints( 0, 1, 3, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));                               
            
        this.setResizable(true);
        this.pack(); 
        this.setModal(true);
        this.setSize(Settings.getValueAsDimension(Repository.settings.stFieldSelectorSize));
    }    

    private void setSizeSetting() {
        Settings.setValue(Repository.settings.stFieldSelectorSize,
                          getSize());
    }
    
    private class OkAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setSizeSetting();
            success = true;
            setModal(false);
            setVisible(false);
            dispose();
        }        
    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            success = false;
            setModal(false);
            setVisible(false);
            dispose();
        }        
    }
    
    private class MoveToRightAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int[] selected = leftList.getSelectedIndices();
            if (selected != null || selected.length > 0) {
                Vector vLeft = new Vector();
                Vector vRight = new Vector();
                for (int i = 0; i < leftList.getModel().getSize(); i++) {
                    boolean moved = false;
                    for (int j = 0; j < selected.length; j++) {
                        if (i == selected[j]) {
                            vRight.addElement(leftList.getModel().getElementAt(i));
                            moved = true;
                        }   
                    }
                    
                    if (!moved) {
                        vLeft.addElement(leftList.getModel().getElementAt(i));
                    }                        
                }
                
                for (int i = 0; i < rightList.getModel().getSize(); i++) {
                    vRight.addElement(rightList.getModel().getElementAt(i));
                }
                
                leftList.setListData(vLeft);
                rightList.setListData(vRight);
            }
        }
    }
    
    private class MoveToLeftAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            int[] selected = rightList.getSelectedIndices();
            if (selected != null || selected.length > 0) {
                Vector vLeft = new Vector();
                Vector vRight = new Vector();
                for (int i = 0; i < rightList.getModel().getSize(); i++) {
                    boolean moved = false;
                    for (int j = 0; j < selected.length; j++) {
                        if (i == selected[j]) {
                            vLeft.addElement(rightList.getModel().getElementAt(i));
                            moved = true;
                        }   
                    }
                    
                    if (!moved) {
                        vRight.addElement(rightList.getModel().getElementAt(i));
                    }                        
                }
                
                for (int i = 0; i < leftList.getModel().getSize(); i++) {
                    vLeft.addElement(leftList.getModel().getElementAt(i));
                }
                
                leftList.setListData(vLeft);
                rightList.setListData(vRight);
            }
        }
    }
}
