/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.dc.util.SwingUtil;

/**
 * Displays all events
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class LogForm extends JFrame {

    private JTextArea           text;
    private JLabel              labelPicture;
    private JLabel              labelVersion;
    private JLabel              labelProductName;
    private JLabel              labelLastUpd;

    private JPanel              panelLogging;
    private JPanel              panelPicture;
    private JPanel              panelProductInfo;

    /**
     * Initializes and builds the frame
     */
    public LogForm() {
        buildPanel();
        this.addMouseListener(new ActiveFrame());
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        panelLogging        = new JPanel(false);
        panelPicture        = new JPanel(false);
        panelProductInfo    = new JPanel(false);

        // ************************************************************************
        // PICTURE PANEL
        // Set layout style
        panelPicture.setLayout(new GridBagLayout());

        // Initialize components
        labelPicture    = new JLabel();
        labelPicture.setIcon(new ImageIcon("./Images/logo.jpg"));

        labelPicture.setPreferredSize(new Dimension(520, 200));

        // Build the panel
        panelPicture.add(labelPicture,  new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                       ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL ,
                                        new Insets( 0, 5, 0, 5), 0, 0));


        // ************************************************************************
        // SEARCH OPTION PANEL
        // Set layout style
        panelPicture.setLayout(new GridBagLayout());

        // Initialize components
        text = ComponentFactory.getTextArea();
        text.setPreferredSize(new Dimension(510, 2500));
        text.setMargin(new Insets(5,5,5,5));
        text.setEditable(false);
        text.setDoubleBuffered(true);
        text.setWrapStyleWord(true);
        text.setLineWrap(true);
        text.setBackground(ComponentFactory.getTableColor());

        JScrollPane scroll = new JScrollPane(text);
        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroll.setPreferredSize(new Dimension(510, 300));

        // Build the panel
        panelLogging.add(   scroll,  new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                     new Insets( 0, 5, 0, 5), 0, 0));

        // ************************************************************************
        // PRODUCT INFORMATION PANEL
        // Set layout style
        panelProductInfo.setLayout(new GridBagLayout());

        // Initialize components
        labelProductName    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDataCrowFullName), 200);
        labelVersion        = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion), 200);
        labelLastUpd        = ComponentFactory.getLabel(Resources.getText(DCLabels.lblDataCrowDate), 200);

        labelProductName.setPreferredSize(new Dimension(515, 16));
        labelVersion.setPreferredSize(new Dimension(515, 16));
        labelLastUpd.setPreferredSize(new Dimension(515, 16));

        // Build the panel
        panelProductInfo.add(labelProductName, new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 5, 0, 0), 0, 0));
        panelProductInfo.add(labelVersion,     new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 5, 0, 0), 0, 0));
        panelProductInfo.add(labelLastUpd,     new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                               new Insets( 0, 5, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        this.getContentPane().setLayout(new GridBagLayout());

        // Initialize components
        setTitle(Resources.getText(DCLabels.lblLogEvent));
        setResizable(false);

        ImageIcon image = new ImageIcon("./Images/LogForm.gif");
        this.setIconImage(image.getImage());


        // Build the panel
        this.getContentPane().add(   panelPicture,  new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(   panelLogging,  new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelProductInfo, new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 0, 0, 5, 0), 0, 0));

        this.setBounds(100,100,100,100);
        pack();
    }

    /**
     * Moves this frame to the front
     */
    public void logToFront(boolean show) {
        this.setVisible(show);

        if (show) {
            this.toFront();
        }
    }

    /**
     * Writes an event to the screen. New events are written to the top
     *
     * @param message   event String
     */
    public void writeToScreen(String message) {
        text.insert(message, 0);
    }
    
    /**
     * Forces a refresh of the content of the Software categories combobox
     */
    public class ActiveFrame implements MouseListener {
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {
            SwingUtil.setRootFrame((JFrame) e.getSource());
        }
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}        
    }     
}
