/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;

import rjw.apps.organizemp3.OrganizeMp3UI;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.da.DataExchangePanel;
import rjw.dc.console.movie.InsertMoviePanel;
import rjw.dc.console.movie.SearchMoviePanel;
import rjw.dc.console.mp3.InsertMP3Panel;
import rjw.dc.console.mp3.SearchMP3Panel;
import rjw.dc.console.note.NotePanel;
import rjw.dc.console.software.InsertSoftwarePanel;
import rjw.dc.console.software.SearchSoftwarePanel;
import rjw.dc.console.software.SoftwareCatagoryForm;
import rjw.dc.console.sql.SQLPanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.mp3.Mp3;
import rjw.dc.core.objects.software.Software;
import rjw.dc.core.objects.software.SoftwareCatagories;
import rjw.dc.core.objects.software.SoftwareCatagory;
import rjw.dc.db.Database;
import rjw.dc.db.DatabaseConnection;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;
import rjw.flex.settings.SettingsFile;
import rjw.flex.settings.ui.SettingsView;

/**
 * Holder of all the UI panels. The panels are shown as tabs.
 * By using this frame background colors can be set and requests can be past
 * to the panels by using the <url>RequestorCollection</url> class.
 * At the top of the frame a menubar is shown. From this menu the module can
 * be selected, the log form and record form can be opened and the MP3
 * organizer can be selected.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class MainFrame extends JFrame {
    private static JTabbedPane tabbedPane;

    private static JButton toolCatagory;
    private static JButton toolMP3Organizer;

    public  static SoftwareCatagoryForm     catagoryForm = new SoftwareCatagoryForm();
    private static SearchSoftwarePanel      searchSoftwarePanel;
    private static InsertSoftwarePanel      insertSoftwarePanel;
    private static SQLPanel                 sqlPanel;
    private static NotePanel                notePanel;
    private static SearchMP3Panel           searchMP3Panel;
    private static InsertMP3Panel           insertMP3Panel;
    private static InsertMoviePanel         insertMoviePanel;
    private static SearchMoviePanel         searchMoviePanel;
    private static DataExchangePanel        daSoftwarePanel;
    private static DataExchangePanel        daMp3Panel;
    private static DataExchangePanel        daMoviePanel;
    private static SettingsView             settingsView;

    private static JComboBox                comboModule;
    private static JMenuBar                 menuBar;

    private static boolean                  firstTime       = true;

    /**
     * Initializes all the panels and puts them in a tabbed pane. The menubar
     * is build and its actions specified. The background color for all UI
     * components is set
     */
    public MainFrame() {
        if (firstTime == true) {
            GridBagConstraints c = new GridBagConstraints();
            GridBagLayout gridbag = new GridBagLayout();
            this.getContentPane().setLayout(gridbag);

            c.fill = GridBagConstraints.VERTICAL;
            tabbedPane = new JTabbedPane();

            settingsView = new SettingsView("Data Crow Settings Panel");
            settingsView.setDisclaimer(
                "              __                       " + '\n' +
                "       <-----/@@\\----->               " + '\n' +
                "      <-< <  \\\\//  > >->             " + '\n' +
                "        <-<-\\ __ /->->                " + '\n' +
                "        Data /  \\ Crow                " + '\n' +
                "            ^    ^                     " + '\n' +
                "Created By Robert-Jan van der Waals" + '\n' +
                "Original product: CD Library 3.03" + '\n' +
                "" + '\n' +
                "This product includes Hypersonic SQL."  + '\n' +
                "Originally developed by Thomas Mueller and the Hypersonic SQL Group"  + '\n' +
                ""  + '\n' +
                "This product used JFlexSett 0.1 alpha" + '\n' +
                "Created By Robert-Jan van der Waals" + '\n' +
                "" + '\n' +
                "Find me at http://sourceforge.net/projects/datacrow");

            settingsView.setSize(Settings.getValueAsDimension(Repository.settings.stSettingsViewSize));
            ImageIcon image = new ImageIcon("./Images/Settings.gif");
            settingsView.setIconImage(image.getImage());

            searchSoftwarePanel = new SearchSoftwarePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitSearch), Messages._STANDARD);
            insertSoftwarePanel = new InsertSoftwarePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitInsert), Messages._STANDARD);
            daSoftwarePanel = new DataExchangePanel(Repository.module._SOFTWARE);
            daMp3Panel = new DataExchangePanel(Repository.module._MP3);
            daMoviePanel = new DataExchangePanel(Repository.module._MOVIE);
            Messages.setMessage(Resources.getText(DCMessages.msgInitDataExchange), Messages._STANDARD);
            sqlPanel = new SQLPanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitSQL), Messages._STANDARD);
            searchMP3Panel = new SearchMP3Panel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMP3Search), Messages._STANDARD);
            insertMP3Panel = new InsertMP3Panel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMP3Insert), Messages._STANDARD);
            notePanel = new NotePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitNote), Messages._STANDARD);
            insertMoviePanel = new InsertMoviePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMovieInsert), Messages._STANDARD);
            searchMoviePanel = new SearchMoviePanel();
            Messages.setMessage(Resources.getText(DCMessages.msgInitMovieSearch), Messages._STANDARD);

            try {
                setTitle(Resources.getText(DCLabels.lblMainTitle));
                setResizable(false);
                addWindowListener(new WindowAdapter() {
                    public void windowClosing(WindowEvent e) {
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgApplicationStops),
                        Messages._STANDARD);

                    if (DatabaseConnection.bServerConnection && 
                        Settings.getValueAsBoolean(Repository.settings.stCloseServerQuestion)) {
                            
                        GeneralDialog dialog = new GeneralDialog(
                                                   Resources.getText(DCMessages.msgCloseServer));
                                                   
                        if (GeneralDialog.result.booleanValue()) {
                            WorkFlow wf = new WorkFlow();
                            wf.runQueryDirect("SHUTDOWN");
                        }
                    } else {
                        Database.close();
                    }
                    
                    Settings.setValue(Repository.settings.stMainViewSize, getSize());
                    SettingsFile.save();

                    System.exit(0);
                    }
                });

                image = new ImageIcon("./Images/mainIcon.gif");
                this.setIconImage(image.getImage());

                menuBar = createMenuBar();

                int iDefModule = Settings.getValueAsInt(Repository.settings.stModule);
                if (iDefModule > -1) {
                    comboModule.setSelectedIndex(iDefModule);
                }
                setModule(false);

                searchSoftwarePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                insertSoftwarePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                sqlPanel.setBorder(BorderFactory.createRaisedBevelBorder());
                searchMP3Panel.setBorder(BorderFactory.createRaisedBevelBorder());
                insertMP3Panel.setBorder(BorderFactory.createRaisedBevelBorder());
                notePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                insertMoviePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                searchMoviePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                daSoftwarePanel.setBorder(BorderFactory.createRaisedBevelBorder());
                daMp3Panel.setBorder(BorderFactory.createRaisedBevelBorder());
                daMoviePanel.setBorder(BorderFactory.createRaisedBevelBorder());

                getContentPane().add(menuBar, new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                     new Insets(0, 0, 0, 0), 0, 0));
                getContentPane().add(tabbedPane, new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                     new Insets(0, 0, 0, 0), 0, 0));

                this.setBounds(50,50,50,50);

                pack();
                
                setHiddenFields();
            } catch (Error err) {
                Messages.setMessage(err.getMessage(), Messages._STANDARD);
            }
            final Color bgColor = Settings.getValueAsColor(Repository.settings.stColor);
            this.applyBackgroundColor(bgColor);
            
            this.setRequiredFields();

            this.setResizable(true);
            this.setSize(Settings.getValueAsDimension(Repository.settings.stMainViewSize));

            firstTime = false;
        }
    }

    /**
     * Creates the menu
     */
    private JMenuBar createMenuBar() {
        
        JButton toolLogForm =  ComponentFactory.getMenuButton(
                                ".//images//logForm.gif",
                                Resources.getText(DCLabels.lblOpenLog));
        JButton toolSettings = ComponentFactory.getMenuButton(
                                ".//images//Settings.gif",
                                Resources.getText(DCLabels.lblSettings));
        toolMP3Organizer =     ComponentFactory.getMenuButton(
                                ".//images/mp3.gif",
                                Resources.getText(DCLabels.lblOpenMP3Organizer));
        toolCatagory =         ComponentFactory.getMenuButton(
                                ".//images/category.gif",
                                Resources.getText(DCLabels.lblCatagoryForm));

        toolLogForm.setMnemonic(KeyEvent.VK_L);
        toolSettings.setMnemonic(KeyEvent.VK_E);
        toolMP3Organizer.setMnemonic(KeyEvent.VK_M);
        toolCatagory.setMnemonic(KeyEvent.VK_C);

        comboModule = ComponentFactory.getComboBox();
        comboModule.setFont(new Font("Arial", Font.BOLD, 12));

        comboModule.addItem(Resources.getText(DCLabels.lblModuleSoftware));
        comboModule.addItem(Resources.getText(DCLabels.lblModuleMP3));
        comboModule.addItem(Resources.getText(DCLabels.lblModuleMovie));
        comboModule.addActionListener(new ModuleAction());

        toolLogForm.addActionListener(new LogFormMenuAction());
        toolMP3Organizer.addActionListener(new MP3OrganizerFormMenuAction());
        toolSettings.addActionListener(new SettingsFormMenuAction());
        toolCatagory.addActionListener(new CatagoryFormMenuAction());

        JMenuBar toolBar = new JMenuBar();

        toolBar.add(toolLogForm); 
        toolBar.add(toolMP3Organizer);
        toolBar.add(toolCatagory);
        toolBar.add(toolSettings);
        toolBar.add(comboModule);

        return toolBar;
    }

    public void setRequiredFields() {
        searchSoftwarePanel.setRequiredFields(new Software());
        insertSoftwarePanel.setRequiredFields(new Software());
        searchMoviePanel.setRequiredFields(new Movie());
        insertMoviePanel.setRequiredFields(new Movie());
        searchMP3Panel.setRequiredFields(new Mp3());
        insertMP3Panel.setRequiredFields(new Mp3());
    }
    
    private static void setSearchConditionFields() {
        int[] softHidden  = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsSoftware);
        int[] softCatHidden  = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsSoftwareCat);
        int[] mp3Hidden   = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsMP3);
        int[] movieHidden = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsMovie);
        
        searchSoftwarePanel.panelSearchConds.removeHiddenFields(softHidden);
        searchMP3Panel.panelSearchConds.removeHiddenFields(mp3Hidden);
        searchMoviePanel.panelSearchConds.removeHiddenFields(movieHidden);
        catagoryForm.searchCat.panelSearchConds.removeHiddenFields(softCatHidden);
    }
    
    public static void setHiddenFields() {
        setSearchConditionFields();
        int[] softHidden  = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsSoftware);
        int[] softCatHidden  = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsSoftwareCat);
        int[] mp3Hidden   = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsMP3);
        int[] movieHidden = Settings.getValueAsIntegerArray(
                            Repository.settings.stHiddenFieldsMovie);

        searchSoftwarePanel.removeHiddenFields(softHidden);
        insertSoftwarePanel.removeHiddenFields(softHidden);
        searchMoviePanel.removeHiddenFields(movieHidden);
        insertMoviePanel.removeHiddenFields(movieHidden);
        searchMP3Panel.removeHiddenFields(mp3Hidden);
        insertMP3Panel.removeHiddenFields(mp3Hidden);
        catagoryForm.searchCat.removeHiddenFields(softCatHidden);
        catagoryForm.insertCat.removeHiddenFields(softCatHidden);
    }

    /**
     * Applies the background color to all panels and their components
     *
     * @color   background color
     */
    public void applyBackgroundColor(Color color) {
        try {
            searchSoftwarePanel.setBackground(color);
            insertSoftwarePanel.setBackground(color);
            sqlPanel.setBackground(color);
            searchMP3Panel.setBackground(color);
            insertMP3Panel.setBackground(color);
            notePanel.setBackground(color);
            insertMoviePanel.setBackground(color);
            searchMoviePanel.setBackground(color);
            daSoftwarePanel.setBackgrounds(color);
            daMp3Panel.setBackgrounds(color);
            daMoviePanel.setBackgrounds(color);
            catagoryForm.setBackgrounds(color);
            settingsView.setBackgroundColor(color);

            setCompBackColors(searchSoftwarePanel.getComponents(), color);
            setCompBackColors(insertSoftwarePanel.getComponents(), color);
            setCompBackColors(sqlPanel.getComponents(), color);
            setCompBackColors(insertMP3Panel.getComponents(), color);
            setCompBackColors(searchMP3Panel.getComponents(), color);
            setCompBackColors(this.getComponents(), color);
            setCompBackColors(notePanel.getComponents(), color);
            setCompBackColors(searchMoviePanel.getComponents(), color);
            setCompBackColors(insertMoviePanel.getComponents(), color);
            setCompBackColors(daSoftwarePanel.getComponents(), color);
            setCompBackColors(daMp3Panel.getComponents(), color);
            setCompBackColors(daMoviePanel.getComponents(), color);
            setCompBackColors(settingsView.getComponents(), color);
            setCompBackColors(catagoryForm.getComponents(), color);
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Applied the background color to all components
     *
     * @param comps     the UI components
     * @param color     background color
     */
    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Sets the module. Only the tabs for this module are shown. The module
     * is selected from the module combobox from the menubar
     */
    private void setModule(boolean checkSame) {
        
        int iModule = Settings.getValueAsInt(Repository.settings.stModule);

        if (checkSame && iModule == comboModule.getSelectedIndex()) {
            return;
        }
        
        catagoryForm = new SoftwareCatagoryForm();

        Icon icoExisting = new ImageIcon(".//images//existing.gif");
        Icon icoNew = new ImageIcon(".//images//new.gif");
        Icon icoExchange = new ImageIcon(".//images//exchange.gif");
        Icon icoNote = new ImageIcon(".//images//note.gif");
        Icon icoSQL = new ImageIcon(".//images//SQL.gif");

        if (comboModule.getSelectedItem().toString().trim()
            .equals(Resources.getText(DCLabels.lblModuleSoftware))) {

            tabbedPane.removeAll();
            tabbedPane.addTab(Resources.getText(DCLabels.lblExistingValues), icoExisting, searchSoftwarePanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblNewValues), icoNew, insertSoftwarePanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabDataExchange), icoExchange, daSoftwarePanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), icoSQL, sqlPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), icoNote, notePanel);
            toolCatagory.setVisible(true);
            toolMP3Organizer.setVisible(false);
        } else if (comboModule.getSelectedItem().toString().trim()
                    .equals(Resources.getText(DCLabels.lblModuleMP3))) {

            tabbedPane.removeAll();
            tabbedPane.addTab(Resources.getText(DCLabels.lblExistingValues), icoExisting, searchMP3Panel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblNewValues), icoNew, insertMP3Panel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabDataExchange), icoExchange, daMp3Panel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), icoSQL, sqlPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), icoNote, notePanel);
            toolCatagory.setVisible(false);
            toolMP3Organizer.setVisible(true);
        } else if (comboModule.getSelectedItem().toString().trim()
                    .equals(Resources.getText(DCLabels.lblModuleMovie))) {
            tabbedPane.removeAll();
            tabbedPane.addTab(Resources.getText(DCLabels.lblExistingValues), icoExisting, searchMoviePanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblNewValues), icoNew, insertMoviePanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabDataExchange), icoExchange, daMoviePanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), icoSQL, sqlPanel);
            tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), icoNote, notePanel);
            toolCatagory.setVisible(false);
            toolMP3Organizer.setVisible(false);
        }

        Settings.setValue(Repository.settings.stModule, new Integer(comboModule.getSelectedIndex()));
    }

    /**
     * Display requests and actions are passed to the different panels
     *
     * @param requestors    a collection of requests for specified panels
     * @param vData         data to be displayed (mainly used for tables)
     */
    public void displayResults(RequestorCollection requestors, Vector vData) {
        Requestor[] reqObjects = requestors.toRequestorArray();
        Requestor requestor = null;

        for (int i = 0; i < reqObjects.length; i++) {
            requestor = reqObjects[i];

            Class clazz = requestor.getClazz();
            String sMethod = requestor.getMethod();
            Object[] parameters = requestor.getParameters();

            if (clazz.getName().equals(TablePanel.class.getName())) {
                if (sMethod.equals("closeRecord")) {
                    ((RecordForm) parameters[0]).setVisible(false);
                    ((RecordForm) parameters[0]).dispose();
                }
            }

            if (clazz.getName().equals(catagoryForm.searchCat.getClass().getName())) {
                if (sMethod.equals("removeRows")) {
                    catagoryForm.searchCat.removeRows((int[]) parameters[0]);
                    catagoryForm.refreshCatagoryCache();
                }
                
                if (sMethod.equals("setStatus")) {
                    catagoryForm.searchCat.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }  

                if (sMethod.equals("updateRow")) {
                    catagoryForm.searchCat.updateRow(parameters[0].toString(),
                                                    (Object[]) parameters[1]);
                    catagoryForm.refreshCatagoryCache();
                }

                if (sMethod.equals("fillTable")) {
                    catagoryForm.searchCat.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI),
                        Messages._STANDARD);
                }
                
            }

            if (clazz.getName().equals(catagoryForm.insertCat.getClass().getName())) {
                if (sMethod.equals("setStatus")) {
                    catagoryForm.insertCat.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }                
            }

            if (clazz.getName().equals(SearchSoftwarePanel.class.getName())) {
                if (sMethod.equals("fillTable")) {
                    searchSoftwarePanel.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI),
                        Messages._STANDARD);
                }
                
                if (sMethod.equals("transferData")) {
                    searchSoftwarePanel.updateRowAt(((Integer)  parameters[0]).intValue(),
                                                    (Object[]) parameters[1],
                                                    ((Boolean)  parameters[2]).booleanValue());
                }                

                if (sMethod.equals("removeRows")) {
                    searchSoftwarePanel.removeRows((int[]) parameters[0]);
                }

                if (sMethod.equals("updateRow")) {
                    searchSoftwarePanel.updateRow(parameters[0].toString(),
                                          (Object[]) parameters[1]);
                }
            }

            if (clazz.getName().equals(SoftwareCatagory.class.getName())) {
                if (sMethod.equals("setCatagories")) {
                    SoftwareCatagories.load();
                }
            }

            if (clazz.getName().equals(InsertMP3Panel.class.getName())) {
                if (sMethod.equals("setStatus")) {
                    insertMP3Panel.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }

                if (sMethod.equals("addRow")) {
                    insertMP3Panel.addRow(vData);
                }
            }

            if (clazz.getName().equals(SearchMP3Panel.class.getName())) {
                if (sMethod.equals("fillTable")) {
                    searchMP3Panel.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI),
                        Messages._STANDARD);
                }

                if (sMethod.equals("removeRows")) {
                    searchMP3Panel.removeRows((int[]) parameters[0]);
                }

                if (sMethod.equals("updateRow")) {
                    searchMP3Panel.updateRow(parameters[0].toString(),
                                            (Object[]) parameters[1]);
                }
            }

            if (clazz.getName().equals(InsertSoftwarePanel.class.getName())) {
                if (sMethod.equals("setStatus")) {
                    insertSoftwarePanel.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }

                if (sMethod.equals("addRow")) {
                    insertSoftwarePanel.addRow(vData);
                }
                
                if (sMethod.equals("transferData")) {
                    insertSoftwarePanel.updateRowAt(((Integer)  parameters[0]).intValue(),
                                                    (Object[]) parameters[1],
                                                    ((Boolean)  parameters[2]).booleanValue());
                }                 
            }

            if (clazz.getName().equals(RecordForm.class.getName())) {
                if (sMethod.equals("transferData")) {
                    RecordForm form = (RecordForm) parameters[0];
                    Object[] row = (Object[]) parameters[1];
                    boolean overwrite = ((Boolean) parameters[2]).booleanValue();
                    int[] destinationFields = (int[]) parameters[3];
                    form.setData(row, overwrite, destinationFields);
                }
            }

            if (clazz.getName().equals(InsertMoviePanel.class.getName())) {
                if (sMethod.equals("setStatus")) {
                    insertMoviePanel.setStatus(Resources.getText(DCMessages.msgDataSaved));
                }

                if (sMethod.equals("transferData")) {
                    insertMoviePanel.updateRowAt(((Integer)  parameters[0]).intValue(),
                                               (Object[]) parameters[1],
                                               ((Boolean)  parameters[2]).booleanValue());
                }
                
                if (sMethod.equals("addRow")) {
                    insertMoviePanel.addRow(vData);
                }
            }

            if (clazz.getName().equals(SearchMoviePanel.class.getName())) {
                if (sMethod.equals("fillTable")) {
                    searchMoviePanel.fillTable(vData, false);
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgResultPassedToUI),
                        Messages._STANDARD);
                }

                if (sMethod.equals("removeRows")) {
                    searchMoviePanel.removeRows((int[]) parameters[0]);
                }

                if (sMethod.equals("updateRow")) {
                    searchMoviePanel.updateRow(parameters[0].toString(),
                                          (Object[]) parameters[1]);
                }
                if (sMethod.equals("transferData")) {
                    searchMoviePanel.updateRowAt(((Integer)  parameters[0]).intValue(),
                                               (Object[]) parameters[1],
                                               ((Boolean)  parameters[2]).booleanValue());
                }
            }
        }
    }

    private class MP3OrganizerFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            OrganizeMp3UI organizeMP3UI = new OrganizeMp3UI();
            organizeMP3UI.setVisible(true);
        }
    }

    private class SettingsFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            settingsView.setVisible(true);

        }
    }

    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            catagoryForm.setVisible(true);
        }
    }

    private class LogFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;
        }
    }

   private class ModuleAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setModule(true);
        }
    }
}