/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.LongTextField;
import rjw.dc.console.components.ShortTextField;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.software.Software;
import rjw.dc.messages.Messages;
import rjw.dc.processes.AmazonUpdate;
import rjw.dc.processes.ImdbUpdate;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.dc.util.SwingUtil;
import rjw.flex.settings.Settings;


/**
 * Form used to represent one record in the database or to create
 * a new record
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class RecordForm extends JFrame {
    
    private DataCrowObject o;

    private JLabel[] cLabels;
    private JComponent[] cInputFields;
    private JScrollPane[] cScrollPanes;

    private JPanel panelInputLeft = new JPanel(false);
    private JPanel panelInputRight = new JPanel(false);
    private JPanel panelAction = new JPanel(false);

    private Color color = Settings.getValueAsColor(
                                   Repository.settings.stColor);

    private final Class clazz;
    private final boolean update;
    private final int row;
    
    

    public RecordForm(  DataCrowObject o, 
                        boolean imdb,
                        boolean amazon,
                        Class clazz, 
                        int row) {
                            
        SwingUtil.setRootFrame(this);
        this.setResizable(true);
        this.setIconImage(new ImageIcon("./images/RecordForm.gif").getImage());
        
        this.o = o;
        this.clazz = clazz;
        this.row = row;
        
        String id = (String) o.getValue(DataCrowObject._ID);
        this.update = id.length() >= 10;
        
        if (id.length() < 9) {
            setTitle(Resources.getText(DCLabels.lblNewRecord));
        } else {
            setTitle(Resources.getText(DCLabels.lblRecord, id));
        }
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stRecordFormViewSize, 
                                  getSize());
            }
        });            
        
        Color colorRequired = ComponentFactory.getRequiredColor();
        int[] requiredFields = o.getRequiredFields();
        
        int[] visibleFields = o.getVisibleFields();
        String[] labelText = o.getLabels();
        int[] maxFieldLength = o.getMaxFieldLengths();
        int[] componentType = o.getFieldTypes();
        
        cLabels = new JLabel[componentType.length];
        cInputFields = new JComponent[componentType.length];
        cScrollPanes = new JScrollPane[componentType.length];
        
        this.getContentPane().setBackground(color);
        this.setBackground(color);

        // Main panel
        GridBagConstraints c = new GridBagConstraints();
        this.getContentPane().setLayout(ComponentFactory.getLayoutManager());
        c.fill = GridBagConstraints.VERTICAL;

        // Input panel
        GridBagConstraints cInput = new GridBagConstraints();
        panelInputLeft.setLayout(ComponentFactory.getLayoutManager());
        panelInputRight.setLayout(ComponentFactory.getLayoutManager());
        cInput.fill = GridBagConstraints.HORIZONTAL;

        // Action Panel
        GridBagConstraints cAction = new GridBagConstraints();
        panelAction.setLayout(ComponentFactory.getLayoutManager());
        cAction.fill = GridBagConstraints.HORIZONTAL;

        int start = 0;
        cLabels[0] = ComponentFactory.getLabel(labelText[0]);
        ShortTextField textField = ComponentFactory.getShortTextField(100);
        textField.setBackground(ComponentFactory.getDisabledColor());
        textField.setEditable(false);

        cInputFields[0] = textField;

        for (int i = 1; i < componentType.length; i ++) {
            cLabels[i] = ComponentFactory.getLabel(labelText[i]);
            JComponent comp = ComponentFactory.getComponent(componentType[i], 
                                                            labelText[i],
                                                            maxFieldLength[i]);
            cInputFields[i]= comp;
        }

        // fill the input panel with the generated fields
        int y = 0;
        int x = 0;
        panelInputLeft.setBackground(color);
        panelInputRight.setBackground(color);
        boolean right = false;
        
        for (int j = 0 ; j < componentType.length; j++) {
            if (j == 12) {
               x = 2; 
               y = 0;
               right = true;
            }
            
            JComponent comp = cInputFields[j];
            cLabels[j].setBackground(color);
            
            if (!right) {
                panelInputLeft.add(cLabels[j], new GridBagConstraints(x, y, 1, 1, 1.0, 1.0
                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                               new Insets(5, 5, 5, 5), 0, 0));
            } else {
                panelInputRight.add(cLabels[j], new GridBagConstraints(x, y, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
            }

            int stretch = GridBagConstraints.HORIZONTAL;
            int factor = 1;
            
            if (componentType[j] == ComponentFactory._LONGTEXTFIELD) {
                stretch = GridBagConstraints.BOTH; 
                factor  = 5;
                
                LongTextField longText = (LongTextField) comp;
                longText.setMargin(new Insets(0, 0, 0, 0));
                JScrollPane pane = new JScrollPane(longText);
                pane.setHorizontalScrollBarPolicy(
                           ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                pane.setVerticalScrollBarPolicy(
                           ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
                comp = pane;
                cScrollPanes[j] = pane;
            }

            if (!right) {        
                panelInputLeft.add(comp, new GridBagConstraints(x + 1, y, 1, 1, factor, factor
                              ,GridBagConstraints.NORTHWEST, stretch,
                               new Insets(5, 5, 5, 5), 0, 0));
            } else {
                panelInputRight.add(comp, new GridBagConstraints(x + 1, y, 1, 1, factor, factor
                              ,GridBagConstraints.NORTHWEST, stretch,
                               new Insets(5, 5, 5, 5), 0, 0));
            }
            
            y++;
        }

        int[] required = o.getRequiredFields(); 
        for (int i = 0; i < o.getRequiredFields().length; i++) {
            cLabels[required[i]].setForeground(colorRequired);
        }

        // Fill the actions panel
        JButton buttonClose = ComponentFactory.getButton(
                              Resources.getText(DCLabels.lblClose));
        JButton buttonSave = ComponentFactory.getButton(
                             Resources.getText(DCLabels.lblSave));
        JButton buttonDelete = ComponentFactory.getButton(
                             Resources.getText(DCLabels.lblDelete));
        JButton buttonInternet= null;

        buttonSave.setMnemonic(KeyEvent.VK_S);
        buttonClose.setMnemonic(KeyEvent.VK_C);
        buttonDelete.setMnemonic(KeyEvent.VK_D);

        buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSave));
        buttonDelete.setToolTipText(Resources.getText(DCTooltips.tpDelete));
        buttonClose.setToolTipText(Resources.getText(DCTooltips.tpClose));

        buttonDelete.addActionListener(new DeleteAction());
        buttonClose.addActionListener(new ExitAction());
        buttonSave.addActionListener(new SaveAction());
        
        if (imdb) {
            buttonInternet = ComponentFactory.getButton(
                             Resources.getText(DCLabels.lblIMDBUpdate));
            buttonInternet.addActionListener(new IMDBAction());
            buttonInternet.setToolTipText(Resources.getText(DCTooltips.tpIMDBUpdate));
            buttonInternet.setMnemonic(KeyEvent.VK_U);

        } else if (amazon) {
            buttonInternet = ComponentFactory.getButton(
                             Resources.getText(DCLabels.lblAmazonUpdate));
            buttonInternet.addActionListener(new AmazonAction());
            buttonInternet.setToolTipText(Resources.getText(DCTooltips.tpAmazonUpdate));
            buttonInternet.setMnemonic(KeyEvent.VK_U);
            
        }
        
        panelAction.setBackground(color);
        panelAction.add(buttonSave,         new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 5, 5, 5), 0, 0));
                        
        if (amazon || imdb) {
            panelAction.add(buttonInternet,   new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                            new Insets(5, 5, 5, 5), 0, 0));
        }
        
        if (update) {
            panelAction.add(buttonDelete,       new GridBagConstraints(2, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                            new Insets(5, 5, 5, 5), 0, 0));
        }
        
        panelAction.add(buttonClose ,       new GridBagConstraints(3, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 5, 5, 5), 0, 0));

        this.getContentPane().add(panelInputLeft, new GridBagConstraints(0, 0, 1, 1, 20.0, 20.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                       new Insets(5, 5, 5, 5), 0, 0));
        int width = 1;
        if (right) {
            this.getContentPane().add(panelInputRight, new GridBagConstraints(1, 0, 1, 1, 20.0, 20.0
                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                           new Insets(5, 5, 5, 5), 0, 0));
            width = 2;
        }

        
        this.getContentPane().add(panelAction, new GridBagConstraints(0, 1, width, 1, 0.0, 0.0
                      ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                       new Insets(5, 5, 5, 5), 0, 0));

        hideFields(o.getHiddenFields());
        pack();
        setSize(Settings.getValueAsDimension(Repository.settings.stRecordFormViewSize));
        
        setData(o.getValues(), true, o.getFieldIndices());
        
        setVisible(true);
    }
    
    private void hideFields(int[] indices) {
        for (int i = 0; i < indices.length; i++) {
            cLabels[indices[i]].setVisible(false);
            cInputFields[indices[i]].setVisible(false);
            
            if (cScrollPanes[indices[i]] != null) {
                cScrollPanes[indices[i]].setVisible(false);
            }
        }
    }

    /**
     * Applies data to the record form
     *
     * @param values    values to be displayed. Values is based on all fields, both
     *                  hidden and visible
     * @param ID        the ID of the record from the database
     */
    public void setData(Object[] values, boolean overwrite, int[] destinationFields) {
        int counter = 0;
        for (int i = 0; i < destinationFields.length; i++) {
            ComponentFactory.setComponentValue(
                        o.getFieldType(destinationFields[i]),
                        cInputFields[destinationFields[i]],
                        values[i]);
        }
    }    

    /**
     * Returns the data from screen
     *
     * @return (Object[])   the data as array
     */
    private Object[] getData() {
        Object[] values = o.getValues();
        for (int i = 0; i < cInputFields.length; i++) {
            Object value = ComponentFactory.getComponentValue(
                            o.getFieldType(i), 
                            cInputFields[i]);
            if (value instanceof String) {
                value = convertValue(value.toString());   
            }
            values[i] = value;
        }
        
        if (!update) {
            values[DataCrowObject._ID] = "";
        }
        
        return values;
    }

    public void startAmazonUpdate() {
        Object[] data = this.getData();
        Object oName = data[Software._A_NAME];
        Object oID = null;
        
        if (oName != null) {
            Object[][] o = {{oID, oName}};
            Messages.setMessage(Resources.getText(DCMessages.msgIMDBUpdateStarts), 
                                Messages._STANDARD);
            Thread threadAmazon = new Thread(new AmazonUpdate(o, this));
            threadAmazon.setDaemon(true);
            threadAmazon.setPriority(1);
            threadAmazon.start();            
        } else {
            new MessageBox(Resources.getText(DCMessages.msgIMDBNeedsTitle),
                           true, MessageBox._INFORMATION);
            return;   
        } 
    }

    public void startIMDBUpdate() {
        Object[] data = this.getData();
        Object oTitle = data[Movie._A_TITLE];
        Object oID = null;
        
        if (oTitle != null) {
            Object[][] o = {{oID, oTitle}};
            Messages.setMessage(Resources.getText(DCMessages.msgIMDBUpdateStarts), 
                                Messages._STANDARD);
            Thread threadIMDB = new Thread(new ImdbUpdate(o, this));
            threadIMDB.setDaemon(true);
            threadIMDB.setPriority(1);
            threadIMDB.start();            
        } else {
            new MessageBox(Resources.getText(DCMessages.msgIMDBNeedsTitle),
                           true, MessageBox._INFORMATION);
            return;   
        } 
    }


    /**
     * Transforms the value to match the database needs
     *
     * @param sValue    value as String
     */
    private String convertValue(String value) {
        String sValue = value;
        if (sValue == null) {
            return "Null";
        } else if (sValue.trim().equals("")) {
            return "Null";
        } else {
            return sValue.trim();
        }
    }
    
    protected void deleteValues() {
        GeneralDialog dialog = new GeneralDialog(Resources.getText(DCMessages.msgDeleteQuestion));
        
        if (((Boolean) GeneralDialog.result).booleanValue()) {
            String id = o.getValue(DataCrowObject._ID).toString();
            o.clearValues();
            o.setValue(DataCrowObject._ID, id);
            
            Object[] params = {this};
            Requestor requestClose = new Requestor(TablePanel.class, "closeRecord", params);
            RequestorCollection requestors = new RequestorCollection();
            requestors.addRequestor(requestClose);
    
            Requestor requestor = new Requestor(clazz, 
                                                "removeRows", 
                                                new Object[] {(new int[] {row})});
            requestors.addRequestor(requestor);
            o.delete(requestors);
        }
    }

    protected void saveValues() {
        String id = o.getValue(DataCrowObject._ID).toString();
        
        Object[] params = {this};
        Requestor requestClose = new Requestor(TablePanel.class, "closeRecord", params);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestClose);

        Object[] data = getData();
        
        for (int i = 0; i < data.length; i++) {
            if (data[i] == null || data[i].toString().trim().equals("")) {
                data[i] = "Null";
            }
        }

        o.setValues(data);
        if (!update)  {
            o.setValue(DataCrowObject._ID, "");
            o.saveNew(requestors);
        } else {
            Requestor requestor = new Requestor(clazz, 
                                                "updateRow", 
                                                new Object[] {id, getData()});
            requestors.addRequestor(requestor);
            o.saveUpdate(requestors);
        }
    }

    /**
     * Closes this form by setting the visibility to false
     */
    public void close() {
        Settings.setValue(Repository.settings.stRecordFormViewSize, 
                          getSize());        
        setVisible(false);
    }

    private class IMDBAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate();
        }
    }

    private class AmazonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveValues();
        }
    }

    private class DeleteAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            deleteValues();
        }
    }

    private class ExitAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
