/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.software.SoftwareCatagoryForm;
import rjw.dc.core.objects.DataCrowCollection;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;

/**
 * 
 * Base for all insert panels (for objects like Software, Mp3 and Movie)
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class TableInsertPanel extends TablePanel {

    /**
     * Initializes the panel. Should be called by the child by calling
     * super(module). The popup menu and the action panel are automatically
     * created. 
     * 
     * @param module the module indentifier
     */
    public TableInsertPanel (int module) {
        super();        
        this.module = module;

        DataCrowObject o = WorkFlow.getObject(module); 
        boolean imdb = o.isImdbUpdateAllowed();
        boolean amazon = o.isAmazonUpdateAllowed();
        boolean open = this instanceof SoftwareCatagoryForm.CatagoryInsertPanel ?
                       false : true;

        panelTableActions = new TableActionsPanel(true);
        panelTableActions.addSaveActionListener(new SaveNewValuesAction());
            
        popupMenu = new PopupMenu(imdb, amazon, false, true, open, false);
        popupMenu.addAmazonActionListener(new AmazonUpdateAction());
        popupMenu.addIMDBActionListener(new ImdbUpdateAction());
        popupMenu.addSaveActionListener(new SaveSelectedValuesAction());
        
        buildPanel();
        
        for (Enumeration enum = table.getColumnModel().getColumns();
             enum.hasMoreElements(); ) {
            
            vColumns.addElement(enum.nextElement());
        }
    }

    /**
     * Saves the selected values only
     */
    public void saveSelectedValues() {
        cancelEdit();
        int[] rows = table.getSelectedRows();
        DataCrowCollection collection = new DataCrowCollection();
        if (rows != null && rows.length > 0) {
            DataCrowObject o = WorkFlow.getObject(module);
            for (int i = 0; i < rows.length; i++) {
                Object[] row = getValuesAtRow(rows[i], true);
                o.setValues(row);
                o.setValue(DataCrowObject._ID, "");
                o.setUIRowNumber(rows[i]);
                collection.addObject(o, new Integer(rows[i]));
            }
        }
        
        RequestorCollection requestors = new RequestorCollection();
        requestors = addUpdateSoftCatRequest(requestors);

        collection.saveNew(requestors);
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));        
    }

    /**
     * Saves all the new values
     */
    public void saveNewValues() {
        cancelEdit();
        DataCrowCollection collection = new DataCrowCollection();
        DataCrowObject oTemp = WorkFlow.getObject(module);
        
        for (int i = 0; i < table.getRowCount(); i++) {
            Object[] row = getValuesAtRow(i, true);
            DataCrowObject o = WorkFlow.getObject(module);
            o.setValues(row);
            o.setUIRowNumber(i);
            o.setValue(DataCrowObject._ID, "");
            collection.addObject(o, new Integer(i));
        }
        
        Requestor requestor = new Requestor(getClass(), "setStatus", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);      
        
        requestors = addUpdateSoftCatRequest(requestors);
        collection.saveNew(requestors);

        vChangedIDs.removeAllElements();
        
        setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
    }

    protected int[] getRecordHeaderFields() {
        return null;   
    }
    
    /**
     * Creates the panel
     */
    private void buildPanel() {
        JPanel panelInput  = new JPanel(false);
        JPanel panelStatus = new JPanel(false);

        // ************************************************************************
        // INPUT PANEL
        // Set layout style
        GridBagLayout gridbInput = new GridBagLayout();
        panelInput.setLayout(gridbInput);

        // Initialize components
        table = ComponentFactory.getTableForObject(WorkFlow.getObject(module), tableModel);
        table.addMouseListener(new TableMouseListener());
        table.addKeyListener(new TableKeyListener());

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelInput.add( scrollResults,      new GridBagConstraints( 0, 0, 7, 1, 20.0, 20.0
                       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(    labelStatus,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag     = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelInput.setBorder(ComponentFactory.getBorder(
                             Resources.getText(DCLabels.lblInsert)));
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));

        // Fill the main panel
        add(panelInput,     new GridBagConstraints( 0, 0, 8, 1, 20.0, 20.0
                           ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelTableActions,new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelStatus,    new GridBagConstraints( 0, 2, 8, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }
    
    
    private class SaveSelectedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            saveSelectedValues();
        }
    }  
    
    private class SaveNewValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNewValues();
        }
    }    
    
    private class AmazonUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate(false);
        }
    }

    private class ImdbUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate(false);
        }
    }      
}
