/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.software.SoftwareCatagoryForm;
import rjw.dc.core.DataCrow;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.software.SoftwareCatagory;
import rjw.dc.export.TableToFileExporter;
import rjw.dc.messages.Messages;
import rjw.dc.processes.AmazonUpdate;
import rjw.dc.processes.ImdbUpdate;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.dc.util.StringUtils;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Base panel for all panels which contain a JTable
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */ 
public abstract class TablePanel extends DataCrowPanel {

    protected DefaultTableModel tableModel = new DefaultTableModel();
    protected JTable table;
    protected TableActionsPanel panelTableActions;
    protected PopupMenu popupMenu;

    protected Vector vChangedIDs = new Vector();
    
    /** Vector of columns which are not part of the view */
    protected Vector vHiddenColumns = new Vector();
    protected Vector vColumns = new Vector();

    protected int module = -1;

    public TablePanel() {
        super();  
    }
    
    public void cancelEdit() {
        int selectedRow = table.getSelectedRow();
        int selectedColumn = table.getSelectedColumn();
        
        if (selectedRow != -1 && selectedColumn != -1) {
            boolean success = table.getCellEditor(selectedRow, selectedColumn).
                              stopCellEditing();
            if (!success) {
                table.getCellEditor(selectedRow, selectedColumn).cancelCellEditing();
            }
        }
    }
    
    private void setHiddenFieldsSetting(int[] modelIndices) {
        String sKey = null;
        switch (module) {
            case Repository.module._SOFTWARE:
                sKey = Repository.settings.stHiddenFieldsSoftware;
                break;
            case Repository.module._MOVIE:
                sKey = Repository.settings.stHiddenFieldsMovie;
                break;
            case Repository.module._MP3:
                sKey = Repository.settings.stHiddenFieldsMP3;
                break;
            case Repository.module._SOFTWARECAT:
                sKey = Repository.settings.stHiddenFieldsSoftwareCat;
                break;
        }
        
        if (sKey == null) {
            return;   
        } else {
            Settings.setValue(sKey, modelIndices);
        }
    }
    
    /**
     * Opens the fields selector dialog and applies the hidden fields on the
     * table. Also the hidden fields setting is set for the current module.
     */
    public void startFieldSelector() {
        Vector vVisibleColumns = new Vector();
        for (Enumeration enum = table.getColumnModel().getColumns(); enum.hasMoreElements(); ) {
            vVisibleColumns.addElement((TableColumn) enum.nextElement());
        }
             
        FieldSelectorDialog dialog = new FieldSelectorDialog(vVisibleColumns, 
                                                             vHiddenColumns);
        dialog.setVisible(true);
        if (FieldSelectorDialog.isSuccessful()) {
            vVisibleColumns = dialog.getVisibleFields(); 
            vHiddenColumns = dialog.getHiddenFields();

            // get the hidden columns table model indices by comparing the
            // columns of the table model with the table              
            int[] modelIndices = new int[vHiddenColumns.size()];
            int counter = 0;
            for (Enumeration enum = vHiddenColumns.elements(); enum.hasMoreElements(); ) {
                TableColumn column = (TableColumn) enum.nextElement();
                modelIndices[counter++] = column.getModelIndex();
            }            
            
            setHiddenFieldsSetting(modelIndices);
            //removeHiddenFields(vHiddenColumns);
            MainFrame.setHiddenFields();
        }
    }
    
    private void resetTable() {
        try {
            for (Enumeration enum = vColumns.elements(); enum.hasMoreElements(); ) {
                TableColumn column = (TableColumn) enum.nextElement();
                
                try {
                    table.getColumn(column.getIdentifier());
                } catch (Exception exp) {
                    table.addColumn(column);
                }
            }
        } catch (Exception ignore) {}
    }
    
    /**
     * Removes the hidden fields from the view.
     * 
     * @param hiddenFields The hidden fields to be removed
     */
    private void removeHiddenFields(Vector hiddenFields) {
        try {
            for (Enumeration enum = hiddenFields.elements(); enum.hasMoreElements(); ) {
                TableColumn column = (TableColumn) enum.nextElement();
                table.removeColumn(column);
            }
        } catch (Exception ignore) {}
        setRequiredFields(WorkFlow.getObject(module));
    }
    
    /**
     * External use. All columns to be removed are still visible
     * in the table. The columns are retrieved, stored in a Vector 
     * and then removed from the view
     * 
     * @param modelIndices The model indices of the columns
     */
    public void removeHiddenFields(int[] hiddenColumns) {
        cancelEdit();
        resetTable();
        
        Vector vCache = vHiddenColumns;
        vHiddenColumns = new Vector(0);
        
        for (int i = 0; i < hiddenColumns.length; i++) {
            int tableIndex = table.convertColumnIndexToView(hiddenColumns[i]);
            TableColumn column = table.getColumnModel().getColumn(tableIndex);
            vHiddenColumns.addElement(column); 
        }
        removeHiddenFields(vHiddenColumns);   
    }

    protected abstract int[] getRecordHeaderFields();   
    
    /**
     * Fills the table with the specified values
     *
     * @param vTable    vector containing the rows of data to be displayed
     */
    public void fillTable(Vector vTable, boolean displayID) {
        clearTable();
        try {
            DataCrowObject o = null;
            for (Enumeration enum = vTable.elements(); enum.hasMoreElements(); ) {
                o = (DataCrowObject) enum.nextElement();
                tableModel.addRow(o.getValues());
            }
            String sRecords = new Integer(vTable.size()).toString();
            setStatus(Resources.getText(DCMessages.msgSearchResult, sRecords));
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Returns the row number of the record with the given ID
     * (product object ID; ie. the value of Software._ID)
     *
     * @param ID
     */
    public int getRowNumberWithID(String ID) {
        cancelEdit();
        for (int i = 0; i < tableModel.getRowCount(); i++) {
            String sID = tableModel.getValueAt(i, 0).toString();
            if (sID.equals(ID)) {
                return i;
            }
        }
        return -1;
    }

    public RequestorCollection addUpdateSoftCatRequest(RequestorCollection requestors) {
        if (this instanceof SoftwareCatagoryForm.CatagorySearchPanel ||
            this instanceof SoftwareCatagoryForm.CatagoryInsertPanel) {
            Requestor requestor = new Requestor(SoftwareCatagory.class,
                                                "setCatagories",
                                                null);
            requestors.addRequestor(requestor);;
        }
        return requestors;
    }

    /**
     * Updates the row with the given ID with the data (values)
     *
     * @param ID the ID (Product Object ID)
     * @param values the new values
     */
    public void updateRow(String ID, Object[] values) {
        cancelEdit();

        int row = getRowNumberWithID(ID);
        if (row != -1) {
            for (int j = 0; j < values.length; j++) {
                tableModel.setValueAt(values[j], row, j);
            }
        }
    }

    public Object[] getValuesAtRow(int rowNumber, boolean hidden) {
        cancelEdit();
        
        Object[] row = null;
        if (hidden) {
            row = new Object[tableModel.getColumnCount()];
        } else {
            row = new Object[table.getColumnCount()];
        }
        for (int j = 0; j < row.length; j++) {
            Object oTest = null;
            if (hidden) {
                oTest = tableModel.getValueAt(rowNumber, j);
            } else {
                oTest = table.getValueAt(rowNumber, j);    
            }
            if (oTest != null) {
                row[j] = oTest.toString();
                if (row[j].toString().trim().equals("")) {
                    row[j] = Repository.dataBase.dBEmptyValue;
                }
            } else {
                row[j] = Repository.dataBase.dBEmptyValue;
            }                
        }
        return row;
    }

    /**
     * Adds the datacrow objects to the table
     *
     * @param vDataCrowObjects  vector of datacrow objects
     */
    public void addRow(Vector vDataCrowObjects) {
        for (Enumeration enum = vDataCrowObjects.elements(); enum.hasMoreElements(); ) {
            DataCrowObject lcObj = (DataCrowObject) enum.nextElement();
            Object[] row = lcObj.getValues();
            row[0] = String.valueOf(tableModel.getRowCount() + 1);
            tableModel.addRow(row);
        }
    }

    /**
     * Clear the table of all values
     */
    public void clearTable() {
        cancelEdit();
        tableModel.removeTableModelListener(new TableValueChangedAction());
        tableModel.setRowCount(0);
        tableModel.addTableModelListener(new TableValueChangedAction());
        setStatus(Resources.getText(DCMessages.msgTableCleared));
    }

    /**
     * Updates a row at the given rownumber. Skips the ID number
     * and thus starts with the update in column 1 instead of 0
     */
    public void updateRowAt(int rowNumber,
                            Object[] rowData,
                            boolean overWrite) {
        cancelEdit();

        for (int i = 0; i < rowData.length; i++) {
            Object oNew = rowData[i];
            Object oExist = tableModel.getValueAt(rowNumber, i + 1);

            if (!overWrite && (oExist != null &&
                               !oExist.toString().trim().equals(""))) {
                oNew = oExist;
            }

            if (oNew == null || oNew.toString().trim().equals("")) {
                oNew = oExist;
            }

            tableModel.setValueAt(oNew, rowNumber, i + 1);
        }
    }

    private Object[][] getDataForInternetUpdate(int[] rows) {
        cancelEdit();
        Vector vNames = new Vector(0);
        Vector vIDs = new Vector(0);

        if (rows == null || rows.length < 1) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowUpdate),
                           true, MessageBox._INFORMATION);
            return null;
        }

        for (int i = 0; i < rows.length; i++) {
            Object oName = tableModel.getValueAt(rows[i], 1);
            oName = oName == null ? "" : oName;
            vNames.add(oName);
            vIDs.add(new Integer(rows[i]));
        }

        Object[][] o = new Object[vNames.size()][2];
        for (int j = 0; j < o.length; j++) {
            o[j][0] = vIDs.elementAt(j);
            o[j][1] = vNames.elementAt(j);
        }

        return o;
    }
    
    private void exportTableToText() {
        cancelEdit();
        TableToFileExporter.exportTableToText(this, table);   
    }
    
    private void exportTableToPDF() {
        cancelEdit();
        TableToFileExporter.exportTableToPDF(this, table, getRecordHeaderFields());
    }

    private void exportTableToHTML() {
        cancelEdit();
        TableToFileExporter.exportTableToHTML(this, table, getRecordHeaderFields());
    }

    public void startAmazonUpdate(boolean update) {
        cancelEdit();
        Messages.setMessage(Resources.getText(DCMessages.msgAmazonUpdateStarts),
                            Messages._STANDARD);

        Object[][] o = getDataForInternetUpdate(table.getSelectedRows());

        Thread threadAmazon = new Thread(new AmazonUpdate(o, this));
        threadAmazon.setDaemon(true);
        threadAmazon.setPriority(1);
        threadAmazon.start();
    }

    public void startIMDBUpdate(boolean update) {
        cancelEdit();
        Messages.setMessage(Resources.getText(DCMessages.msgIMDBUpdateStarts),
                            Messages._STANDARD);

        Object[][] o = getDataForInternetUpdate(table.getSelectedRows());

        Thread threadIMDB = new Thread(new ImdbUpdate(o, this));
        threadIMDB.setDaemon(true);
        threadIMDB.setPriority(1);
        threadIMDB.start();
    }

    public void openRecord() {
        cancelEdit();
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowToOpen),
                           true, MessageBox._INFORMATION);
            return;
        }

        Object[] row = new Object[tableModel.getColumnCount()];
        for (int i = 0; i < row.length; i++) {
            Object oSelected = tableModel.getValueAt(selectedRow, i);
            if (oSelected == null) {
                row[i] = "";
            } else {
                row[i] = oSelected;
            }
        }

        DataCrowObject o = WorkFlow.getObject(module);
        boolean imdb = o.isImdbUpdateAllowed();
        boolean amazon = o.isAmazonUpdateAllowed();
        o.setValues(row);

        RecordForm recordForm = new RecordForm( o,
                                                imdb,
                                                amazon,
                                                getClass(),
                                                selectedRow);
    }

    public void removeRows(int[] rows) {
        cancelEdit();
        for (int i = rows.length -1; i > -1; i--) {
            tableModel.removeRow(rows[i]);
        }
    }

    public void setRequiredFields(DataCrowObject o) {
        int counter = 0; 
        for (Enumeration enum = vColumns.elements(); enum.hasMoreElements(); ) {
            TableColumn column = (TableColumn) enum.nextElement();
            if (o.isRequired(counter)) {
                column.setHeaderRenderer(
                    ComponentFactory.getRequiredTableHeaderRenderer());
            } else {
                column.setHeaderRenderer(
                    ComponentFactory.getTableHeaderRenderer());
            }
            counter++;
        }


/**        for (int j = 0; j < table.getColumnCount(); j++) {
            TableColumn column = table.getColumnModel().getColumn(j);
            boolean required = false;
            for (int i = 0; i < requiredFields.length; i++) {
                if (j == requiredFields[i]) {
                    required = true;
                }
            }

            if (required) {
                column.setHeaderRenderer(
                    ComponentFactory.getRequiredTableHeaderRenderer());
            } else if (!required) {
                column.setHeaderRenderer(
                    ComponentFactory.getTableHeaderRenderer());
            }
        } */
        this.repaint();
    }

    public class PopupMenu extends JPopupMenu {

        JMenuItem menuOpen;
        JMenuItem menuSave;
        JMenuItem menuDelete;
        JMenuItem menuUpdateAll;
        JMenuItem menuIMDB;
        JMenuItem menuAmazon;
        JMenuItem menuRemoveRow;
        JMenuItem menuAddRow;
        JMenuItem menuExportToPDF;
        JMenuItem menuExportToTxt;
        JMenuItem menuExportToHtml;
        JMenuItem menuFieldSelector;

        public PopupMenu(boolean showImdb,
                        boolean showAmazon,
                        boolean showDelete,
                        boolean showAddRemove,
                        boolean showOpen,
                        boolean showExport) {

            menuOpen = new JMenuItem(Resources.getText(DCLabels.lblOpenRecord));
            menuSave = new JMenuItem(Resources.getText(DCLabels.lblSave));
            menuDelete = new JMenuItem(Resources.getText(DCLabels.lblDelete));
            menuIMDB = new JMenuItem(Resources.getText(DCLabels.lblIMDBUpdate));
            menuAmazon = new JMenuItem(Resources.getText(DCLabels.lblAmazonUpdate));
            menuRemoveRow = new JMenuItem(Resources.getText(DCLabels.lblRemoveRow));
            menuAddRow = new JMenuItem(Resources.getText(DCLabels.lblAddRow));
            menuExportToPDF = new JMenuItem(Resources.getText(DCLabels.lblExportToPDF));
            menuExportToTxt = new JMenuItem(Resources.getText(DCLabels.lblExportToTxt));
            menuExportToHtml = new JMenuItem(Resources.getText(DCLabels.lblExportToHmtl));
            menuFieldSelector = new JMenuItem(Resources.getText(DCLabels.lblShowFieldSelector));
            menuUpdateAll = new JMenuItem(Resources.getText(DCLabels.lblUpdateAll));

            menuFieldSelector.addActionListener(new FieldSelectorAction());
            menuOpen.addActionListener(new OpenRecordAction());
            menuRemoveRow.addActionListener(new RemoveRowAction());
            menuAddRow.addActionListener(new AddRowAction());
            menuExportToPDF.addActionListener(new ExportToPDFAction());
            menuExportToTxt.addActionListener(new ExportToTextAction());
            menuExportToHtml.addActionListener(new ExportToHTMLAction());
            menuUpdateAll.addActionListener(new UpdateAllAction());

            this.add(menuFieldSelector);
            this.add(menuUpdateAll);
            this.add(new JSeparator(JSeparator.HORIZONTAL));

            if (showOpen) {
                this.add(menuOpen);
            }
            
            this.add(menuSave);
            if (showDelete) {
                this.add(menuDelete);
            }
            if (showAddRemove) {
                this.add(menuRemoveRow);
                this.add(menuAddRow);
            }
            
            if (showImdb) {
                this.add(new JSeparator(JSeparator.HORIZONTAL));
                this.add(menuIMDB);
            }

            if (showAmazon) {
                this.add(new JSeparator(JSeparator.HORIZONTAL));
                this.add(menuAmazon);
            }
            
            if (showExport) {
                this.add(new JSeparator(JSeparator.HORIZONTAL));
                this.add(menuExportToPDF);
                this.add(menuExportToTxt);
                this.add(menuExportToHtml);
            }
        }

        public void addAmazonActionListener(ActionListener listener) {
            menuAmazon.addActionListener(listener);
        }

        public void addDeleteActionListener(ActionListener listener) {
            menuDelete.addActionListener(listener);
        }

        public void addIMDBActionListener(ActionListener listener) {
            menuIMDB.addActionListener(listener);
        }

        public void addSaveActionListener(ActionListener listener) {
            menuSave.addActionListener(listener);
        }
    }


    public class TableActionsPanel extends JPanel {
        private JButton buttonClear;
        private JButton buttonSave;
        private JButton buttonAddRow;
        private JButton buttonRemoveRow;

        public TableActionsPanel(boolean showAddRemoveRow) {
            setLayout(new GridBagLayout());

            // Create the components
            buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSaveChanges));
            buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
            buttonAddRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow));
            buttonRemoveRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblRemoveRow));

            buttonSave.setMnemonic(KeyEvent.VK_S);
            buttonClear.setMnemonic(KeyEvent.VK_C);
            buttonAddRow.setMnemonic(KeyEvent.VK_A);
            buttonRemoveRow.setMnemonic(KeyEvent.VK_R);

            buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSaveChanges));
            buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));
            buttonAddRow.setToolTipText(Resources.getText(DCTooltips.tpAddRow));
            buttonRemoveRow.setToolTipText(Resources.getText(DCTooltips.tpRemoveRow));

            buttonClear.addActionListener(new ClearButtonAction());
            buttonAddRow.addActionListener(new AddRowAction());
            buttonRemoveRow.addActionListener(new RemoveRowAction());

            // Build the panel
            if (showAddRemoveRow) {
                add(buttonAddRow,   new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
                add(buttonRemoveRow,new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
            }

            add(buttonSave,     new GridBagConstraints( 2, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                new Insets(5, 5, 5, 5), 0, 0));

            add(buttonClear,    new GridBagConstraints( 3, 0, 1, 1, 1.0, 1.0
                               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                                new Insets(5, 5, 5, 5), 0, 0));
        }

        public void addSaveActionListener(ActionListener listener) {
            buttonSave.addActionListener(listener);
        }

        public void removeSaveActionListener(ActionListener listener) {
            buttonSave.removeActionListener(listener);
        }
    }

    /**
     * Clears the table
     */
    public class OpenRecordAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            openRecord();
        }
    }

    /**
     * Clears the table
     */
    public class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearTable();
        }
    }

    /**
     * Handles changes made to the table (on screen)
     */
    public class TableValueChangedAction implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            if (e.getType() == TableModelEvent.INSERT || e.getType() == TableModelEvent.UPDATE) {
                if (table.getSelectedRow() != -1) {
                    if (tableModel.getValueAt(table.getSelectedRow(),0) != null) {
                        String sID = tableModel.getValueAt(table.getSelectedRow(), 0).toString();
                        if (!vChangedIDs.contains(sID)) {
                            vChangedIDs.addElement(sID);
                        }
                    }
                }
            }
        }
    }
    
    public class ExportToHTMLAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            exportTableToHTML();
        }        
    }    
    
    public class ExportToPDFAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            exportTableToPDF();
        }        
    }

    public class FieldSelectorAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startFieldSelector();
        }        
    } 
    
    public class ExportToTextAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            exportTableToText();
        }        
    }    

    public class UpdateAllAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            UpdateAllDialog dialog =  new UpdateAllDialog(WorkFlow.getObject(module),
                                                          DataCrow.mainFrame);
            try {
                if (dialog.isSuccess()) {
                    dialog.setModal(false);
                    Object[] values = dialog.getData();
                    
                    int rows = tableModel.getRowCount();
                    for (int i = 0; i < rows; i++) {
                        for (int j = 0; j < values.length; j++) {
                            if (values[j] != null &&
                                !StringUtils.isEmpty(values[j].toString()) &&
                                !values[j].toString().equals("-1")) {
                                    
                                tableModel.setValueAt(values[j], i, j);
                                vChangedIDs.addElement(tableModel.getValueAt(i, 0));
                            }
                        } 
                    }
                }
            } catch (Exception exp) {
                Messages.setMessage(exp.toString(), Messages._ERROR);
            } finally {
                dialog.dispose();
                dialog = null;
            }
        }        
    }    

    public class AddRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Vector v = new Vector();
            v.add(String.valueOf(tableModel.getRowCount() + 1));
            tableModel.addRow(v);
        }
    }

    public class RemoveRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (tableModel.getRowCount() < 1 ) {
                return;
            }
            int[] rows = table.getSelectedRows();
            if (rows.length == 0) {
                tableModel.removeRow(tableModel.getRowCount() -1);

            } else {
                for (int i = rows.length; i > 0; i--) {
                    tableModel.removeRow(rows[i-1]);
                }
            }
        }
    }

    public class TableKeyListener implements KeyListener {
        public void keyPressed(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_F12) {
                openRecord();
            }
        }

        public void keyReleased(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}
    }

    public class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (SwingUtilities.isRightMouseButton(e)) {
                if (table.getSelectedRows() != null &&
                    table.getSelectedRows().length < 2) {

                    int row = table.rowAtPoint(e.getPoint());
                    int col = table.columnAtPoint(e.getPoint());
                    table.changeSelection(row, col, false, false);
                }
                popupMenu.show(table, e.getX(), e.getY());
            }

            if (e.getClickCount() == 2) {
                openRecord();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }
}
