 /******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultMutableTreeNode;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.core.objects.software.SoftwareCatagories;
import rjw.dc.core.objects.software.SoftwareCatagory;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;

/**
 * This class is the Component holder for the UI. The look and feel, as well
 * as all the other preference are set here
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ComponentFactory {

    public static final int _SOFTWARETYPECOMBO = 0;
    public static final int _YESNOCOMBO = 1;
    public static final int _MP3GENRECOMBO = 2;
    public static final int _LONGTEXTFIELD = 3;
    public static final int _YEARCOMBO = 5;
    public static final int _CHECKBOX = 6;
    public static final int _NUMBERFIELD = 7;
    public static final int _SHORTTEXTFIELD = 8;
    public static final int _URLFIELD = 9;

    /**
     * Initializes this class and sets the look and feel for all UI components.
     * The look and feel is only set the first time
     */
    static {
        try {
            UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
        } catch (Exception e) {
            Messages.setMessage(e.fillInStackTrace().toString(), Messages._ERROR);
        }
    }

    public static JComponent getComponent(  int componentIndex,
                                            String sLabel,
                                            int maxTextLength) {
        switch (componentIndex) {
            case _SOFTWARETYPECOMBO:
                return getSoftwareCatagoriesComboBox();
            case _YESNOCOMBO:
                return getYesNoCombo();
            case _MP3GENRECOMBO:
                return getMP3GenreComboBox();
            case _LONGTEXTFIELD:
                return getLongTextField(maxTextLength);
            case _YEARCOMBO:
                return getYearComboBox();
            case _CHECKBOX:
                return getCheckBox(sLabel);
            case _NUMBERFIELD:
                return getNumberField();
            case _URLFIELD:
                return getURLField(maxTextLength);
            default:
                return getShortTextField(maxTextLength);
        }
    }

    public static JList getList(Vector vData) {
        JList list = new JList(vData);
        return list;
    }

    public static URLField getURLField(int maxLength) {
        URLField urlField = new URLField(maxLength);
        return urlField;
    }

    public static JColorChooser getColorChooser() {
        JColorChooser colorChooser = new JColorChooser();
        return colorChooser;
    }

    public static JTree getTree(DefaultMutableTreeNode model) {
        JTree tree = new JTree(model);
        return tree;
    }

    public static GridBagLayout getLayoutManager() {
        GridBagLayout gridb = new GridBagLayout();
        return gridb;
    }

    public static NumberField getNumberField() {
        NumberField numberField = new NumberField();
        return numberField;
    }

    /**
     * Returns a combobox with the Software database field names
     *
     * @return (JComboBox)  combobox filled with the database field names.
     *                      The first item in the list is an empty String
     */
    public static JComboBox getFieldsComboBox(DataCrowObject o) {
        JComboBox fieldsComboBox = new JComboBox();
        fieldsComboBox.addItem(" ");
        
        String[] fields = o.getFieldNames();
        for (int i = 0; i < fields.length; i++) {
            fieldsComboBox.addItem(fields[i]);
        }
        fieldsComboBox.setFont(getStandardFont());

        return fieldsComboBox;
    }

    public static JComboBox getYearComboBox() {
        JComboBox yearComboBox = new JComboBox();
        yearComboBox.addItem("");
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int year = cal.get(Calendar.YEAR);

        for (int i = year; i > 1939; i--) {
            yearComboBox.addItem(String.valueOf(i));
        }

        return yearComboBox;
    }

    /**
     * Returns a combobox with MP3 genres
     *
     * @return (JComboBox)  combobox filled with MP3 genres. The first item is
     *                      an empty String
     */
    public static JComboBox getMP3GenreComboBox() {
        JComboBox genreComboBox = new JComboBox();
        genreComboBox.addItem("");
        genreComboBox.setSelectedIndex(0);
        genreComboBox.setFont(getStandardFont());

        for (int i = 0; i < Repository.collections.colMP3Genres.length; i++) {
            genreComboBox.addItem(Repository.collections.colMP3Genres[i]);
        }

        return genreComboBox;
    }

    /**
     * Returns a combobox with Sofware types (Catagories
     *
     * @return (JComboBox)  combobox filled with Software types. The first item
     *                      is an empty String.
     */
    public static JComboBox getSoftwareCatagoriesComboBox() {
        JComboBox typeComboBox = new JComboBox();
        typeComboBox.addItem("");
        typeComboBox = fillSoftwareCatagories(typeComboBox);
        typeComboBox.setSelectedIndex(0);
        typeComboBox.setFont(getStandardFont());
        
        Component[] comps = typeComboBox.getComponents();
        for (int i = 0; i < comps.length; i++) {
            if (comps[i] instanceof AbstractButton) {
                ((AbstractButton) comps[i]).addMouseListener(
                    new RefreshSoftwareCategoriesAction());    
            }   
        }

        return typeComboBox;
    }

    /**
     * Adds the software catagories to the Software type combobox. The Software
     * types are loaded from the CatagoryCollection class.
     */
    public static JComboBox fillSoftwareCatagories(JComboBox typeComboBox) {
        // initialize the catagory (first time init will init the Collection)
        Object oSelected = typeComboBox.getSelectedItem();
        SoftwareCatagory cat = new SoftwareCatagory();
        Vector vData = SoftwareCatagories.getCatagories();
        typeComboBox.removeAllItems();
        typeComboBox.addItem("");
        typeComboBox.setFont(getStandardFont());

        for (Enumeration enum = vData.elements(); enum.hasMoreElements(); ) {
            cat = (SoftwareCatagory) enum.nextElement();
            typeComboBox.addItem(cat.getValue(SoftwareCatagory._A_NAME));
        }

        try {
            typeComboBox.setSelectedItem(oSelected);
        } catch (Exception exp) {}

        return typeComboBox;
    }

    /**
     * Returns an empty combobox
     *
     * @param length        length of this field
     * @return (JComboBox)  an empty combobox
     */
    public static JComboBox getComboBox(int length) {
        JComboBox comboBox = getComboBox();
        comboBox.setPreferredSize(new Dimension(length, 23));
        return comboBox;
    }

    /**
     * Returns an empty combobox
     *
     * @param nameOfField   name of this field
     * @return (JComboBox)  an empty combobox
     */
    public static JComboBox getComboBox() {
        JComboBox comboBox = new JComboBox();
        comboBox.setFont(getStandardFont());
        return comboBox;
    }

    /**
     * Returns a combobox with the items "yes" and "no", not starting with
     * and empty String
     *
     * @return (JComboBox)  combobox with the items "yes" and "no"
     */
    public static JComboBox getRealYesNoComboBox() {
        JComboBox comboBox = new JComboBox();
        comboBox.addItem(Resources.getText(DCLabels.lblNo));
        comboBox.addItem(Resources.getText(DCLabels.lblYes));
        comboBox.setFont(getStandardFont());
        return comboBox;
    }

    /**
     * Returns a standard checkbox
     *
     * @param nameOfField   name of this field
     * @return (JCheckBox)  simple checbox, no prefenrences are set
     */
    public static JCheckBox getCheckBox(String labelText) {
        JCheckBox checkBox = new JCheckBox(labelText);
        checkBox.setFont(getStandardFont());

        return checkBox;
    }

    public static JButton getMenuButton(String imageFile, String buttonText) {
        Icon ico = new ImageIcon(imageFile);
        JButton button = new JButton(buttonText, ico);
        button.setFont(getStandardFont());
        return button;
    }

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @param length        length of this button
     * @return (JButton)    button, height 23
     */
    public static JButton getButton(String buttonText, int length) {
        JButton button = getButton(buttonText);
        button.setPreferredSize(new Dimension(length, 23));
        return button;
    }

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @param length        length of this button
     * @return (JButton)    button, height 23
     */
    public static JButton getButton(ImageIcon icon) {
        JButton button = new JButton(icon);
        return button;
    }

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @return (JButton)    button, height 23
     */
    public static JButton getButton(String buttonText) {
        JButton button = new JButton();
        button.setName("bt" + buttonText);
        button.setText(buttonText);
        button.setFont(getStandardFont());
        return button;
    }

    /**
     * Returns a long text field with all the defaults set
     */
    public static JTextArea getTextArea() {
        JTextArea textArea = new JTextArea();
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setEditable(true);
        textArea.setDoubleBuffered(true);
        textArea.setBackground(getTableColor());
        textArea.setMargin(new Insets(5,5,5,5));
        textArea.setFont(getStandardFont());

        return textArea;
    }

    /**
     * Returns a text area with all the defaults set
     */
    public static LongTextField getLongTextField(int maxTextLength) {
        LongTextField longText = new LongTextField(maxTextLength);
        longText.setWrapStyleWord(true);
        longText.setLineWrap(true);
        longText.setEditable(true);
        longText.setDoubleBuffered(true);
        longText.setBackground(getTableColor());
        longText.setMargin(new Insets(5,5,5,5));
        longText.setFont(getStandardFont());

        return longText;
    }

    /**
     * Returns a standard label with a text
     *
     * @param length        length of this field
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static  JLabel getLabel(String labelText, int length) {
        JLabel label = getLabel(labelText);
        label.setPreferredSize(new Dimension(length, 23));
        return label;
    }

    /**
     * Returns a standard label with a text
     *
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static JLabel getLabel(String labelText) {
        JLabel label = new JLabel();
        label.setText(labelText);
        label.setRequestFocusEnabled(false);
        label.setFont(getStandardFont());
        label.setToolTipText(labelText);

        return label;
    }

    /**
     * Returns a standard editor pane
     *
     * @return (JEditorPane)    editor pane, no preferences are set
     */
    public static JEditorPane getEditorPane() {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setFont(getStandardFont());
        editorPane.setBackground(getTableColor());

        return editorPane;
    }

    public static ShortTextField getShortTextField(int maxTextLength) {
        ShortTextField textField = new ShortTextField(maxTextLength);
        textField.setFont(getStandardFont());
        return textField;
    }

    /**
     * Returns a textfield
     *
     * @param length  length of this field
     */
    public static JTextField getTextField(int length) {
        JTextField textField = getTextField();
        return textField;
    }

    /**
     * Returns a textfield
     *
     * @return JTextField   textfield, height 23
     */
    public static JTextField getTextField() {
        JTextField textField = new JTextField();
        textField.setFont(getStandardFont());
        return textField;
    }

    /**
     * Returns a standard tabbed pane
     *
     * @return (JTabbedPane)    tabbed pane, no prefences are set
     */
    public static JTabbedPane getTabbedPane() {
        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.setFont(getStandardFont());

        return tabbedPane;
    }

    /**
     * Returns a standard icon
     *
     * @param iconLocation  location of the icon image
     * @return (Icon)       normal icon
     */
    public static Icon getIcon(String iconLocation) {
        Icon icon = new ImageIcon(iconLocation);
        return icon;
    }

    /**
     * Returns a standard menubar
     *
     * @param nameOfField   name of this field
     * @return (JMenuBar)   standard menubar, no prefences are set
     */
    public static JMenuBar getMenuBar() {
        JMenuBar menuBar = new JMenuBar();
        menuBar.setFont(getStandardFont());
        return menuBar;
    }

    /**
     * Returns a combobox with the items " ", "yes" and "no"
     *
     * @param nameOfField   name of this field
     * @return (JComboBox)  a combox with the items "yes" and "no", the first
     *                      item is an empty String
     */
    public static JComboBox getYesNoCombo() {
        JComboBox comboBox = new JComboBox();
        comboBox.addItem("");
        comboBox.addItem(Resources.getText(DCLabels.lblYes));
        comboBox.addItem(Resources.getText(DCLabels.lblNo));
        comboBox.setFont(getStandardFont());
        return comboBox;
    }

    public static JTable getTableForObject(DataCrowObject o, 
                                           DefaultTableModel tableModel) {
        JTable table = getSimpleTable(tableModel, true);

        String[] headers = o.getLabels();
        int[] maxLength = o.getMaxFieldLengths();
        int[] fieldTypes = o.getFieldTypes();

        ((TableSorter) table.getModel()).getModel().setColumnCount(fieldTypes.length);

        for (int i = 0; i < fieldTypes.length; i++) {
            TableColumn columnNew = table.getColumnModel().getColumn(i);
            columnNew.setHeaderValue(headers[i]);

            JComponent comp = getComponent(fieldTypes[i],
                                           headers[i],
                                           maxLength[i]);
            if (fieldTypes[i] == _LONGTEXTFIELD) {
                comp = getComponent(_SHORTTEXTFIELD,
                                    headers[i],
                                    maxLength[i]);
            }

            if (i == 0) {
                JTextField text = getTextField();
                text.setEditable(false);
                columnNew.setCellEditor(new DefaultCellEditor(text));
                DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
                renderer.setBackground(getDisabledColor());
                columnNew.setCellRenderer(renderer);
            } else {
                switch (fieldTypes[i]) {
                    case _NUMBERFIELD:
                    case _LONGTEXTFIELD :
                    case _SHORTTEXTFIELD :
                    case _URLFIELD :
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) comp));                        
                        break;
                    case _SOFTWARETYPECOMBO:
                    case _MP3GENRECOMBO:
                    case _YEARCOMBO:
                    case _YESNOCOMBO:
                        JComboBox combo = getMP3GenreComboBox();
                        columnNew.setCellEditor(new DefaultCellEditor((JComboBox) comp));
                        break;
                }              
            }
        }
        return table;
    }


    public static JTable getSimpleTable(DefaultTableModel tableModel,
                                          boolean autoScrolls) {

        TableSorter sorter = new TableSorter(tableModel);
        JTable table = new JTable(sorter);
        sorter.addMouseListenerToHeaderInTable(table);

        table.setBackground(getTableColor());
        table.setAutoscrolls(autoScrolls);

        table.setDefaultRenderer(Object.class, new MyTableCellRenderer());
        table.setFont(getStandardFont());

        table.setRowSelectionAllowed(true);
        table.setColumnSelectionAllowed(true);
        table.setCellSelectionEnabled(true);
        table.setRequestFocusEnabled(true);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        table.setAlignmentY(JTable.TOP_ALIGNMENT);
        table.getTableHeader().setReorderingAllowed(true);
        table.setShowHorizontalLines(false);

        table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        return table;
    }

    public static DefaultTableCellRenderer getRequiredTableHeaderRenderer() {
        DefaultTableCellRenderer hdrRenderer =
                        new DefaultTableCellRenderer();
        hdrRenderer.setForeground(getRequiredColor());
        hdrRenderer.setBorder(new EtchedBorder());
        return hdrRenderer;
    }

    public static DefaultTableCellRenderer getTableHeaderRenderer() {
        DefaultTableCellRenderer hdrRenderer =
                        new DefaultTableCellRenderer();
        hdrRenderer.setForeground(new Color(0,0,0));
        hdrRenderer.setBorder(new EtchedBorder());
        return hdrRenderer;
    }

    /**
     * Table cell renderer for displaying tooltips of the content of the cells
     */
    private static class MyTableCellRenderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(
                JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {

            Component c = super.getTableCellRendererComponent(  table,
                                                                value,
                                                                isSelected,
                                                                hasFocus,
                                                                row,
                                                                column);

            if (value instanceof Object) {
                ((JComponent) c).setToolTipText(value.toString());
            }
            
            setEnabled(table == null || table.isEnabled());

            if ((row % 2) == 0) {
                setBackground(new Color(253, 254, 242));
            } else {
                setBackground(null);
            }

            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

            //return this;

            
            return c;
        }
    }

    /**
     * Returns a standard border with a title
     *
     * @param title             display String
     * @return (TitledBorder)   border with text
     */
    public static TitledBorder getBorder(String title) {
        TitledBorder border = new TitledBorder(new javax.swing.border.EtchedBorder(), title);
        border.setTitleFont(new Font("Comic Sans MS", Font.BOLD, 11));
        border.setTitleColor(new Color(0,0,0));
        return border;
    }

    /**
     * Returns the default font
     *
     * @return (Font)   Verdana font, pixelheight 11, true type
     */
    public static Font getStandardFont() {
        return new Font("Verdana", Font.TRUETYPE_FONT, 11);
    }

    /**
     * Returns the color for the application background
     */
    public static Color getApplicationColor() {
        return new Color(200,200, 200);
    }

    /**
     * Returns the color for displaying disabled fields
     */
    public static Color getDisabledColor() {
        return new Color(204,204,204);
    }

    /**
     * Returns the color for displaying required fields
     */
    public static Color getRequiredColor() {
        return new Color(255, 0, 0);
    }

    /**
     * Returns the color for the table header
     */
    public static Color getTableHeaderColor() {
        return new Color(150,150,150);
    }

    /**
     * Returns the color for the background of tables
     */
    public static Color getTableColor() {
        return new Color(255, 255, 255);
    }


    public static class RefreshSoftwareCategoriesAction implements MouseListener {
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {
            refresh(e);
        }
        public void mousePressed(MouseEvent e) {}
        
        private void refresh(MouseEvent e) {
            JComboBox comboBox = null;
            if (e.getSource() instanceof AbstractButton) {
                AbstractButton button = (AbstractButton) e.getSource();
                if (button.getParent() instanceof JComboBox) {
                    comboBox = (JComboBox) button.getParent();
                }
            }
            
            if (comboBox != null) { 
                Object o = comboBox.getSelectedItem();
                comboBox = fillSoftwareCatagories(comboBox);
                comboBox.setSelectedItem(o);
            }
        }
    }

    public static void setComponentValue(int fieldType, JComponent comp, Object value) {
        switch (fieldType) {
            case (ComponentFactory._SOFTWARETYPECOMBO):
            case (ComponentFactory._YESNOCOMBO):
            case (ComponentFactory._MP3GENRECOMBO):
            case (ComponentFactory._YEARCOMBO):
                ((JComboBox) comp).setSelectedItem(value);
                break;
            case (ComponentFactory._NUMBERFIELD):
                ((NumberField) comp).setValue(value);
                break;
            case (ComponentFactory._LONGTEXTFIELD):
                ((LongTextField) comp).setValue(value);
                break;
            default:
                ((JTextField) comp).setText(value.toString());
        }
    }

    public static Object getComponentValue(int fieldType, JComponent comp) {
        switch (fieldType) {
            case (ComponentFactory._SOFTWARETYPECOMBO):
            case (ComponentFactory._YESNOCOMBO):
            case (ComponentFactory._MP3GENRECOMBO):
            case (ComponentFactory._YEARCOMBO):
                return ((JComboBox) comp).getSelectedItem().toString();
            case (ComponentFactory._NUMBERFIELD):
                return ((NumberField) comp).getValue();
            case (ComponentFactory._LONGTEXTFIELD):
                return ((LongTextField) comp).getValue();
            default:
                return ((JTextField) comp).getText();
        }
    }
}
