package rjw.dc.console.components;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JTextField;

import rjw.dc.console.BrowserDialog;
import rjw.dc.console.MessageBox;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;

public class FileField extends JComponent {

    private final JTextField text;
    private final JButton button;
    
    private ActionListener al;
    
    private File file = null;
    
    public FileField(boolean save, boolean select, boolean dirsOnly) {
        text = ComponentFactory.getTextField();
        text.setEditable(false);
        text.setBackground(ComponentFactory.getDisabledColor());
        button = ComponentFactory.getButton("..");
        
        if (save) {
            button.addActionListener(new ShowSaveDialogAction());
        } else {
            if (dirsOnly) {
                button.addActionListener(new ShowSelectDirectoryDialogAction());
            } else {    
                button.addActionListener(new ShowSelectDialogAction());
            }
        }
        
        this.setLayout(new GridBagLayout());
        
        add( text,   new GridBagConstraints( 0, 0, 1, 1, 80.0, 80.0
                    ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 0, 5, 5), 0, 0));
                     
        button.setSize(button.getWidth(), text.getHeight());
        add( button, new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                     new Insets(0, 0, 8, 0), 0, 0));
    }

    public File getFile() {
        return file;
    }
    
    public String getFileAsString() {
        if (file == null) {
            return "";
        } else {
            return file.toString();
        }
    }
    
    public void setFile(File file) {
        this.file = file; 
        if (file == null) {
            text.setText("");  
        } else {
            text.setText(file.toString());
        }
    }
    
    protected void showFileSaveDialog() {
        BrowserDialog dialog = new BrowserDialog(Resources.getText(
                                     DCLabels.lblFileBrowser));
                                     
        file = dialog.showCreateFileDialog(this);
        text.setText(getFileAsString());

        if (file != null) {
            try {
                RandomAccessFile random = new RandomAccessFile(file, "rw");
                random.close();
            } catch (Exception exp) {
                new MessageBox( Resources.getText(DCMessages.msgFileCouldNotbeCreated), 
                                true, MessageBox._ERROR);
                Messages.setMessage(
                    Resources.getText(DCMessages.msgFileCouldNotbeCreated), 
                    Messages._ERROR);
                Messages.setMessage(exp.getMessage(), Messages._ERROR);
            }
        }
    }
    
    public void addListener(ActionListener al) {
        this.al = al;
    }
    
    protected void showFileOpenDialog() {
        BrowserDialog dialog = new BrowserDialog(Resources.getText(DCLabels.lblFileBrowser));
        
        file = dialog.showOpenFileDialog(this, null);
        text.setText(getFileAsString());

        al.actionPerformed(new ActionEvent(this.getParent(), 1, ""));
    }

    protected void showDirectoryOpenDialog() {
        BrowserDialog dialog = new BrowserDialog(Resources.getText(DCLabels.lblFileBrowser));
        
        file = dialog.showSelectDirectoryDialog(this, null);
        text.setText(getFileAsString());
    }

    private class ShowSelectDirectoryDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showDirectoryOpenDialog();
        }
    }

    private class ShowSelectDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileOpenDialog();
        }
    }

    private class ShowSaveDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showFileSaveDialog();
        }
    }
}
