/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.components;

import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import rjw.flex.settings.ui.ISettingsComponent;

/**
 * Simple field, derived from JTextField, which does not allow non
 * numeric characters to be inserted
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class NumberField extends JTextField implements ISettingsComponent {

    protected class WholeNumberDocument extends PlainDocument {

        protected WholeNumberDocument() {
        }

        public void insertString(int i, String s, AttributeSet attributeset)
            throws BadLocationException {

            char ac[] = s.toCharArray();
            char ac1[] = new char[ac.length];
            int j = 0;
            for(int k = 0; k < ac1.length; k++)
                if(Character.isDigit(ac[k]))
                    ac1[j++] = ac[k];
                else
                    toolkit.beep();

            super.insertString(i, new String(ac1, 0, j), attributeset);
        }
    }

    private Toolkit toolkit;
    private NumberFormat integerFormatter;

    public NumberField() {
        super();
        toolkit = Toolkit.getDefaultToolkit();
        integerFormatter = NumberFormat.getNumberInstance(Locale.US);
        integerFormatter.setParseIntegerOnly(true);
    }

    protected Document createDefaultModel() {
        return new WholeNumberDocument();
    }

    public Object getValue() {
        int i = -1;
        try {
            i = integerFormatter.parse(getText()).intValue();
        } catch(ParseException _ex) {
            toolkit.beep();
        }
        return new Integer(i);
    }

    public void setValue(int i) {
        setText(integerFormatter.format(i));
    }

    public void setValue(Object o) {
        int i = 0;
        try {
            i = integerFormatter.parse(o.toString()).intValue();
        } catch (Exception ignore) {}
        
        o = new Integer(i);

        setText(integerFormatter.format(((Integer) o).intValue()));
    }
}
