/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.components;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import rjw.dc.core.Repository;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.flex.settings.ui.ISettingsComponent;

/**
 * Table in which the required fields for a module can be set 
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class RequiredFieldsTable extends JComponent implements ISettingsComponent {
    
    private JTable table;
    private final int module;
    private final DefaultTableModel tableModel = new DefaultTableModel();
    
    public RequiredFieldsTable(int module) {
        this.module = module;
        buildPanel();
    }

    private void buildPanel() {
        JPanel panelRequiredFields = new JPanel(false);
        panelRequiredFields.setLayout(ComponentFactory.getLayoutManager());

        table = ComponentFactory.getSimpleTable(tableModel, true);
        table.setPreferredScrollableViewportSize(new Dimension(402, 285));

        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();

        tableModel.setColumnCount(2);
        TableColumn columnNew   = table.getColumnModel().getColumn(0);
        columnNew.setHeaderValue(Resources.getText(DCLabels.lblFields));

        JTextField  text    = new JTextField();
        text.setEditable(false);
        columnNew.setCellEditor(new DefaultCellEditor(text));
        renderer = new DefaultTableCellRenderer();
        renderer.setBackground(ComponentFactory.getDisabledColor());
        columnNew.setCellRenderer(renderer);
        
        columnNew = table.getColumnModel().getColumn(1);
        columnNew.setHeaderValue(Resources.getText(DCLabels.lblRequiredYN));
        JComboBox comboBox = ComponentFactory.getRealYesNoComboBox();
        columnNew.setCellEditor(new DefaultCellEditor(comboBox));
        renderer  = new DefaultTableCellRenderer();
        columnNew.setCellRenderer(renderer);

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);        

        panelRequiredFields.add(scrollResults,  new GridBagConstraints( 0, 0, 1, 1, 10.0, 10.0
                                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                new Insets( 5, 5, 5, 5), 0, 0));    

        setLayout(ComponentFactory.getLayoutManager());

        add(panelRequiredFields, new GridBagConstraints( 0, 0, 1, 1, 10.0, 10.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets( 0, 0, 0, 0), 0, 0));
    }
    
    public void setValue(Object value) {
        tableModel.setRowCount(0);
        
        String[] fields = null;
        int[] requiredFieldIds = (int[]) value;
        
        switch (module) {
            case 0:
                fields = Repository.dataBase.dbSoftColArray;
                break;
            case 1:
                fields = Repository.dataBase.dbMP3ColArray;
                break;
            case 2:
                fields = Repository.dataBase.dbMovieColArray;
                break;
        }
        
        for (int i = 0; i < fields.length; i++) {
            String[] row = new String[2];
            row[0] = fields[i];
            row[1] = Resources.getText(DCLabels.lblNo);
            for (int j = 0; j < requiredFieldIds.length; j++) {
                if (requiredFieldIds[j] == i + 1) {
                    row[1] = Resources.getText(DCLabels.lblYes);   
                }
            }   
            tableModel.addRow(row);
        }
    }    
    
    public Object getValue() {
        Vector vFields = new Vector(0);
        for (int i = 0; i < table.getRowCount(); i++) {
            String sText = table.getValueAt(i, 1).toString();
            if (sText.equals(Resources.getText(DCLabels.lblYes))) {
                vFields.add(new Integer(i + 1));
            }
        }

        int[] fields = new int[vFields.size()];
        
        int i = 0;
        while (!vFields.isEmpty()) {
            fields[i++] = ((Integer) vFields.remove(0)).intValue();
        }
        return fields;
    }
}
