/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.da;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;

import rjw.dc.console.ExportCDDialog;
import rjw.dc.console.GeneralDialog;
import rjw.dc.console.MessageBox;
import rjw.dc.console.TablePanel;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.console.components.FileField;
import rjw.dc.core.Repository;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.processes.ExportMP3CD;
import rjw.dc.processes.ExportMovieCD;
import rjw.dc.processes.ExportSoftwareCD;
import rjw.dc.processes.ImporList;
import rjw.dc.processes.ImportInformationFile;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Resources;
import rjw.dc.util.StringUtils;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataExchangePanel extends TablePanel {

    private JPanel             panelExportActions;
    private JPanel             panelImportActions;
    private JPanel             panelExportCD;
    private JPanel             panelImportFile;
    private JPanel             panelImportSettings;
    private JPanel             panelTabAct;
    private JPanel             panelStatus;

    private JButton            buttonSaveTable;
    private JButton            buttonLoadTable;
    private JButton            buttonAddRow;
    private JButton            buttonRemoveRow;

    private FileField          fieldExportFile;
    private FileField          fieldImportFile;

    private JButton            buttonFileImport;
    private JLabel             labelSource;
    private JTextField         textCDName;
    private JLabel             labelCDName;
    private JLabel             labelFile;
    private JButton            buttonCDExport;
    private JLabel             labelLocation;
    private FileField          fieldLocation;

    private JCheckBox checkVerify;
    private JCheckBox checkToFile;
    private JCheckBox checkToTable;

    private static final WorkFlow wf = new WorkFlow();


    public DataExchangePanel(int module) {
        super();
        this.module = module;
        buildPanel();
    }

    protected int[] getRecordHeaderFields() {
        return null;   
    }
    
    protected void importFile() {
        Object      oTrue   = new Boolean(true);

        File file = getImportFile();
        
        if (file == null || !file.exists()) {
            new MessageBox( Resources.getText(DCMessages.msgIIFSelectImpFile), 
                            true, MessageBox._NOTIFICATION);
            return;
        }
        
        ImportInformationFile iff = new ImportInformationFile(file.toString());
        GeneralDialog dialog = new GeneralDialog(Resources.getText(DCMessages.msgStartImport));

        if (GeneralDialog.result != null) {
            if (GeneralDialog.result.equals(oTrue)) {
                Messages.setMessage(Resources.getText(DCMessages.msgImportStarts), Messages._STANDARD);
                MessagesWriter._useLogFile = false;
                Thread threadImportFile = new Thread(new ImporList(file.toString()), "ImportFile");
                threadImportFile.setDaemon(true);
                threadImportFile.setPriority(1);
                threadImportFile.start();
                MessagesWriter._windowToFront = true;
                MessagesWriter._windowShow    = true;
            }
        }
    }

    protected void startCDExport() {
        boolean bVerify = checkVerify.isSelected();
        boolean bToFile = checkToFile.isSelected();
        boolean bToTable = checkToTable.isSelected();
        
        String location = "";
        if (fieldLocation.getFile() != null) {
            location = fieldLocation.getFileAsString();
        }
        
        if (location == null || location.equals("")) {
            new MessageBox(Resources.getText(DCMessages.msgSelectDirectory), 
                    true, MessageBox._INFORMATION);
            return;
        }
        
        String cdName = textCDName.getText().trim();

        if (!bToFile && !bToTable) {
            new MessageBox(Resources.getText(DCMessages.msgSelectExportTarget), 
                           true, MessageBox._INFORMATION);
            return;
        }

        if (cdName.equals("")) {
            new MessageBox(Resources.getText(DCMessages.msgEnterCDName), 
                           true, MessageBox._INFORMATION);
            return;
        }
        
        if (bToFile && (getExportFile() == null || 
                        getExportFile().equals(""))) {
                                   
            new MessageBox(Resources.getText(DCMessages.msgEnterFilename), 
                           true, MessageBox._INFORMATION);
            return;
        }

        if (bToFile) {
            try {
                File test = getExportFile();
                RandomAccessFile randomAccessTest;
                randomAccessTest = new RandomAccessFile(test, "rw");
                randomAccessTest.close();
            } catch (Exception ioExp) {
                new MessageBox( Resources.getText(DCMessages.msgInvalidFileName), 
                                true, MessageBox._INFORMATION);
                return;
            }
        }


        String exportFile = getExportFile() == null ? "" :
                            getExportFile().toString(); 
        Object oTrue   = new Boolean(true);
        new ExportCDDialog(   location,
                              new Boolean(bVerify).toString(),
                              exportFile,
                              cdName,
                              new Boolean(bToFile).toString(),
                              new Boolean(bToTable).toString());

        if (ExportCDDialog.result != null) {
            if (ExportCDDialog.result.equals(oTrue)) {
                Messages.setMessage(Resources.getText(DCMessages.msgExportStarts), 
                                    Messages._STANDARD);
                MessagesWriter._useLogFile    = false;

                Thread  exportCDThread = null;
                if (module == Repository.module._SOFTWARE) {
                    exportCDThread  = new Thread(new ExportSoftwareCD(
                                                            location, 
                                                            bVerify, 
                                                            exportFile, 
                                                            cdName, 
                                                            bToFile, 
                                                            bToTable), 
                                        "ExportCD-Prio5");
                } else if (module == Repository.module._MP3) {
                    exportCDThread  = new Thread(new ExportMP3CD(
                                                            location, 
                                                            bVerify, 
                                                            exportFile, 
                                                            cdName, 
                                                            bToFile, 
                                                            bToTable), 
                                                 "ExportCD-Prio5");
                } else if (module == Repository.module._MOVIE) {
                    exportCDThread  = new Thread(new ExportMovieCD(
                                                            location, 
                                                            bVerify, 
                                                            exportFile, 
                                                            cdName, 
                                                            bToFile, 
                                                            bToTable), 
                                                 "ExportCD-Prio5"); 
                }
                
                exportCDThread.setPriority(1);
                exportCDThread.setDaemon(true);
                exportCDThread.start();
                MessagesWriter._windowToFront = true;
                MessagesWriter._windowShow    = true;
            }
        }
    }

    private void setValues() {
        String  sourceFile = "";
        String  targetFile = "";

        if (module == 0) {
            sourceFile = Settings.getValueAsString(Repository.settings.stSoftImportFile);
            targetFile = Settings.getValueAsString(Repository.settings.stSoftExportFile);
        } else if (module == 1) {
            sourceFile = Settings.getValueAsString(Repository.settings.stMP3ImportFile);
            targetFile = Settings.getValueAsString(Repository.settings.stMP3ExportFile);
        } else if (module == 2) {
            sourceFile = Settings.getValueAsString(Repository.settings.stMovieImportFile);
            targetFile = Settings.getValueAsString(Repository.settings.stMovieExportFile);
        } 

        try {
            setImportFile(new File(targetFile.trim()));
        } catch (Exception ignore) {}

        try {
            setExportFile(new File(sourceFile.trim()));
        } catch (Exception ignore) {}
        fillTable();
    }

    protected void fillTable() {
        clearTable();
        String[][]  data    = null;

        if (getImportFile() != null) {
            ImportInformationFile iif = new ImportInformationFile(getImportFile().toString());
            if (iif != null) {
                data = iif.getIIFValues();
            }

            if (data == null) {
                return;
            } else if (data.length <= 0) {
                return;
            }

            for (int i = 0; i < data.length; i++) {
                String[] temp = new String[tableModel.getColumnCount()];
                temp[0] = data[i][0];
                temp[1] = data[i][1];

                tableModel.addRow(temp);
            }
        }
    }

    private File getExportFile() {
        return fieldExportFile.getFile();
    }

    private File getImportFile() {
        return fieldImportFile.getFile();
    }

    private class ImportFileButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (ExportSoftwareCD.running == false && ImporList.running == false) {
                importFile();
            } else {
                new MessageBox( Resources.getText(DCMessages.msgProcessRunsAlready), 
                                true, MessageBox._INFORMATION);
            }
        }
    }

    private String[][] getAllTableValues() {
        String[][] values   = new String[table.getRowCount()][table.getColumnCount()];
        for (int i = 0; i < table.getRowCount(); i++) {
            for (int j = 0; j < table.getColumnCount(); j++) {
                Object oValue = table.getValueAt(i, j);
                if (oValue != null) {
                    String sValue = oValue.toString();
                    if (!StringUtils.isEmpty(oValue.toString())) {
                        values[i][j] = sValue;
                    } else {
                        new MessageBox(Resources.getText(DCMessages.msgNotAllIFFValuesFilled),
                                       true, MessageBox._INFORMATION);
                        return null;   
                    }
                }
            }
        }
        return values;
    }

    protected void saveAllTableValues() {
        File file = getImportFile();

        if (file == null) {
            new MessageBox( Resources.getText(DCMessages.msgIIFSelectImpFile), 
                            true, MessageBox._NOTIFICATION);
            return;
        } else {
            if (!file.exists()) {
                new MessageBox(Resources.getText(DCMessages.msgIIFSelectImpFile), 
                               true, MessageBox._NOTIFICATION);
                return;
            }
        }
        String[][] values = getAllTableValues();
        
        if (values != null) {
            new ImportInformationFile(values, file.toString());
            new MessageBox( Resources.getText(DCMessages.msgDataSaved), 
                            true, MessageBox._INFORMATION);
        }
    }

    private void setExportFile(File exportFile) {
        fieldExportFile.setFile(exportFile);
        String sFileName = exportFile.toString();
        
        if (module == 0) {
            Settings.setValue(Repository.settings.stSoftExportFile, sFileName);
        } else if (module == 1) {
            Settings.setValue(Repository.settings.stMP3ExportFile, sFileName);
        } else if (module == 2) {
            Settings.setValue(Repository.settings.stMovieExportFile, sFileName);
        }
    }

    private boolean validateImportFile(File importFile) {
        String sModule = "";

        if (importFile == null) {
            return false;
        } else {
            String sFileName = importFile.toString();
            if (sFileName.trim().equals("")) {
                return false;
            }
        }

        try {
            RandomAccessFile access = new RandomAccessFile(importFile, "r");
            sModule = access.readLine().trim();

            if (module == 0 &&
                !sModule.equals("->" + Repository.dataBase.dBSoftTableName.trim() + "<-")) {
                new MessageBox(Resources.getText(DCMessages.msgInvalidSoftwareImportFile),
                               true, MessageBox._ERROR);
                return false;
            } else if (module == 1 &&
                !sModule.equals("->" + Repository.dataBase.dBMP3TableName.trim() + "<-")) {
                new MessageBox(Resources.getText(DCMessages.msgInvalidMP3ImportFile),
                               true, MessageBox._ERROR);
                return false;
            } else if (module == 2 &&
                !sModule.equals("->" + Repository.dataBase.dBMovieTableName.trim() + "<-")) {
                new MessageBox(Resources.getText(DCMessages.msgInvalidMovieImportFile),
                               true, MessageBox._ERROR);
                return false;
            }         
        } catch (Exception exp) {
            return false;
        }
        return true;
    }

    private void setImportFile(File importFile) {
        fieldImportFile.setFile(importFile);

        if (!validateImportFile(importFile)) {
            if (module == 0) {
                Settings.setValue(Repository.settings.stSoftImportFile, "");
            } else if (module == 1) {
                Settings.setValue(Repository.settings.stMP3ImportFile, "");
            } else if (module == 2) {
               Settings.setValue(Repository.settings.stMovieImportFile, "");
            } 

            clearTable();
            return;
        }

        fillTable();
        if (module == 0) {
            Settings.setValue(Repository.settings.stSoftImportFile,
                              fieldImportFile.getFileAsString());
        } else if (module == 1) {
            Settings.setValue(Repository.settings.stMP3ImportFile,
                              fieldImportFile.getFileAsString());
        } else if (module == 2) {
            Settings.setValue(Repository.settings.stMovieImportFile,
                              fieldImportFile.getFileAsString());
        } 
    }

    private void buildPanel() {
        panelImportSettings = new JPanel(false);
        panelImportFile     = new JPanel(false);
        panelExportCD       = new JPanel(false);
        panelTabAct         = new JPanel(false);
        panelExportActions  = new JPanel(false);
        panelImportActions  = new JPanel(false);
        panelStatus         = new JPanel(false);


        // ************************************************************************
        // EXPORT CD PANEL
        // Set layout style
        GridBagLayout       gridbExportCD   = new GridBagLayout();
        panelExportCD.setLayout(gridbExportCD);

        // Initialize components
        labelLocation           = ComponentFactory.getLabel(Resources.getText(DCLabels.lblCDLocation));
        fieldLocation           = new FileField(false, true, true);
        checkVerify             = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblAskPerDir));
        labelFile               = ComponentFactory.getLabel(Resources.getText(DCLabels.lblToExpFile) + ":");
        fieldExportFile         = new FileField(true, false, false);
        labelCDName             = ComponentFactory.getLabel(Resources.getText(DCLabels.lblLocationName));
        textCDName              = ComponentFactory.getTextField();
        checkToFile             = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblExportToFile));
        checkToTable            = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblExportToTable));

        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        checkToFile.setBackground(color);
        checkToTable.setBackground(color);
        checkVerify.setBackground(color);

        // Build the panel
        panelExportCD.add(  labelLocation,  new GridBagConstraints( 0, 0, 2, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  fieldLocation,  new GridBagConstraints( 0, 1, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  labelCDName,    new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  textCDName,     new GridBagConstraints( 1, 2, 2, 1, 20.0, 20.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  checkVerify,    new GridBagConstraints( 0, 3, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 0, 5), 0, 0));
        panelExportCD.add(  checkToTable,   new GridBagConstraints( 0, 4, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  checkToFile,    new GridBagConstraints( 0, 5, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelExportCD.add(  labelFile     , new GridBagConstraints( 0, 6, 2, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 0, 5), 0, 0));
        panelExportCD.add(  fieldExportFile,new GridBagConstraints( 0, 7, 3, 1, 1.0, 1.0
                                           ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(2, 5, 0, 5), 0, 0));


        // ************************************************************************
        // EXPORT FILE ACTION PANEL
        // Set layout style
        GridBagLayout       gridbExpAct = new GridBagLayout();
        panelExportActions.setLayout(gridbExpAct);

        // Initilialize components
        buttonCDExport      = ComponentFactory.getButton(Resources.getText(DCLabels.lblStartImport));
        buttonCDExport.addActionListener(new ExportCDButtonAction());

        // Build the panel
        panelExportActions.add(buttonCDExport,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                                new Insets( 0, 10, 10, 10), 0, 0));


        // ************************************************************************
        // IMPORT FILE PANEL
        // Set layout style
        GridBagLayout       gridbImportFile = new GridBagLayout();
        panelImportFile.setLayout(gridbImportFile);

        // Initialize components
        labelSource = ComponentFactory.getLabel(Resources.getText(DCLabels.lblSourceFile));
        fieldImportFile = new FileField(false, true, false);      
        fieldImportFile.addListener(new LoadTableButtonAction());

        labelSource.setToolTipText(Resources.getText(DCTooltips.tpSourceFile) + ":");

        // Build the panel
        panelImportFile.add(  labelSource,     new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets(0, 5, 0, 5), 0, 0));
        panelImportFile.add(  fieldImportFile, new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets(2, 5, 0, 5), 0, 0));


        // ************************************************************************
        // IMPORT FILE ACTION PANEL
        // Set layout style
        GridBagLayout       gridbImpAct = new GridBagLayout();
        panelImportActions.setLayout(gridbImpAct);

        // Initialize components
        buttonFileImport    = ComponentFactory.getButton(Resources.getText(DCLabels.lblStartImport));
        buttonFileImport.addActionListener(new ImportFileButtonAction());

        // Build the panel
        panelImportActions.add( buttonFileImport,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                                   new Insets( 0, 10, 5, 10), 0, 0));


        // ************************************************************************
        // IMPORT SETTINGS PANEL
        // Set layout style
        GridBagLayout       gridbImportSettings = new GridBagLayout();
        panelImportSettings.setLayout(gridbImportSettings);

        // Initialize components
        tableModel.setColumnCount(2);
        table   = ComponentFactory.getSimpleTable(tableModel, false);
        JScrollPane scrollSettings = new JScrollPane(table);
        scrollSettings.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        TableColumn column1  = table.getColumnModel().getColumn(0);
        JComboBox combo = ComponentFactory.getFieldsComboBox(WorkFlow.getObject(module));
        column1.setCellEditor(new DefaultCellEditor(combo));
        column1.setHeaderRenderer(ComponentFactory.getTableHeaderRenderer());
        column1.setHeaderValue(Resources.getText(DCLabels.lblFields));

        TableColumn column2  = table.getColumnModel().getColumn(1);
        column2.setHeaderRenderer(ComponentFactory.getTableHeaderRenderer());
        column2.setHeaderValue(Resources.getText(DCLabels.lblSeperator));

        // Build the panel
        panelImportSettings.add( scrollSettings,  new GridBagConstraints( 0, 0, 1, 1, 10.0, 10.0
                                                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                                                  new Insets( 0, 5, 5, 5), 0, 0));


        // ************************************************************************
        // TABLE ACTION PANEL
        // Set layout style
        GridBagLayout       gridbTabAct = new GridBagLayout();
        panelTabAct.setLayout(gridbTabAct);

        // Initialize Components
        buttonSaveTable = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave), 100);
        buttonLoadTable = ComponentFactory.getButton(Resources.getText(DCLabels.lblLoad), 100);
        buttonRemoveRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblRemoveRow), 100);
        buttonAddRow = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddRow), 100);

        buttonSaveTable.setMnemonic(KeyEvent.VK_S);
        buttonLoadTable.setMnemonic(KeyEvent.VK_L);
        buttonRemoveRow.setMnemonic(KeyEvent.VK_R);
        buttonAddRow.setMnemonic(KeyEvent.VK_A);

        buttonSaveTable.addActionListener(new SaveTableButtonAction());
        buttonLoadTable.addActionListener(new LoadTableButtonAction());
        buttonAddRow.addActionListener(new AddRowAction());
        buttonRemoveRow.addActionListener(new RemoveRowAction());

        // Build the panel
        panelTabAct.add( buttonSaveTable,  new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 10, 0, 5), 0, 0));
        panelTabAct.add( buttonLoadTable,  new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 0, 0, 5), 0, 0));
        panelTabAct.add( buttonRemoveRow,  new GridBagConstraints( 2, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 0, 0, 5), 0, 0));
        panelTabAct.add( buttonAddRow,     new GridBagConstraints( 3, 0, 1, 1, 1.0, 1.0
                                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                           new Insets( 0, 0, 0, 10), 0, 0));


        //  ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

         // Initialize components
        labelStatus = ComponentFactory.getLabel(
                      Resources.getText(DCLabels.lblVersion));

        // Build the panel
        panelStatus.add(labelStatus, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbag         = new GridBagLayout();
        setLayout(gridbag);

        // Initialize components
        panelImportSettings.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblIIFData)));
        panelImportFile.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblImportaFile)));
        panelExportCD.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblExportaCD)));
        panelStatus.setBorder(ComponentFactory.getBorder(Resources.getText(DCLabels.lblStatus)));

        // Build the panel
        add( panelExportCD,         new GridBagConstraints( 0, 0, 1, 1, 20.0, 20.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelExportActions,    new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.WEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( new JPanel(false),     new GridBagConstraints( 0, 2, 1, 1, 5.0, 5.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelImportFile,       new GridBagConstraints( 0, 8, 1, 1, 20.0, 20.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelImportActions,    new GridBagConstraints( 0, 9, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelImportSettings,   new GridBagConstraints( 1, 0, 1, 9, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelTabAct,           new GridBagConstraints( 1, 9, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                    new Insets(5, 5, 5, 5), 0, 0));
        add( panelStatus,           new GridBagConstraints( 0, 10, 2, 1, 1.0, 1.0
                                   ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                    new Insets(5, 5, 5, 5), 0, 0));

        setValues();
    }

    public void setBackgrounds(Color color) {
        super.setBackgrounds(color);
        checkToFile.setBackground(color);
        checkToTable.setBackground(color);
        checkVerify.setBackground(color);
    }


    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************

    private class SaveTableButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveAllTableValues();
        }
    }

    private class LoadTableButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            fillTable();
        }
    }
    
    public class AddRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            tableModel.addRow(new Object[] {null, null, null});
        }
    }    

    private class ExportCDButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (ExportSoftwareCD.running == false && ImporList.running == false) {
                startCDExport();
            } else {
                new MessageBox(Resources.getText(DCMessages.msgProcessRunsAlready), 
                               true, 
                               MessageBox._INFORMATION);
            }
        }
    }
    
    private class LoadIFFForImport extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            fillTable();
        }
    }
}
