/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.software;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import rjw.dc.console.MessageBox;
import rjw.dc.console.SearchForm;
import rjw.dc.core.Repository;
import rjw.dc.http.AmazonQuery;
import rjw.dc.http.HttpConnection;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.util.SwingUtil;
import rjw.flex.settings.Settings;

/**
 * Searches Amazon.com for information about programs and games
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.75
 */
public class AmazonSearchForm extends SearchForm {
    
    public AmazonSearchForm(int rowNumber, 
                            String title, 
                            String name, 
                            Object sender) {
                          
        super(sender, new int[] {1, 2, 3});
        super.setTitle(title);
        SwingUtil.setRootFrame(this);        
    
        isUpdateRunning = true;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                                  getSize());
                running = false;
            }
        });        
    
        this.rowNumber = rowNumber;
    
        textSearch.setText(name);
        startSearch();
    
        setBackgrounds(Settings.getValueAsColor(Repository.settings.stColor));
        setVisible(true);
    }

    protected void startSearch() {
        String name = textSearch.getText();
        
        if (name == null || name.equals("")) {
            return;   
        }
        threadSearch = new Thread(new SearchJob(name), "SearchJob");
        threadSearch.setDaemon(true);
        threadSearch.setPriority(1);
        threadSearch.start();        
    }

    private class SearchJob extends Thread {
    
        final String name;
    
        public SearchJob(String name) {
            this.name = name;
        }
    
        public void run() {
            try {
                running = true;
                buttonSearch.setEnabled(false);
                buttonStop.setEnabled(true);
                buttonStopAll.setEnabled(true);                
                searchAmazon(name);
            } catch (Exception exp) {
                exp.printStackTrace();
                Messages.setMessage(exp.toString(), Messages._ERROR);
                new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                               true, MessageBox._ERROR);
            }
        }      

        private void searchAmazon(String name) {
            HttpConnection connection = null;
            try {
                connection = new HttpConnection("xml.amazon.com");
                AmazonQuery amazonQuery = new AmazonQuery(name);
                Vector v = amazonQuery.search(connection);
    
                progress.setMaximum(v.size());
                progress.setValue(0);
                
                int i = 1;
                Object[] row = null;
                while (v.size() > 0) {
                    row = amazonQuery.parseInformation(v.remove(0).toString());
                    if (row != null) {
                        setData(row);
                    }                    
                    progress.setValue(i++);    
                } 
                
            } catch (Exception exp) {
                new MessageBox(exp.getMessage(), 
                               true, 
                               MessageBox._ERROR);   
            } finally {
                buttonSearch.setEnabled(true);
                buttonStop.setEnabled(false);
                buttonStopAll.setEnabled(false);
                if (connection != null) {
                    connection.close();
                }     
                finishAll();       
            }
        }
    
        /**
         * Stops the process
         */
        private void finishAll() {
            try {
                Messages.setMessage(
                    Resources.getText(DCMessages.msgIMDBProcessStops), 
                    Messages._STANDARD);
            } catch (Exception ignore) {}
        }
    }
    
    public void setBackgrounds(Color color) {
        this.setBackground(color);
        setCompBackColors(this.getComponents(), color);
    }        

    private void setCompBackColors(Component[] comps, Color color) {
        try {
            for (int i = 0; i < comps.length; i++) {
                comps[i].setBackground(color);
            }
        } catch (java.lang.Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }    
}
