/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.software;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;

import rjw.dc.console.MainFrame;
import rjw.dc.console.TableInsertPanel;
import rjw.dc.console.TableSearchPanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.software.SoftwareCatagories;
import rjw.dc.core.objects.software.SoftwareCatagory;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.dc.util.SwingUtil;
import rjw.flex.settings.Settings;

public class SoftwareCatagoryForm extends JFrame {
    private JTabbedPane tabbedPane = new JTabbedPane();

    public CatagorySearchPanel searchCat = new CatagorySearchPanel();
    public CatagoryInsertPanel insertCat = new CatagoryInsertPanel();

    public SoftwareCatagoryForm() {
        this.setTitle(Resources.getText(DCLabels.lblSoftCatagories));
        ImageIcon image = new ImageIcon("./Images/category.gif");
        this.setIconImage(image.getImage());
        
        tabbedPane.add(Resources.getText(DCLabels.lblExistingValues), searchCat);
        tabbedPane.add(Resources.getText(DCLabels.lblNewValues), insertCat);
        getContentPane().add(tabbedPane);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stSoftwareCatFormSize,
                                  getSize());
                SoftwareCatagories.load();
                                  
            }
        });        
        
        this.setBackgrounds(Settings.getValueAsColor(Repository.settings.stColor));
        pack();
        
        this.setSize(Settings.getValueAsDimension(
                     Repository.settings.stSoftwareCatFormSize));
        
    }

    public CatagorySearchPanel getSearchPanel() {
        return searchCat;
    }

    public CatagoryInsertPanel getInsertPanel() {
        return insertCat;
    }

    public void setBackgrounds(Color color) {
        searchCat.setBackgrounds(color);
        insertCat.setBackgrounds(color);
    }

    public class CatagorySearchPanel extends TableSearchPanel {
        public CatagorySearchPanel () {
            super(Repository.module._SOFTWARECAT);
            this.addMouseListener(new ActiveFrameAction());
        }

        protected int[] getRecordHeaderFields() {
            return new int[] {SoftwareCatagory._A_NAME};   
        }
    }

    public class CatagoryInsertPanel extends TableInsertPanel {
        public CatagoryInsertPanel() {
            super(Repository.module._SOFTWARECAT);
            this.addMouseListener(new ActiveFrameAction());
            table.removeKeyListener(new TableKeyListener());
        }
    }
    
    public void refreshCatagoryCache() {
        SoftwareCatagories.refresh();
    }

    public static class ActiveFrameAction implements MouseListener {
        public void mouseReleased(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {
            SwingUtil.setRootFrame(MainFrame.catagoryForm);
        }
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}        
    }     
}
