/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.console.sql;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.RandomAccessFile;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.ToolTipManager;
import javax.swing.table.TableColumn;

import rjw.dc.console.MessageBox;
import rjw.dc.console.TablePanel;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.util.Converter;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;

/**
 * Thru this panel any kind of query can be run directly on the database.
 * This panel makes no use of the QueryQueue, the queries are run directly.
 * Queries which have been run successfully are stored in a file and added
 * to the listbox
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SQLPanel extends TablePanel {

    private JPanel                 panelComm;
    private JPanel                 panelSQL;
    private JPanel                 panelOut;
    private JPanel                 panelExtra;
    private JPanel                 panelStatus;

    private JEditorPane            textInput;
    private JButton                buttonRunSql;
    private JButton                buttonClear;

    private JComboBox              comboSQLCommands;
    private JLabel                 labelSQLCommands;
    private JLabel                 labelStatus;

    private static Vector          vQueries  = new Vector(0);
    private WorkFlow               wf        = new WorkFlow();

    /**
     * Initializes and creates the panel
     */
    public SQLPanel() {
        super();
        buildPanel();
    }

    protected int[] getRecordHeaderFields() {
        return null;   
    }

    /**
     * Fills the table with the resuls of the query. Header are read from
     * the ResultSet and applied on the headers of result table.
     *
     * @param result    the resultset of which the data should be shown in the
     *                  result table
     */
    private void fillTable(ResultSet result) throws Exception {
        clearTable();

        int iColumns = result.getMetaData().getColumnCount();

        for (int i = 0; i < iColumns; i++) {
            TableColumn column = new TableColumn();
            tableModel.addColumn(column);
        }

        // Workaround for the setHeaderValue problem while adding
        // columns. A newly added column does nothing with the headerValue.
        for (int i = 0; i < iColumns; i++) {
           TableColumn columnNew = table.getColumnModel().getColumn(i);
           columnNew.setHeaderRenderer(ComponentFactory.getTableHeaderRenderer());
           columnNew.setHeaderValue(result.getMetaData().getColumnName(i + 1));
        }

        String[] sValues = new String[iColumns];
        while (result.next()) {
            for (int i = 0; i < iColumns; i++) {
                sValues[i] = result.getString((result.getMetaData().getColumnName(i + 1)).toString());
            }
            tableModel.addRow(sValues);
        }

        setStatus(Resources.getText(DCMessages.msgSQLTableHasBeenFilled));
    }

    /**
     * Runs the specified query directly on the database, without making use
     * of the QueryQueue. If an error occurs, it shown to user in a messagebox.
     * If the query is empty (no query was entered in the screen), a messagebox is
     * shown is well.
     * If the query was successfull, the query itself is stored in the query listbox
     * for later use
     */
    protected void runQuery() {
        String      sQuery   = textInput.getText().trim();

        if (sQuery.trim().equals("")){
            new MessageBox( Resources.getText(DCMessages.msgNoInput), 
                            true, MessageBox._NOTIFICATION);
            return;
        }

        Object result = wf.runQueryDirect(sQuery);

        if (result == null) {
            new MessageBox(Resources.getText(DCMessages.msgQueryWasSuccessFull), 
                           true, MessageBox._NOTIFICATION);
            addQueryToComboBox(sQuery);
            saveDataToFile();
        } else if (result instanceof SQLException) {
            SQLException sqlExp = (SQLException) result;

            if (sqlExp.getMessage().toUpperCase().equals("NO RESULTSET WAS PRODUCED")) {
                if (sQuery.toUpperCase().startsWith("INSERT")) {
                    new MessageBox(
                        Resources.getText(DCMessages.msgInsertQuerySuccesful), 
                        true, MessageBox._INFORMATION);
                } else if (sQuery.toUpperCase().startsWith("CREATE")) {
                    new MessageBox(
                        Resources.getText(DCMessages.msgCreateQuerySuccesful), 
                        true, MessageBox._INFORMATION);
                } else if (sQuery.toUpperCase().startsWith("UPDATE")) {
                    new MessageBox(
                        Resources.getText(DCMessages.msgUpdateQuerySuccesful),
                        true, MessageBox._INFORMATION);
                } else if (sQuery.toUpperCase().startsWith("DELETE")) {
                    new MessageBox(
                        Resources.getText(DCMessages.msgDeleteQuerySuccesful), 
                        true, MessageBox._INFORMATION);
                }
                addQueryToComboBox(sQuery);
                saveDataToFile();
            } else {
                new MessageBox(((SQLException) result).toString(), 
                                true, MessageBox._ERROR);
            }
        } else if (result instanceof ResultSet) {
            try {
                fillTable((ResultSet) result);
                addQueryToComboBox(sQuery);
                saveDataToFile();
            } catch (Exception exp) {
                Messages.setMessage(exp.toString(), Messages._ERROR);
                new MessageBox(exp.toString(), true, MessageBox._ERROR);
            }
        }
    }

    /**
     * Adds a query to the bottom of the query combobox
     */
    private void addQueryToComboBox(String sNewQuery) {
        sNewQuery       = Converter.deleteEntersFromString(
                          sNewQuery.trim());
        String sTemp    = sNewQuery.toLowerCase();

        boolean found = false;
        for (Enumeration enum = vQueries.elements(); enum.hasMoreElements(); ) {
            String sExistingQuery = (String) enum.nextElement();
            if (sExistingQuery.toLowerCase().equals(sTemp)) {
                found = true;
            }
        }

        if (!found) {
            vQueries.add(sNewQuery);
            comboSQLCommands.addItem(sNewQuery);
        }
    }

    /**
     * Fills the query combobox by reading the query file
     */
    private void fillQueryComboBox() {
        File queryFile      = new File(Repository.files.flQueryFile);

        if (queryFile.exists()) {
            try {
                RandomAccessFile    fileAccess  = new RandomAccessFile(queryFile, "rw");
                long                filePointer = 0;
                long                fileLength  = queryFile.length();

                while (filePointer < fileLength) {
                    String query = fileAccess.readLine().trim();
                    addQueryToComboBox(query);
                    filePointer = fileAccess.getFilePointer();
                    vQueries.addElement(query.trim());
                }
            } catch (Exception exp) {}
            if (comboSQLCommands.getItemCount() > 0) {
                return;
            }
        }
        Messages.setMessage(
            Resources.getText(DCMessages.msgFileNotFound, queryFile.toString()), 
            Messages._ERROR);
        comboSQLCommands.addItem("");
        for (int i = 0; i < Repository.sql.predefinedQueries.length; i ++) {
            addQueryToComboBox(Repository.sql.predefinedQueries[i]);
            vQueries.addElement(Repository.sql.predefinedQueries[i]);
        }
        saveDataToFile();
    }

    /**
     * Saves the content of the query combobox to the query file
     */
    protected void saveDataToFile() {
        File queryFile = new File(Repository.files.flQueryFile);
        if (queryFile.exists()) {
            queryFile.delete();
            queryFile = new File(Repository.files.flQueryFile);
        }

        try {
            RandomAccessFile    access  = new RandomAccessFile(queryFile, "rw");
            for (int i = 0; i < comboSQLCommands.getItemCount(); i++) {
                if (comboSQLCommands.getItemAt(i) != null) {
                    String query = comboSQLCommands.getItemAt(i).toString();
                    access.writeBytes(query + '\n');
                }
            }
        } catch (Exception exp) {
            Messages.setMessage(
                Resources.getText(DCMessages.msgFileSaveError, queryFile.toString()), 
                Messages._ERROR);
            Messages.setMessage(exp.getMessage(), Messages._ERROR);
        }
        Messages.setMessage(
            Resources.getText(DCMessages.msgFileSaved, queryFile.toString()), 
            Messages._STANDARD);
    }

    /**
     * Display a status in lower part of the screen
     *
     * @param text  display String
     */
    public void setStatus(String text) {
        labelStatus.setText(text);
    }

    /**
     * Transfers a selected query from the query combobox to the query input field
     */
    protected void setQuery() {
        if (comboSQLCommands.getSelectedItem() != null) {
            textInput.setText(comboSQLCommands.getSelectedItem().toString());
        }
    }

    /**
     * Clears all the values in the panel; the query input field and the result table
     */
    protected void clearPanel() {
        textInput.setText("");
        clearTable();
    }

    /**
     * Clears the table of all values by removing all columns and rows
     */
    public void clearTable() {
        tableModel.setRowCount(0) ;
        tableModel.setColumnCount(0);
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        panelComm       = new JPanel(false);
        panelSQL        = new JPanel(false);
        panelOut        = new JPanel(false);
        panelExtra      = new JPanel(false);
        panelStatus     = new JPanel(false);


        // ************************************************************************
        // STATUS PANEL
        // Set layout style
        GridBagLayout gridbStatus       = new GridBagLayout();
        panelStatus.setLayout(gridbStatus);

        // Initialize components
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));


        // Build the panel
        panelStatus.add(    labelStatus,       new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  0, 0, 0), 0, 0));


        // ************************************************************************
        // SQL PANEL
        // Set layout style
        GridBagLayout       gridbagSQL  = new GridBagLayout();
        panelSQL.setLayout(gridbagSQL);

        // Initialize components
        textInput           = ComponentFactory.getEditorPane();
        textInput.setBackground(ComponentFactory.getTableColor());

        JScrollPane scrollIn = new JScrollPane(textInput);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelSQL.add(    scrollIn,       new GridBagConstraints( 0, 0, 1, 1, 40.0, 40.0
                                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                         new Insets(0, 5, 5, 5), 0, 0));


        // ************************************************************************
        // EXTRA COMMANDS PANEL
        // Set layout style
        GridBagLayout       gridbagExtra    = new GridBagLayout();
        panelExtra.setLayout(gridbagExtra);

        // Initialize components
        labelSQLCommands    = ComponentFactory.getLabel(Resources.getText(DCLabels.lblStoredSQL));
        comboSQLCommands    = ComponentFactory.getComboBox();
        comboSQLCommands.addItemListener(new PredefineQueryAction());
        fillQueryComboBox();

        // Build the panel
        panelExtra.add(    labelSQLCommands, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                            new Insets(5, 10, 15, 10), 0, 0));
        panelExtra.add(    comboSQLCommands, new GridBagConstraints( 1, 0, 1, 1, 1.0, 20.0
                                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 10, 15, 10), 0, 0));


        // ************************************************************************
        // OUTPUT PANEL
        // Set layout style
        GridBagLayout       gridbagOut  = new GridBagLayout();
        panelOut.setLayout(gridbagOut);

        // Initialize components
        table           = ComponentFactory.getSimpleTable(tableModel, true);

        JScrollPane scrollOut = new JScrollPane(table);
        scrollOut.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollOut.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        // Build the panel
        panelOut.add(    scrollOut, new GridBagConstraints( 0, 0, 1, 1, 80.0, 80.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                      new Insets(5, 5, 5, 5), 0, 0));


        // ************************************************************************
        // COMMANDS PANEL
        // Set layout style
        GridBagLayout       gridbagComm = new GridBagLayout();
        panelComm.setLayout(gridbagComm);

        // Initialize components
        buttonRunSql        = ComponentFactory.getButton(Resources.getText(DCLabels.lblRun));
        buttonClear         = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));

        buttonRunSql.setMnemonic(KeyEvent.VK_R);
        buttonClear.setMnemonic(KeyEvent.VK_C);

        buttonRunSql.addActionListener(new QueryButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonRunSql.setToolTipText(Resources.getText(DCTooltips.tpRunSQL));
        buttonClear.setToolTipText(Resources.getText(DCTooltips.tpClear));


        // Build the panel
        panelComm.add(  buttonRunSql, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                      new Insets(0, 5, 0, 5), 0, 0));
        panelComm.add(   buttonClear, new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                      new Insets(0, 0, 0, 5), 0, 0));


        // ************************************************************************
        // MAIN PANEL
        // Set layout style
        GridBagLayout       gridbagMain = new GridBagLayout();
        setLayout(gridbagMain);

        // Initialize components
        panelStatus.setBorder(ComponentFactory.getBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelSQL.setBorder(ComponentFactory.getBorder(
                           Resources.getText(DCLabels.lblQueryIn)));
        panelOut.setBorder(ComponentFactory.getBorder(
                           Resources.getText(DCLabels.lblQueryOut)));

        // Build the panel
        add(  panelSQL,     new GridBagConstraints( 0, 0, 10, 1, 40.0, 40.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelExtra,   new GridBagConstraints( 0, 1, 10, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelOut,     new GridBagConstraints( 0, 2, 10, 1, 80.0, 80.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelComm,    new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelStatus,  new GridBagConstraints( 0, 4, 10, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));


        ToolTipManager.sharedInstance().registerComponent(table);
    }



    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    private class QueryButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            runQuery();
        }
    }

    private class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearPanel();
        }
    }

    private class PredefineQueryAction implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            setQuery();
        }
    }
}





