/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core;

import java.util.ResourceBundle;

import rjw.dc.console.LoginDialog;
import rjw.dc.console.MainFrame;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.resources.DCTooltips;
import rjw.dc.settings.SettingInitializator;
import rjw.dc.util.ModularResourceBundle;
import rjw.dc.util.Resources;

/**
 * DataCrow is the starting point of the application. It is responsible for
 * initializing the UI, starting the logging, initialization of the settings,
 * initializing the database connection and starting the UI.
 * <p>
 * There are no arguments needed for calling the main method, all will be ignored.
 * Input for the program is located in the settings file (Datacrow.properties)
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataCrow {

    private static ComponentFactory  cf;

    /** Starting point of the UI */
    public static MainFrame mainFrame;

    /** Container of all the settings */
    private SettingInitializator settings;
    /** Logging class */
    private Messages messages;

    /**
     * Initializes Data Crow. No arguments are needed. If arguments are given,
     * they will be ignored. The application is started in this sequence;
     * <ul>
     * <li> Logging (Messages)
     * <li> Reading of the settings (SettingsHandler)
     * <li> Database connection initialization (LoginDialog)
     * <li> Initialization of the UI (MainFrame)
     * <li> Showing the UI (MainFrame)
     * </ul>
     * <p>
     */
    public static void main(String[] args) {
        DataCrow start = new DataCrow();

        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCTooltips.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCLabels.class.getName()));
        Resources.addTextResourceBundle((ModularResourceBundle) ResourceBundle.getBundle(DCMessages.class.getName()));

        start.messages = new Messages();
        Messages.setMessage(Resources.getText(DCMessages.msgNewSession0) + '\n' +
                            Resources.getText(DCMessages.msgNewSession1) + '\n' +
                            Resources.getText(DCMessages.msgNewSession2) + '\n' +
                            Resources.getText(DCMessages.msgNewSession3) + '\n',
                            Messages._TABLESS);
        
        cf = new ComponentFactory();

		Messages.setMessage(
            Resources.getText(DCMessages.msgApplicationStarts),
            Messages._STANDARD);
        start.settings = new SettingInitializator();
        Messages.setMessage(Resources.getText(DCMessages.msgSettingsLoaded),
                            Messages._STANDARD);
                            
        LoginDialog dialog = new LoginDialog();
        dialog.setVisible(true);

        while (!LoginDialog.done) {
        }

        try {
            // Start all processes
            start.initializeUI();
        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._STANDARD);
            System.exit(0);
        }
    }

     /**
     * Starts the UI
     */
    private void initializeUI() {
        // Initialize the UI
        mainFrame = new MainFrame();
        mainFrame.setVisible(true);
    }
}
