/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects;

import java.util.Enumeration;
import java.util.Hashtable;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.db.Query;
import rjw.dc.db.QueryBatch;
import rjw.dc.db.QueryWhat;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.util.Utilities;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * A collection of Data Crow objects. Normal actions like
 * save / update and delete can be performed on a collection
 * of objects
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DataCrowCollection {
    
    private Hashtable htObjects;
    private static WorkFlow wf;
    
    public DataCrowCollection() {
        htObjects = new Hashtable();
        if (wf == null) {
            wf = new WorkFlow();
        }
    }
    
    public void addObject(DataCrowObject o, Integer rowNumber) {
        if (o.getValue(DataCrowObject._ID) == null || 
           (o.getValue(DataCrowObject._ID).toString().equals("") ||
            o.getValue(DataCrowObject._ID).toString().equalsIgnoreCase("null"))) {

            o.setValue(DataCrowObject._ID, new Utilities().getUniqueID());
        }
        htObjects.put(rowNumber, o);
    }    
    
    public void saveUpdate(RequestorCollection requestors) {
        wf.updateCollection(this, requestors);

    }
    
    public void delete(RequestorCollection requestors) {
        wf.deleteCollection(this, requestors);   
    }

    public void saveNew(RequestorCollection requestors) {
        wf.insertCollection(this, requestors);
    }
    
    public QueryBatch toQueryBatch(int queryType) {
        QueryBatch batch = new QueryBatch();
        for (Enumeration enum = htObjects.keys(); enum.hasMoreElements(); ) {
            Integer row = (Integer) enum.nextElement();
            DataCrowObject o = (DataCrowObject) htObjects.get(row);
            
            boolean bValid = false;
            if (queryType == Query._INSERT) {
                bValid = o.checkIntegrity(false);
            } else if (queryType == Query._UPDATE) {
                bValid = o.checkIntegrity(true);
            } else if (queryType == Query._DELETE) {
                bValid = true;   
            }
            
            if (bValid && (queryType == Query._DELETE || uniqueObject(o, row))) {
                QueryWhat what = new QueryWhat(o.getFieldNames(),
                                               o.getTableName());
                Query query = new Query(what, 
                                        o.toQueryCondition(false),
                                        null,
                                        queryType,
                                        new RequestorCollection(),
                                        o.getValue(DataCrowObject._ID).toString(),
                                        row.intValue());
                try {
                    batch.addQuery(query);
                } catch (Exception ignore) {}            
            } else {
                return null;
            }
        }
        return batch;
    }
    
    
    public boolean uniqueObject(DataCrowObject o, Integer newRow) {
        if (Settings.getValueAsBoolean(Repository.settings.stCheckUniqueness)) {
            int[] requiredFields = o.getRequiredFields();
            Integer oldRow = null;
            
            for (Enumeration enum = htObjects.keys(); enum.hasMoreElements(); ) {
                oldRow = (Integer) enum.nextElement();
                DataCrowObject dcTemp = (DataCrowObject) htObjects.get(oldRow);
                
                if (!dcTemp.equals(o)) {
                    
                    boolean same = true;
                    for (int k = 0; k < requiredFields.length; k++) {
                        if (!dcTemp.getValue(requiredFields[k]).equals(
                            o.getValue(requiredFields[k]))) {
                            same = false;  
                        }
                    }

                    if (same) {
                        new MessageBox(Resources.getText(DCMessages.msgRowIsSameAsRow, 
                                       new String[] {newRow.toString(), oldRow.toString()}), 
                                       true, MessageBox._ERROR); 
                        return false;
                    }
                }
            }
        }          
        return true;
    }    
}
