/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.movie;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Movie extends DataCrowObject {

    public static final int _A_TITLE = 1;
    public static final int _B_DESCRIPTION = 2;
    public static final int _C_WEBPAGE = 3;
    public static final int _D_GENRE = 4;
    public static final int _E_ACTORS = 5;
    public static final int _F_DIRECTOR = 6;
    public static final int _G_COUNTRY = 7;
    public static final int _H_YEAR = 8;
    public static final int _I_PLAYLENGTH = 9;
    public static final int _J_FILENAME = 10;
    public static final int _K_VIDEOCODEC = 11;
    public static final int _L_AUDIOCODEC = 12;
    public static final int _M_WIDTH = 13;
    public static final int _N_HEIGHT = 14;
    public static final int _O_FPS = 15;
    public static final int _P_FRAMES = 16;
    public static final int _Q_AUDIOBITRATE = 17;
    public static final int _R_AUDIOSAMPLINGRATE = 18;
    public static final int _S_AUDIOCHANNEL = 19;
    public static final int _T_VIDEOBITRATE = 20;
    public static final int _U_LOCATION = 21;

    private static WorkFlow wf;

    /**
     * Initializes the Movie object
     */
    public Movie() {
       super();
    }

    /**
     * Applies values to this Movie record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;

        try {
            if (row.length < Repository.dataBase.dbMovieColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbMovieColArrayPK.length) {
                if (alwaysGenerateID) {
                    i++; 
                } else {
                    setValue(_ID, row[i++]);
                }
            }


            setValue(_A_TITLE, row[i++]);
            setValue(_B_DESCRIPTION, row[i++]);
            setValue(_C_WEBPAGE, row[i++]);
            setValue(_D_GENRE, row[i++]);
            setValue(_E_ACTORS, row[i++]);
            setValue(_F_DIRECTOR, row[i++]);
            setValue(_G_COUNTRY, row[i++]);
            setValue(_H_YEAR, row[i++]);
            setValue(_I_PLAYLENGTH, row[i++]);
            setValue(_J_FILENAME, row[i++]);
            setValue(_K_VIDEOCODEC, row[i++]);
            setValue(_L_AUDIOCODEC, row[i++]);
            setValue(_M_WIDTH, row[i++]);
            setValue(_N_HEIGHT, row[i++]);
            setValue(_O_FPS, row[i++]);
            setValue(_P_FRAMES, row[i++]);
            setValue(_Q_AUDIOBITRATE, row[i++]);
            setValue(_R_AUDIOSAMPLINGRATE, row[i++]);
            setValue(_S_AUDIOCHANNEL, row[i++]);
            setValue(_T_VIDEOBITRATE, row[i++]);
            setValue(_U_LOCATION, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }


    // =========================================================================
    // INITIALIZATION
    // =========================================================================
    protected void setFieldTypes() {
        fieldTypes = Repository.objects.movieFieldTypes;
    }
    
    protected void setFieldNames() {
        fieldNames = Repository.dataBase.dbMovieColArrayPK;
    }
    
    protected void setLabels() {
        labels = new String[Repository.dataBase.dbMovieColArrayPK.length];
        int i = 0;
        labels[i++] = Resources.getText(DCLabels.lblID);
        labels[i++] = Resources.getText(DCLabels.lblTitle);
        labels[i++] = Resources.getText(DCLabels.lblDescription);
        labels[i++] = Resources.getText(DCLabels.lblWebPage);
        labels[i++] = Resources.getText(DCLabels.lblGenre);
        labels[i++] = Resources.getText(DCLabels.lblActors);
        labels[i++] = Resources.getText(DCLabels.lblDirector);
        labels[i++] = Resources.getText(DCLabels.lblCountry);
        labels[i++] = Resources.getText(DCLabels.lblYear);
        labels[i++] = Resources.getText(DCLabels.lblPlayLength);
        labels[i++] = Resources.getText(DCLabels.lblFileName);
        labels[i++] = Resources.getText(DCLabels.lblVideoCodec);
        labels[i++] = Resources.getText(DCLabels.lblAudioCodec);
        labels[i++] = Resources.getText(DCLabels.lblWidth);
        labels[i++] = Resources.getText(DCLabels.lblHeight);
        labels[i++] = Resources.getText(DCLabels.lblFPS);
        labels[i++] = Resources.getText(DCLabels.lblFrames);
        labels[i++] = Resources.getText(DCLabels.lblAudioBitRate);
        labels[i++] = Resources.getText(DCLabels.lblAudioSampleRate);
        labels[i++] = Resources.getText(DCLabels.lblAudioChannel);
        labels[i++] = Resources.getText(DCLabels.lblVideoBitRate);
        labels[i++] = Resources.getText(DCLabels.lblLocation);
    }
    
    protected void setModuleName() {
        moduleName = Resources.getText(DCLabels.lblMovie);    
    }
    
    protected void setTableName() {
        sTableName = Repository.dataBase.dBMovieTableName.trim();
    }
    
    protected void setRequiredFields() {
        requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stMovieRequiredFields);
    }

    protected void setHiddenFields() {
        hiddenFields = Settings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsMovie);   
    }
    
    protected void setValueTypes() {
        valueTypes = Repository.objects.movieValueTypes;
    }
    
    protected void setMaxFieldLengths() {
        maxFieldLength = Repository.objects.movieFieldLengths;        
    }

    protected void initValues() {
        values = new Object[Repository.dataBase.dbMovieColArrayPK.length];
        for (int j = 0; j < values.length; j++) {
           setValue(j, new String(""));
        }        
    }
    
    protected void setODB() {
        amazon = false;
        imdb = true;
    }
}
