/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.software;

import java.sql.ResultSet;
import java.util.Enumeration;
import java.util.Vector;

import rjw.dc.console.MessageBox;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.db.Query;
import rjw.dc.db.QueryCondition;
import rjw.dc.db.QueryOptions;
import rjw.dc.db.QueryWhat;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;

/**
 * Collection of Software Catagories
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SoftwareCatagories {

    /** Holder of the catagories objects */
    private static Vector vCatagories;
    private static WorkFlow wf;

    /**
     * Initializes the Catagory collection. Loading of the Catagories is not
     * done here
     */
    public SoftwareCatagories() {
        if (wf == null) {
            wf = new WorkFlow();
        }
    }

    /**
     * loads the catagories DIRECTLY from the database, the queuing
     * does not apply! Use with care. For now this is only used for
     * loading the first time
     */
    public static void load() {
        try {
            SoftwareCatagory catagory = new SoftwareCatagory();

            QueryOptions options = new QueryOptions("2", false, false);
            QueryCondition conditions = catagory.toQueryCondition(true);
            QueryWhat what = new QueryWhat(catagory.getFieldNames(), catagory.getTableName());
            Query query = new Query(what, conditions, options, Query._SELECT, null, null);

            ResultSet result = (ResultSet) wf.runQueryDirect(query);

            vCatagories = WorkFlow.createDataCrowObjectArray(result);
        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }

    /**
     * Reloads the catagories from the database (queueing applies)
     */
    public static void refresh() {
        vCatagories = null;

        SoftwareCatagory catagory = new SoftwareCatagory();
        QueryOptions options = new QueryOptions("2", false, false);

        Requestor requestor = new Requestor(catagory.getClass(), "setCatagories", null);
        RequestorCollection requestors = new RequestorCollection();
        requestors.addRequestor(requestor);

        catagory.load(options, requestors);
    }

    /**
     * Fills the catagory vector
     */
    public static void setCatagories(Vector vValues) {
        vCatagories = vValues;
    }

    /**
     * Returns the name of a Catagory with the given ID
     *
     * @param sID           database ID
     * @return (String)     Catagory name
     */
    public static String getCatagoryName(String sID) {
        for (Enumeration enum = vCatagories.elements(); enum.hasMoreElements(); ) {
            SoftwareCatagory cat = (SoftwareCatagory) enum.nextElement();
            if (sID.equals(cat.getValue(SoftwareCatagory._ID))) {
                return cat.getValue(SoftwareCatagory._A_NAME).toString();
            }
        }
        return "Null";
    }

    /**
     * Returns the cached Catagorie objects
     */
    public static Vector getCatagories() {
        if (vCatagories == null) {
            new MessageBox(
                Resources.getText(DCMessages.msgUnfreshCatagories), 
                true, MessageBox._INFORMATION);
        }

        return vCatagories;
    }

    /**
     * Returns the ID of a Catagory by the name
     *
     * @param sName     Catagory name
     * @return (String) ID
     */
    public static String getCatagoryID(String sName) {
        for (Enumeration enum = vCatagories.elements(); enum.hasMoreElements(); ) {
            SoftwareCatagory cat = (SoftwareCatagory) enum.nextElement();
            if (sName.equals(cat.getValue(SoftwareCatagory._A_NAME))) {
                return cat.getValue(SoftwareCatagory._ID).toString();
            }
        }
        return "Null";
    }
}
