/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.core.objects.software;

import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * Representation of a Software Catagory.
 * Allows saving, updating and querying of Catagory records
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SoftwareCatagory extends DataCrowObject {

    /** Catagory name */
    public static final int _A_NAME = 1;

    private static WorkFlow wf;

    /**
     * Initializes the Catagory object; sets the required fields, the table name,
     * initializes the value array and the database field names
     */
    public SoftwareCatagory() {
        super();
        
        if (wf == null) {
            wf = new WorkFlow();
            SoftwareCatagories catagories = new SoftwareCatagories();
            SoftwareCatagories.load();
            catagories = null;
        }
    }

    /**
     * Applies values to this Catagory record. The first in the row is the ID
     *
     * @param row   representation of an database row. First in row is the ID
     */
    public void setValues(Object[] row) {
        int i = 0;
        try {
            if (row.length < Repository.dataBase.dbCatColArray.length) {
                throw new Exception(Resources.getText(DCMessages.msgInvalidArray));
            } else if (row.length == Repository.dataBase.dbCatColArrayPK.length) {
                setValue(_ID, row[i++]);
            }
            setValue(_A_NAME, row[i++]);

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
        }
    }
    
    // =========================================================================
    // INITIALIZATION
    // =========================================================================
    protected void setFieldTypes() {
        fieldTypes = Repository.objects.softwareCatFieldTypes;
    }
    
    protected void setFieldNames() {
        fieldNames = Repository.dataBase.dbCatColArrayPK;
    }
    
    protected void setLabels() {
        labels = new String[Repository.dataBase.dbCatColArrayPK.length];
        int i = 0;
        labels[i++] = Resources.getText(DCLabels.lblID);
        labels[i++] = Resources.getText(DCLabels.lblName);
    }
    
    protected void setModuleName() {
        // TODO: add label
        moduleName = "Software Catagory";    
    }
    
    protected void setTableName() {
        sTableName = Repository.dataBase.dBCatTableName.trim();
    }
    
    protected void setRequiredFields() {
        requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stSoftwareCatRequiredFields);
    }

    protected void setHiddenFields() {
        hiddenFields = Settings.getValueAsIntegerArray(
                       Repository.settings.stHiddenFieldsSoftwareCat);   
    }
    
    protected void setValueTypes() {
        valueTypes = Repository.objects.softwareCatValueTypes;
    }
    
    protected void setMaxFieldLengths() {
        maxFieldLength = Repository.objects.softwareCatFieldLengths;        
    }

    protected void initValues() {
        values = new Object[Repository.dataBase.dbCatColArrayPK.length];
        for (int j = 0; j < values.length; j++) {
           setValue(j, new String(""));
        }        
    }
    
    protected void setODB() {
        amazon = false;
        imdb = false;
    }      
}
