/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.db;

import java.sql.Connection;
import java.sql.DriverManager;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.flex.settings.Settings;

/**
 * Maintains and creates the connection to the database
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class DatabaseConnection {

    /** Specifies whether a connection has been made to the database */
	public static boolean bConnected = false;
    public static boolean bServerConnection = false;
	private static MessagesWriter logWriter = new MessagesWriter();

    /**
     * Creates the connection to the database. If this fails, the connected boolean
     * is left on false. Errors are shown to the user via a messagebox
     */
    public static Connection connect() {
    	try {
            Class.forName(Settings.getValueAsString(Repository.settings.stDatabaseDriver));
            
            String sName = Settings.getValueAsString(Repository.settings.stConnectionString);
            
            String sConnectionString = Repository.dataBase.localString + sName;
            if (sName.startsWith("//")) {
                sConnectionString = Repository.dataBase.serverString + sName;
                bServerConnection = true;
            }
            
            return DriverManager.getConnection(sConnectionString, "sa", "");
    	} catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
            new MessageBox(exp.getMessage(),
                           true, Messages._ERROR);
            while (MessageBox.getVisible()) {}
            return null;
    	}
    }
}
