/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.db;

import java.util.LinkedList;
import rjw.dc.core.objects.RequestorCollection;

/**
 * A collection of actions to be performed on the database
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class QueryBatch {

    private LinkedList lQueries;
    private RequestorCollection requestors = new RequestorCollection();
    
    public QueryBatch() {
        lQueries = new LinkedList();
    }

    public void setRequestors(RequestorCollection requestors) {
        this.requestors = requestors;
    }

    public RequestorCollection getRequestors() {
        return requestors;
    }
    
    public void addQuery(Query query) throws Exception {
        if (query.getQueryType() != Query._INSERT &&
            query.getQueryType() != Query._UPDATE &&
            query.getQueryType() != Query._DELETE) {
            throw new Exception("Query type not allowed!");        
        } else {
            lQueries.addLast(query);
        }
    }    
    
    public Query[] getQueries() {
        Query[] queries = new Query[lQueries.size()];
        
        int i = 0;
        while (!lQueries.isEmpty()) {
            queries[i] = (Query) lQueries.removeFirst();
            i++;
        }
        return queries;
    }
}