
package rjw.dc.export;

import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Enumeration;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.table.TableColumn;

import rjw.dc.console.BrowserDialog;
import rjw.dc.console.MessageBox;
import rjw.dc.console.components.ComponentFactory;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.DataCrowObject;
import rjw.dc.resources.DCLabels;
import rjw.dc.util.Resources;
import rjw.dc.util.StringUtils;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

import com.lowagie.text.Cell;
import com.lowagie.text.Chapter;
import com.lowagie.text.Document;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.HeaderFooter;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Phrase;
import com.lowagie.text.Table;
import com.lowagie.text.html.HtmlWriter;
import com.lowagie.text.pdf.PdfWriter;

public class TableToFileExporter {
    
    public static void exportTableToPDF(JComponent comp, 
                                        JTable table, 
                                        int[] headerFields) {
        exportToFile(true, false, comp, table, headerFields);
    }

    public static void exportTableToHTML(JComponent comp, 
                                         JTable table,
                                         int[] headerFields) {
        exportToFile(false, true, comp, table, headerFields);
    }
    
    public static void exportTableToText(JComponent comp, JTable table) {
        final JTable tableToExport = table;
        BrowserDialog dialog = new BrowserDialog(Resources.getText(
                                                 DCLabels.lblSelectExportFile));
        
        final File file = dialog.showCreateFileDialog(comp);
        if (file == null) {
            return;   
        }

        try {
            Thread thread = new Thread(new Runnable() {
                public void run() {
                    FileOutputStream out = null;
                    try {
                        out = new FileOutputStream(file);
                
                        
                        for (int j = 0; j < tableToExport.getRowCount(); j++) {        
                            StringBuffer line = new StringBuffer("");
                            line.append("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
                            for (int i = 0; i < tableToExport.getColumnCount(); i++) {
                                line.append("\r\n");
                                
                                if (i > 0) {
                                    line.append("\r\n\r\n");
                                }

                                line.append("#");
                                String field = tableToExport.getTableHeader().getColumnModel().getColumn(i).getHeaderValue().toString();
                                
                                line.append(field);
                                line.append("\r\n-----------------------------\r\n");
                                String sVal = tableToExport.getValueAt(j, i).toString();
                                sVal = StringUtils.replaceAll(sVal, "\r", " ");
                                sVal = StringUtils.replaceAll(sVal, "\n", " ");
                                line.append(sVal);
                            }
                            line.append("\r\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\r\n\r\n\r\n\r\n");                            
                            out.write(line.toString().getBytes());
                        }
                    } catch (Exception exp) {
                        new MessageBox(exp.toString(), true, MessageBox._ERROR); 
                        return;
                    } finally {
                        if (out != null) {
                            try {
                                out.close();
                            } catch (Exception exp) {}
                        }
                    }
                    new MessageBox(Resources.getText(DCLabels.lblExportHasFinished), 
                                   true, MessageBox._INFORMATION);
                }
            });
            thread.setPriority(5);
            thread.start();
        } catch (Exception exp) {
            new MessageBox(exp.toString(), true, MessageBox._ERROR); 
        } 
    }

    private static HeaderFooter getPdfFooter() {
        return new HeaderFooter(new Phrase(Resources.getText(
                                DCLabels.lblPage) + " "), 
                                new Phrase("."));
    }
    
    private static Image getPdfFront() throws Exception {
        Image img = Image.getInstance(new ImageIcon("./Images/logo.jpg").getImage(), null, false);
        return img;        
    }
    
    private static void exportToFile(boolean pdf, 
                                     boolean html, 
                                     JComponent comp,
                                     JTable table,
                                     int[] headers) {
                                         
        final boolean HTML = html;
        final DataCrowObject o = WorkFlow.getObject(Settings.getValueAsInt(
                                 Repository.settings.stModule));
        final String moduleName = o.getModuleName();    
        final JTable tableToExport = table;            
        final int[] headerFields = headers;                 
        
        
        String fileExtension = html ? "html" : "pdf";
        
        BrowserDialog dialog = new BrowserDialog("Select export file", 
                                                 fileExtension);
        final File file = dialog.showCreateFileDialog(comp);
        if (file == null) {
            return;   
        }
        
        Thread thread = new Thread(new Runnable() {
            public void run() {
                try {
                    Document document = new Document();
                    document.addCreator("Data Crow");
                    document.addAuthor("Robert-Jan van der Waals");
                    document.addTitle(Resources.getText(DCLabels.lblOverview, moduleName));

                    FileOutputStream out = new FileOutputStream(file);
                    
                    if (HTML) {
                        HtmlWriter writer = HtmlWriter.getInstance(document, out);
                    } else {
                        PdfWriter writer = PdfWriter.getInstance(document, out);
                    }
                    
                    document.open();
                    document.setFooter(getPdfFooter());
                    HeaderFooter header = new HeaderFooter(new Phrase(Resources.getText(DCLabels.lblOverview, moduleName)), false);
                    document.setHeader(header);
                    document.add(getPdfFront());
                    document.newPage();
            
                    for (int j = 0; j < tableToExport.getRowCount(); j++) {        
                        Table iTable = new Table(3);
                        iTable.setBorderWidth(2); 
                        iTable.setBorderColor(new Color(0, 0, 0)); 
                        iTable.setSpaceBetweenCells(5);
                        iTable.setSpaceInsideCell(5);
                        iTable.setSpacing(5);
        
                        String[] values = new String[tableToExport.getColumnCount()];
                        for (int i = 0; i < values.length; i++) {
                            values[i] = tableToExport.getValueAt(j, i).toString();    
                        }
                        
                        String sTabelHeader = "";
                        for (int i = 0; i < headerFields.length; i++) {
                            if (i == 0) {
                                sTabelHeader += tableToExport.getModel().
                                                getValueAt(j, headerFields[i]); 
                            } else {
                                sTabelHeader += " (" + 
                                                tableToExport.getModel().
                                                getValueAt(j, headerFields[i]) +
                                                ") ";
                            }
                        }
                        
                        Cell cellHeader = new Cell(sTabelHeader);
                        cellHeader.setColspan(3);
                        cellHeader.setHeader(true);
                        cellHeader.setBackgroundColor(new Color(0xC0, 0xC0, 0xC0));
                        iTable.addCell(cellHeader);
                        
                        int counter = 0;
                        for (Enumeration enum = tableToExport.getColumnModel().getColumns();
                             enum.hasMoreElements(); ) {
                            
                            TableColumn column = (TableColumn) enum.nextElement();

                            int modelIndex = tableToExport.convertColumnIndexToModel(counter);
                            
                            boolean usedAsHeader = false;
                            for (int i = 0; i < headerFields.length; i++) {
                                if (modelIndex == headerFields[i]) {
                                    usedAsHeader = true;   
                                }
                            }
                            
                            if (!usedAsHeader) {
                                int fieldType = o.getFieldType(modelIndex);
                                
                                Cell cellField = new Cell(o.getFieldName(modelIndex));
                                Cell cellValue = new Cell(values[counter]);
                                if (fieldType == ComponentFactory._LONGTEXTFIELD) {
                                    cellField.setColspan(3);
                                    cellField.setBorder(0);
                                    iTable.addCell(cellField);
                                    cellValue.setColspan(3);
                                    iTable.addCell(cellValue);
                                } else {
                                    cellField.setBorder(0);
                                    iTable.addCell(cellField);
                                    cellValue.setColspan(2);
                                    iTable.addCell(cellValue);
                                }
                            }
                            counter++;
                        }
  

                        Font chapterFont = FontFactory.getFont(FontFactory.COURIER, 10, Font.NORMAL, new Color(255, 255, 255));
                        Paragraph cTitle = new Paragraph(tableToExport.getModel().getValueAt(j, headerFields[1]).toString(), 
                                                         chapterFont);
                        Chapter chapter = new Chapter(cTitle, j + 1);
                        
                        chapter.add(iTable);            
                        document.add(chapter);
                        document.newPage();   
                    }
                    document.close();
                } catch (Exception exp) {
                    new MessageBox(exp.toString(), true, MessageBox._ERROR);
                    return;
                } 
                new MessageBox(Resources.getText(DCLabels.lblExportHasFinished), 
                               true, MessageBox._INFORMATION);
            }
        });
        thread.setPriority(5);
        thread.start();
    }
    
}
