/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.http;

import java.util.Vector;

import rjw.dc.console.MessageBox;
import rjw.dc.util.StringUtils;

/**
 * Searches for software on the Amazon site
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 0.1
 */
public class AmazonQuery {

    private String searchString = "";

    public AmazonQuery(String searchString) {
        this.searchString = searchString;
    }

    /**
     * Searches on the Amazon.com website on the catagories software and
     * videogames.
     *
     * @param connection the internet connection
     */
    public Vector search(HttpConnection connection) {
        Vector v = new Vector(0);

        searchString = StringUtils.replaceAll(searchString, " ", "%20");
        try {
            String sWebPage1 = connection.get("/onca/xml3?t=webservices-20&dev-t=test&KeywordSearch=" +
                                             searchString +
                                             "&mode=videogames&type=heavy&page=1&f=xml");
            String sWebPage2 = connection.get("/onca/xml3?t=webservices-20&dev-t=test&KeywordSearch=" +
                                             searchString +
                                             "&mode=software&type=heavy&page=1&f=xml");
            String sWebPage = sWebPage1 + sWebPage2;

            connection.close();
            v = split(sWebPage);
        } catch (Exception e) {
            new MessageBox(e.toString(), true, MessageBox._ERROR);
        }
        return v;
    }

    /**
     * Parses information from a part of an xml file
     *
     * @param rawResult the raw input
     * @return Object[0] the name of the software
     *         Object[1] the description
     */
    public Object[] parseInformation(String rawResult) {
        Vector v = new Vector(0);
        int indexBegin = 0;
        int indexEnd = 0;

        String productName = "";
        indexBegin = rawResult.indexOf("<ProductName>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = rawResult.indexOf("</ProductName>");
            productName = rawResult.substring(indexBegin, indexEnd);
        }
        
        String amazonUrl = "";
        indexBegin = rawResult.indexOf("url=");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 5;
            indexEnd = rawResult.indexOf(">") - 1;
            amazonUrl = rawResult.substring(indexBegin, indexEnd);
        }        

        v.add(productName);
        v.add(getDescription(rawResult));
        v.add(amazonUrl);

        Object[] o = new Object[v.size()];
        v.toArray(o);
        return o;
    }

    /**
     * Creates a description out of several pieces of information
     *
     * @param rawResult the raw input string
     */
    private String getDescription(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;

        String description = "";
        indexBegin = rawResult.indexOf("<ProductDescription>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 20;
            indexEnd = rawResult.indexOf("</ProductDescription>");
            description = rawResult.substring(indexBegin, indexEnd);

            indexBegin = description.indexOf("&lt;/I>");
            if (indexBegin > -1) {

            }
            description = "\n" + "\n" + "Description: " + "\n" + description;
        }

        String manufacturer = "";
        indexBegin = rawResult.indexOf("<Manufacturer>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 14;
            indexEnd = rawResult.indexOf("</Manufacturer>");
            manufacturer = rawResult.substring(indexBegin, indexEnd);
            manufacturer = "Manufactured by: " + manufacturer;
        }

        String platforms = "";
        indexBegin = rawResult.indexOf("<Platforms>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 11;
            indexEnd = rawResult.indexOf("</Platforms>");
            platforms = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = platforms.indexOf("<Platform>");
            while(indexBegin > -1) {
                indexBegin = indexBegin + 10;
                indexEnd = platforms.indexOf("</Platform>");

                temp = temp + platforms.substring(indexBegin, indexEnd) + " ";

                indexEnd = indexEnd + 11;
                platforms = platforms.substring(indexEnd, platforms.length());
                indexBegin = platforms.indexOf("<Platform>");
            }
            platforms = temp;
            platforms = "\n" + "\n" + "Supported platforms: " + "\n" + platforms;
        }

        String features = "";
        indexBegin = rawResult.indexOf("<Features>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 10;
            indexEnd = rawResult.indexOf("</Features>");
            features = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = features.indexOf("<Feature>");
            while(indexBegin > -1) {
                indexBegin = indexBegin + 9;
                indexEnd = features.indexOf("</Feature>");

                temp = temp + features.substring(indexBegin, indexEnd) + ". ";

                indexEnd = indexEnd + 10;
                features = features.substring(indexEnd, features.length());
                indexBegin = features.indexOf("<Feature>");
            }
            features = temp;
            features = "\n" + "\n" + "Features: " + "\n" + features;
        }

        String releaseDate = "";
        indexBegin = rawResult.indexOf("<ReleaseDate>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = rawResult.indexOf("</ReleaseDate>");
            releaseDate = rawResult.substring(indexBegin, indexEnd);
            releaseDate = "\n" + "Released on: " + releaseDate;
        }

        description = manufacturer +
                      releaseDate +
                      description +
                      features +
                      platforms;

        return clean(description);
    }

    /**
     * Splits one search result in to pieces. Every piece (String)
     * is a presentation of one record (raw data)
     *
     * @param rawResult
     * @return
     */
    private Vector split(String rawResult) {
        Vector v = new Vector(0);

        int index = rawResult.indexOf("<Details");
        while (index > -1) {
            int end = rawResult.indexOf("</Details>") + 10;
            if (end != -1) {
                String part = rawResult.substring(index, end);
                //System.out.println(part);
                v.add(part);
                rawResult = rawResult.substring(end, rawResult.length());
            }
            index = rawResult.indexOf("<Details");
        }
        return v;
    }

    /**
     * Removes unwanted characters and/or replaces them with another
     *
     * @param rawString a raw input string
     */
    private String clean(String rawString) {
        rawString = StringUtils.replaceAll(rawString, "&lt;I>",     "");
        rawString = StringUtils.replaceAll(rawString, "&lt;/I>",    "");
        rawString = StringUtils.replaceAll(rawString, "&lt;p>",     "");
        rawString = StringUtils.replaceAll(rawString, "&lt;/p>",     "");
        rawString = StringUtils.replaceAll(rawString, "&lt;/b>",    "");
        rawString = StringUtils.replaceAll(rawString, "&lt;b>",     "\n\n");
        rawString = StringUtils.replaceAll(rawString, "&lt;li>",    "\n");
        rawString = StringUtils.replaceAll(rawString, "&lt;ul>",    "\n");
        rawString = StringUtils.replaceAll(rawString, "&lt;/ul>",   "");
        rawString = StringUtils.replaceAll(rawString, "&lt;ul>",    "\n");
        rawString = StringUtils.replaceAll(rawString, "&lt;i>",   "");
        rawString = StringUtils.replaceAll(rawString, "&lt;B>",   "");
        rawString = StringUtils.replaceAll(rawString, "&lt;/B>",   "");
        rawString = StringUtils.replaceAll(rawString, "&lt;/i>",   "");
        rawString = StringUtils.replaceAll(rawString, "&amp;#146;",   "`");
        rawString = StringUtils.replaceAll(rawString, "amp;",   "");
        
        return rawString;
    }
}
