/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.http;

import http.client.HTTPConnection;
import http.client.HTTPResponse;
import http.client.NVPair;

import java.util.Hashtable;

import rjw.dc.core.Repository;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.flex.settings.Settings;

/**
 * Creates a connection to a webserver using http
 *
 * @author Robert-Jan van der Waals
 * @since 1.2
 * @version 3.03
 */
public class HttpConnection {

    private String HOST = "";
    private HTTPConnection connection;
    private HTTPResponse oResponse;
    private NVPair[] oForm;
    private Hashtable oForm_ = new Hashtable();

    /**
     * Creates a connection to a webserver
     * 
     * @param sHost the URL of the webserver
     */
    public HttpConnection(String sHost) throws Exception {
        createConnection(sHost);
    }

    /**
     * Closes the internet connection
     */
    public void close() {
        connection.stop();
    }

    /**
     * Create a http connection. If a proxy has been specified in
     * the settings it will be used to create the connection
     *
     * @param sHost the url
     */
    public void createConnection(String sHost) throws Exception {
        String sProxy = Settings.getValueAsString(Repository.settings.stProxyServerName);
        int iProxyPort = Settings.getValueAsInt(Repository.settings.stProxyServerPort);
        String sUserAgent = Settings.getValueAsString(Repository.settings.stHTTPUserAgent);

        HOST = (sHost.startsWith("http://") ? "" : "http://") + sHost;
        try {
            HTTPConnection.setProxyServer(sProxy, iProxyPort);
            connection = new HTTPConnection(sHost);
            connection.setDefaultHeaders( new NVPair[] {
                                             new NVPair(
                                               "User-Agent",
                                                sUserAgent)});
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }
    }

    public void addQuery (String sName, String sValue) {
        oForm_.put(sName, sValue);
    }

    public void clearQueryBuffer () {
        oForm_ = new Hashtable();
    }

    public String post(String sFile) throws Exception {
        return new String(get(sFile, true));
    }

    public String get(String sFile) throws Exception {
        return new String(get(sFile, false));
    }

    /**
     * Retrieves information from the host
     * 
     * @param sFile the address from which data should be read
     * @param bPost post query ?
     * @throws Exception
     */
    private byte[] get(String sFile, boolean bPost) throws Exception {
        try {
            if (bPost) {
                oResponse = connection.Post(sFile, oForm);
            } else {
                oResponse = connection.Get(sFile);
            }

            if (oResponse.getStatusCode() >= 300) {
                throw new Exception("");
            }

            byte[] baRetVal = oResponse.getData();
            return baRetVal;
        } catch (Exception e) {
            throw new Exception(Resources.getText(DCMessages.msgCouldNotAccess,
                                                  HOST + sFile));
        }
    }
}
