/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.http;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import rjw.dc.console.MessageBox;
import rjw.dc.core.Repository;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;
import rjw.dc.util.StringUtils;
import rjw.flex.settings.Settings;

/**
 * Access www.imdb.com to gather movie specific information
 *
 * @author Robert-Jan van der Waals
 * @since 1.2
 * @version 3.03
 */
public class IMDBQuery {

    private Hashtable htData;

    public static final Object _ID = "Id";
    public static final Object _URL = "Url";
    public static final Object _TITLE = "Title";
    public static final Object _YEAR = "Year";
    public static final Object _DIRECTOR = "Director";
    public static final Object _WRITER = "Writer";
    public static final Object _GENRE = "Genre";
    public static final Object _TAGLINE = "Tagline";
    public static final Object _TAGLINEURL = "TaglineUrl";
    public static final Object _PLOT = "Plot";
    public static final Object _PLOTURL = "PlotUrl";
    public static final Object _RATING = "Rating";
    public static final Object _VOTES = "Votes";
    public static final Object _PLACE = "Place";
    public static final Object _ACTORS = "Actors";
    public static final Object _COUNTRY = "Country";
    public static final Object _RUNTIME = "Runtime";

    private String searchString;
    private String searchUrl;

    /**
     * initializes the IMDB query
     *
     * @param searchString the string on which will be searched
     */
    public IMDBQuery(String searchString) throws Exception {
        htData = new Hashtable(0);
        this.searchString = searchString.replace(' ', '+');
    } 

    /**
     * Searches for the maching ID's from the search result(s)
     */
    public String[] getIDs(HttpConnection connection) throws Exception {
        Vector vKeys = new Vector(0);

        searchString = StringUtils.replaceAll(searchString, " ", "%20");
        this.searchUrl = "/Find?" + searchString;
        String sWebPage = connection.get(searchUrl);

        String sTemp = sWebPage;
        if (sTemp.indexOf("/title/tt") > -1) {
            int temp = 0;
            while (temp != -1) {
                temp = sTemp.indexOf("/title/tt");
                if (temp != -1) {
                    sTemp = sTemp.substring(temp + 9, sTemp.length());
                    vKeys.add(sTemp.substring(0, 7));
                }
            }
        }

        if (vKeys.size() < 1) {
            new MessageBox(Resources.getText(DCMessages.msgNoSearchResults), true,
                           MessageBox._INFORMATION);
            return null;
        } else {
            String[] keys = new String[vKeys.size()];
            int i = 0;
            for (Enumeration enum = vKeys.elements(); enum.hasMoreElements(); i++) {
                keys[i] = enum.nextElement().toString();
            }
            return keys;
        }
    }

    /**
     * Reads the information from www.imdb.com
     *
     * @param sID   the movie ID from www.imdb.com
     */
    public void fillImdbData(String sID, HttpConnection connection) throws Exception {
        htData = new Hashtable(0);

        // set some things/vars we already know
        htData.put(_URL, "http://www.imdb.com/Title?" + sID);
        htData.put(_ID, sID);
        htData.put(_TAGLINEURL, "http://www.imdb.com/Taglines?" + sID);
        htData.put(_PLOTURL, "http://www.imdb.com/Plot?" + sID);

        // temp vars used while gathering the info from imdb
        int temp = 0;
        int temp2 = 0;
        String sTemp, sTemp2 = null;

        try {
            if (connection != null) {
                // start connection to web server
                String sODB = Settings.getValueAsString(Repository.settings.stODB);
                connection = new HttpConnection(sODB);
            }
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }
        String sWebPage = "";
        try {
            // save the imdb movie webpage in sWebPage
            sWebPage = connection.get("/Title?" + sID);
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }

        // gather the information from sWebPage
        // first we get the title of the movie and the year from the titel tag
        // from the web page
        sTemp =
            sWebPage.substring(
                sWebPage.indexOf("<title>"),
                sWebPage.indexOf("</title>"));
        htData.put(_TITLE, trim(sTemp.substring(7, sTemp.indexOf("(") - 1).trim()));
        htData.put(_YEAR, sTemp.substring(sTemp.indexOf("(") + 1,
                                          sTemp.indexOf(")")));

        temp = sWebPage.indexOf("Directed by");
        if (temp != -1) {
            sWebPage = sWebPage.substring(temp);
            temp = sWebPage.indexOf("Writing credits");
            if (temp != -1) {
                sTemp2 = sWebPage.substring(temp, sWebPage.indexOf("<br clear=\"left\">"));
                sTemp = sWebPage.substring(0, temp - 10);
            } else {
                sTemp = sWebPage.substring(0, sWebPage.indexOf("<br clear=\"left\">"));
            }
            temp = sTemp.indexOf("(more)");
            if (temp != -1) {
                sTemp = sTemp.substring(0, temp - 10);
            }

            temp = sTemp.indexOf("\">");
            String director = "";
            while (temp != -1) {
                sTemp = sTemp.substring(temp + 2);
                temp = sTemp.indexOf("\">");
                director += sTemp.substring(0, sTemp.indexOf("<")) + ",";
            }
            htData.put(_DIRECTOR, trim(director.substring(0, director.length() - 1)));
        }

        if (sTemp2 == null) {
            temp = sWebPage.indexOf("Writing credits");
            if (temp != -1) {
                sWebPage = sWebPage.substring(temp);
                sTemp2 = sWebPage.substring(0,
                         sWebPage.indexOf("<br clear=\"left\">"));
            }
        }

        if (sTemp2 != null) {
            temp = sTemp2.indexOf("(more)");
            if (temp != -1) {
                sTemp2 = sTemp2.substring(0, temp - 10);
            }
            temp = sTemp2.indexOf(">WGA<");
            if (temp != -1) {
                sTemp2 = sTemp2.substring(temp + 5);
            }

            temp = sTemp2.indexOf("\">");
            String writer = "";
            while (temp != -1) {
                sTemp2 = sTemp2.substring(temp + 2);
                temp = sTemp.indexOf("\">");
                writer += sTemp2.substring(0, sTemp2.indexOf("<")) + ",";
            }
            htData.put(_WRITER, trim(writer.substring(0, writer.length() - 1)));
        }

        temp = sWebPage.indexOf("Genre");
        if (temp != -1) {
            sTemp = sWebPage.substring(temp);
            sTemp = sTemp.substring(0, sTemp.indexOf("<br>"));
            temp = sTemp.indexOf("/\">");
            String genre = "";
            while (temp != -1) {
                sTemp = sTemp.substring(temp + 3);
                genre += sTemp.substring(0, sTemp.indexOf("<")) + (",");
                temp = sTemp.indexOf("/\">");
            }
            htData.put(_GENRE, trim(genre.substring(0, genre.length() - 1)));
        }
        temp = sWebPage.indexOf("Tagline:");
        if (temp != -1) {
            sTemp = sWebPage.substring(temp);
            sTemp = sTemp.substring(13);
            htData.put(_TAGLINE, sTemp.substring(0, sTemp.indexOf("<")));
        }

        temp = sWebPage.indexOf("Plot Outline:");
        if (temp == -1)
            temp = sWebPage.indexOf("Plot Summary:");
        if (temp != -1) {
            sTemp = sWebPage.substring(temp);
            int end = sTemp.indexOf("<a") < sTemp.indexOf("<b")
                      ? sTemp.indexOf("<a")
                      : sTemp.indexOf("<b");
            String plot = trim(sTemp.substring(18, end));
            htData.put(_PLOT, plot);
        }

        temp = sWebPage.indexOf("User Rating:");
        if (temp != -1) {
            sTemp = sWebPage.substring(temp);
            sTemp = sTemp.substring(0, sTemp.indexOf("</p>"));
            temp = sTemp.indexOf("<b>");
            if (temp != -1) {
                sTemp = sTemp.substring(temp);
                try {
                    htData.put(_RATING, sTemp.substring(3, 6));
                    htData.put(_VOTES, sTemp.substring(sTemp.indexOf("(") + 1,
                                       sTemp.indexOf(" votes")));
                } catch (Exception e) {}
            }
        }

        temp = sTemp.indexOf("top 250: #");
        if (temp != -1) {
            sTemp = sTemp.substring(temp);
            htData.put(_PLACE, sTemp.substring(10, sTemp.indexOf("<")));

        }
        
        temp = sWebPage.indexOf("ast overview");
        if (temp == -1)
            temp = sWebPage.indexOf("Complete credited cast:");
        if (temp != -1) {
            sTemp = sWebPage.substring(temp);
            temp = sTemp.indexOf("href=\"");
            sTemp = sTemp.substring(temp, sTemp.indexOf("</table>"));
            temp = sTemp.indexOf("(more)");
            if (temp != -1) {
                sTemp = sTemp.substring(0, temp - 50);
            }

            temp = sTemp.indexOf("\">");
            String actors = "";
            while (temp != -1) {
                try {
                    sTemp = sTemp.substring(temp + 2);
                    actors += sTemp.substring(0, sTemp.indexOf("<")) + (", ");
                    
                    temp = sTemp.indexOf("\"top\">");
                    if (temp > -1) {
                        sTemp = sTemp.substring(temp + 6);
                    }
                    
                    temp = sTemp.indexOf("<");
                    if (temp > -1) {
                        actors += sTemp.substring(0, sTemp.indexOf("<") - 1) + (", ");
                    }
                    
                    temp = sTemp.indexOf("href");
                    if (temp > -1) {
                        sTemp = sTemp.substring(temp + 4);
                    }
                } catch (Exception ignore) {}
                
                temp = sTemp.indexOf("\">");
            }
            htData.put(_ACTORS, trim(actors.substring(0, actors.length() - 1)));
        }

        temp = sWebPage.indexOf("Runtime");
        String runtime = "";
        if (temp != -1) {
            sTemp = sWebPage.substring(temp + 13);
            if (sTemp.indexOf("<") < sTemp.indexOf(" ")) {
                runtime = sTemp.substring(0, sTemp.indexOf("<"));
            } else {
                temp2 = sTemp.indexOf("\t");
                if (temp2 != -1) {
                    if (sTemp.indexOf(" ") < temp2) {
                        runtime = sTemp.substring(0, sTemp.indexOf(" "));
                    } else {
                        runtime = sTemp.substring(0, sTemp.indexOf("\t"));
                    }
                } else {
                     runtime = sTemp.substring(0, sTemp.indexOf(" "));
                }
            }
            try {
                temp = new Integer(runtime).intValue();
            } catch (NumberFormatException e) {
                runtime = "";
            }
        }
        htData.put(_RUNTIME, runtime);

        temp = sWebPage.indexOf("Country:</b>");
        if (temp != -1) {
            sTemp = sWebPage.substring(temp + 12);
            sTemp = sTemp.substring(sTemp.indexOf(">") + 1);
            htData.put(_COUNTRY, sTemp.substring(0, sTemp.indexOf("<")));
        }
    }

    /**
     * Returns the value, parsed from the hmtl page
     *
     * @param identifier ie. _URL or _NAME
     */
    public String getValue(Object identifier) {
        Object o = htData.get(identifier);

        if (o == null) {
            return "";
        } else {
            return o.toString();
        }
    }

    public String trim(String sRaw) {
        try {
            byte[] byties = sRaw.getBytes("UTF8");
            sRaw = new String(byties);
        } catch (Exception exp) {
        }

        String sRetVal = sRaw;
        if ((sRetVal.startsWith("\"")) && (sRetVal.endsWith("\"")))
            sRetVal = sRetVal.substring(1, sRetVal.length() - 1);
            sRetVal = StringUtils.replaceAll(sRetVal, "&lt;i>", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#201;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#223;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#225;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#233;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#235;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#244;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#246;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#228;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#251;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#38;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#34;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "&#243;", "");
            sRetVal = StringUtils.replaceAll(sRetVal, "\n", "");
            return sRetVal;
    }
}
