/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.messages;

import java.util.Date;
import java.util.LinkedList;

import rjw.dc.core.Repository;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;

/**
 * Message queue. All classes can store a message in this class.
 * The message will be queued and written to the logfile and logform
 * when the program has time to do so. The writing and displaying of
 * messages is done by the class MessageWriter.
 * <p>
 * Last in last out mechanism is used
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Messages {

    /** Unspecief message type */
	public static final int _STANDARD = 0;
    /** Dateless message type */
	public static final int _LEAVEDATELESS = 1;
    /** Only the message text is displayed (indented) */
	public static final int _SIMPLE = 2;
    /** Query */
	public static final int _QUERY = 3;
    /** Unindented message type */
	public static final int _TABLESS = 4;
    /** Error message type */
	public static final int _ERROR = 5;

	/** The message pool, a simple vector */
	private static LinkedList lMessagePool = new LinkedList();

	public static boolean isNew = false;

	private static Thread writeToLog;

    /**
     * Starts the message writer thread
     *
     * @see MessagesWriter
     */
	public Messages() {
		writeToLog = new Thread(new MessagesWriter());
		writeToLog.setDaemon(true);
		writeToLog.start();
	}

    /**
     * Adds a message to the message pool
     *
     * @param message   message String
     * @param type      message type
     */
	public synchronized static void setMessage(String message, int type) {
		String newMessage = "";

		switch (type) {
			case _STANDARD :
				newMessage =
					Repository.operators.opTab
						+ getCurrentTime()
						+ Repository.operators.opSeparator
						+ message
						+ '\n';
				break;
			case _LEAVEDATELESS :
				newMessage = Repository.operators.opTab + message + '\n';
				break;
			case _QUERY :
				newMessage = '\n' + message + '\n' + '\n';
				break;
			case _SIMPLE :
				newMessage = Repository.operators.opTab + message + '\n';
				break;
			case _TABLESS :
				newMessage = message;
				break;
			case _ERROR :
				newMessage =
					'\n'
						+ Repository.operators.opTab
						+ Repository.operators.opTab
						+ Resources.getText(DCMessages.msgError)
						+ message
						+ '\n'
						+ '\n';
				break;
			default :
				newMessage = Repository.operators.opTab + message + '\n';
		}
		lMessagePool.add(newMessage);
		isNew = true;
	}

    /**
     * Gets a message from the pool. Always gets the first
     * (last in, last out)
     *
     * @return (String) message text
     */
	public synchronized static String getNewMessages() {
		Object message = null;
		if (lMessagePool.size() <= 0) {
			isNew = false;
			return null;
		} else {
			message = lMessagePool.removeFirst();
		}
		return message.toString();
	}

    /**
     * Gets the current time to use in the log file
     *
     * return (String)  representation of the current date and time
     */
	private static String getCurrentTime() {
        Date date = new Date(System.currentTimeMillis());
		String sCurrentTime = date.toString();
		return sCurrentTime;
	}
}
