/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.processes;

import rjw.dc.console.MessageBox;
import rjw.dc.console.software.AmazonSearchForm;
import rjw.dc.messages.Messages;
import rjw.dc.resources.DCLabels;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Resources;

/**
 * Updates a set of records by contacting the online database.
 * The process uses the AmazonSearchForm to get input from the user.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class AmazonUpdate extends Thread {

    private Object[][] data;
    private Object sender;

    /** Specifies whether the process is running */
    public static boolean running = false;

    /**
     * Initializes the process. All preferences are set
     *
     * @param data[][0] the ID of the row
     * @param data[][1] the name of the movie
     * @param update search values for new or existing data ?
     */
    public AmazonUpdate(  Object[][] data,
                          Object sender) {

        this.sender = sender;
        this.data = data;
    }

    /**
     * Starts the process
     */
    public void run() {
        try {
            running = true;
            searchAmazon(data);
        } catch (Exception exp) {
            exp.printStackTrace();
            Messages.setMessage(exp.toString(), Messages._ERROR);
            new MessageBox(Resources.getText(DCMessages.msgProcessCrash),
                           true, MessageBox._ERROR);
            finishAll();
        }
        finishAll();
    }

    /**
     * Opens a result form for every Imdb request so the user can select
     * the most likely one
     */
    public void searchAmazon(Object[][] data) {
        if (data == null) {
            return;
        }

        for (int i = 0; i < data.length; i++) {
            int id = -1;
            if (data[i][0] != null) {
                id = ((Integer) data[i][0]).intValue();
            }
            String name = data[i][1].toString();
            
            String title = "";
            if (name == null || name.trim().equals("")) {
                
            } else {
                title = Resources.getText(DCLabels.lblSearchingFor,
                             new String[] {name, String.valueOf(i+1),
                                           String.valueOf(data.length)});
            }
            
            AmazonSearchForm searchForm = new AmazonSearchForm(
                                              id,
                                              title,
                                              name,
                                              sender);

            while (searchForm.isUpdateRunning()) {
                try {
                    sleep(3000);
                } catch (InterruptedException ignore) {}
            }

            if (!searchForm.continueNext()) {
                break;
            }
        }
        finishAll();
    }

    /**
     * Stops the process
     */
    private void finishAll() {
        try {
            running = false;
            Messages.setMessage(
                Resources.getText(DCMessages.msgIMDBProcessStops),
                Messages._STANDARD);
        } catch (Exception ignore) {}
    }
}
