/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.dc.processes;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.Enumeration;
import java.util.Vector;

import mediaManager.video.FilePropertiesMovie;
import rjw.dc.console.MessageBox;
import rjw.dc.console.movie.InsertMoviePanel;
import rjw.dc.core.Repository;
import rjw.dc.core.objects.CdRom;
import rjw.dc.core.objects.Requestor;
import rjw.dc.core.objects.RequestorCollection;
import rjw.dc.core.objects.movie.Movie;
import rjw.dc.core.objects.movie.MovieFile;
import rjw.dc.messages.Messages;
import rjw.dc.messages.MessagesWriter;
import rjw.dc.resources.DCMessages;
import rjw.dc.util.Converter;
import rjw.dc.util.Resources;
import rjw.dc.util.StringUtils;
import rjw.dc.wf.WorkFlow;
import rjw.flex.settings.Settings;

/**
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class ExportMovieCD extends Thread {

    private CdRom cdRom = new CdRom();
    private String location = "";
    private boolean bConfirmDirs = false;
    private String cdName = "";
    private String file = "";
    private boolean bToTable = false;
    private boolean bToFile = false;
    /** Specifies whether the export process is running */
    public static boolean running = false;

    private static final WorkFlow wf = new WorkFlow();

    /**
     * Initializes the export process. All preferences are set
     *
     * @param loc location from which the data is read
     * @param bConfirmDirs  indicates if per directory should be asked if the data
     *                      should be read from this directory
     * @param file the export filename
     * @param cdName name of the location, used for grouping data
     */
    public ExportMovieCD(   String loc,
                            boolean bConfirmDirs,
                            String file,
                            String cdName,
                            boolean bToFile,
                            boolean bToTable) {

        if (!loc.endsWith("\\") && !loc.endsWith("/")) {
            this.location = loc;
        } else {
            this.location = loc.substring(0, loc.length() -1);
        }
        this.bConfirmDirs = bConfirmDirs;
        this.file = file;
        this.cdName = cdName;
        this.bToFile = bToFile;
        this.bToTable = bToTable;
    }

    /**
     * Starts the export process. The data read from the location is written
     * to the export file. Information from the files is separated by..
     * seperators as specified in the IIF. Errors are shown to user in
     * messageboxes
     */
    public void run() {
        try {
            running = true;
            Vector filesAndDirs = cdRom.getInformation( location, 
                                                        bConfirmDirs,
                                                        true,
                                                        false,
                                                        null);
            writeToFile(filesAndDirs);
        } catch (Exception exp) {
            exp.printStackTrace();
            MessagesWriter._useLogFile = true;
            Messages.setMessage(exp.toString(), Messages._ERROR);
            new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                           true, MessageBox._ERROR);
            finishAll();
        }
        new MessageBox(Resources.getText(DCMessages.msgExportCompleted), 
                       true, MessageBox._INFORMATION);
        finishAll();
    }

    /**
     * Writes the data to the file. Information from the files is separated by..
     * seperators as specified in the IIF.
     */
    public void writeToFile(Vector data) {
        File sourceFile;

        try {
            RandomAccessFile fileAccess = null;
            
            new MessageBox(Resources.getText(DCMessages.msgWriteToFile), 
                           true, MessageBox._INFORMATION);
            
            if (bToFile) {
                try {
                    sourceFile = new File(file.trim());
                } catch (Exception exp) {
                    sourceFile = new File(Settings.getValueAsString(Repository.settings.stMovieExportFile));
                }
                
                fileAccess = new RandomAccessFile(sourceFile, "rw");
                Messages.setMessage(Resources.getText(DCMessages.msgWritingInfo), 
                                    Messages._STANDARD);
    
                fileAccess.seek(fileAccess.length());
                fileAccess.writeBytes("->" + Repository.dataBase.dBMovieTableName.trim() + "<-" + '\n');
            }

            boolean first = true;

            for (Enumeration enum = data.elements(); enum.hasMoreElements(); ) {
                String sFilename = (String) enum.nextElement();
                Object[] movieInfo = getMovieInformation(sFilename);
                if (movieInfo != null) {
                    Messages.setMessage(
                        Resources.getText(DCMessages.msgWritingToFile, sFilename),
                        Messages._STANDARD);
    
                    if (first && bToFile) {
                        fileAccess.seek(fileAccess.length());
    
                        for (int i = 0; i < movieInfo.length; i++) {
                            fileAccess.writeBytes(movieInfo[i] + " > ");
                        }
    
                        fileAccess.writeBytes("[" + cdName + "]" + '\n');
                        first = false;
                    } else  if (bToFile) {
                        try {
                            sleep(10);
                        } catch (InterruptedException intExp) {}
    
                        for (int i = 0; i < movieInfo.length; i++) {
                            fileAccess.writeBytes(movieInfo[i] + " > ");
                        }
                        fileAccess.writeBytes("\n");
                    }
    
                    if (bToTable) {
                        Movie movie = new Movie();
                        movie.setValues(movieInfo);
                        movie.setValue(Movie._U_LOCATION, cdName);
                        Vector vTable = new Vector(0);
                        vTable.add(movie);
                        Requestor req = new Requestor(InsertMoviePanel.class,
                                                      "addRow",
                                                      null);
                        RequestorCollection reqCol = new RequestorCollection();
                        reqCol.addRequestor(req);
                        wf.displayResults(vTable, reqCol);
                    }
                }
            }

            Messages.setMessage(
                    Resources.getText(DCMessages.msgCreatingIIF), 
                    Messages._STANDARD);

            String[][] IIFData = { {Repository.dataBase.dBMovieColTitle, ">" },
                                   {Repository.dataBase.dBMovieColFileName, ">" },
                                   {Repository.dataBase.dBMovieColPlayLength, ">" },
                                   {Repository.dataBase.dBMovieColVideoCodec, ">" },
                                   {Repository.dataBase.dBMovieColAudioCodec, ">" },
                                   {Repository.dataBase.dBMovieColWidth, ">" },
                                   {Repository.dataBase.dBMovieColHeight, ">" },
                                   {Repository.dataBase.dBMovieColFps, ">" },
                                   {Repository.dataBase.dBMovieColFrames, ">" },
                                   {Repository.dataBase.dBMovieColAudioBitRate, ">" },
                                   {Repository.dataBase.dBMovieColAudioSample, ">" },
                                   {Repository.dataBase.dBMovieColAudioChannels, ">" },
                                   {Repository.dataBase.dBMovieColVideoBitRate, ">" }
                                   
            };

            new ImportInformationFile(IIFData, file);
            Messages.setMessage(
                Resources.getText(DCMessages.msgCreatedIIF), 
                Messages._STANDARD);
            
            if (bToFile) {
                fileAccess.close();
            }

        } catch (Exception exp) {
            MessagesWriter._useLogFile = true;
            new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                           true, MessageBox._ERROR);
            Messages.setMessage(exp.toString(), Messages._ERROR);
            finishAll();
        }
    }

    /**
     * Stops the process
     */
    private void finishAll() {
        try {
            running = false;
            MessagesWriter._useLogFile = true;
            Messages.setMessage(
                Resources.getText(DCMessages.msgExportStops), 
                Messages._STANDARD);
        } catch (Exception exp) {
        }
    }

    private Object[] getMovieInformation(String sFile) {
        Object[] movieInfo = new Object[Repository.dataBase.dbMovieColArrayPK.length];
        try {
            FilePropertiesMovie movieFile1 = new FilePropertiesMovie(sFile);
            

            String sFileName    = (String) toValidString(sFile);
            movieInfo[Movie._A_TITLE] = toValidString(movieFile1.getVideoName());
            
            Integer playlength = new Integer(movieFile1.getDuration() / 60);
            
            // if playlength could not be retrieved, consider this not a movie file!
            if (playlength.intValue() == 0) {
                return null;
            }
            
            movieInfo[Movie._I_PLAYLENGTH] = playlength; 
            movieInfo[Movie._J_FILENAME] = sFile.substring(2, sFile.length());
            movieInfo[Movie._K_VIDEOCODEC] = toValidString(movieFile1.getVideoCodec());
            movieInfo[Movie._L_AUDIOCODEC] = toValidString(movieFile1.getAudioCodec());
            movieInfo[Movie._M_WIDTH] = new Integer(movieFile1.getVideoWidth());
            movieInfo[Movie._N_HEIGHT] = new Integer(movieFile1.getVideoHeight());
            movieInfo[Movie._O_FPS] = new Integer(movieFile1.getVideoFps());
            movieInfo[Movie._P_FRAMES] = new Integer(movieFile1.getDuration() * movieFile1.getVideoFps());
            movieInfo[Movie._Q_AUDIOBITRATE] = new Integer(movieFile1.getAudioBitRate());
            movieInfo[Movie._R_AUDIOSAMPLINGRATE] = new Integer(movieFile1.getAudioSampleRate());
            movieInfo[Movie._S_AUDIOCHANNEL] = new Integer(movieFile1.getAudioChannels());
            
            int bitrate = movieFile1.getVideoBitRate();
            if (bitrate <= 0) {
                try {
                    MovieFile movieFile2 = new MovieFile(sFile);
                    bitrate = new Double(movieFile2.getVideoBitrate()).intValue();
                } catch (Exception ignore) {}
            }
            
            movieInfo[Movie._T_VIDEOBITRATE] = new Integer(bitrate);
            movieInfo[Movie._U_LOCATION] = location;

        } catch (Exception exp) {
            Messages.setMessage(exp.toString(), Messages._ERROR);
            return null;
        }
        return movieInfo;
    }

    /**
     * Null values are converted to the database standard, single quotes are converted
     * and null characters are converted to spaces
     *
     * @param sValue    value to be checked
     * @return (String) value which complies with the database standard
     */         
    protected String toValidString(String value) {
        if (StringUtils.isEmpty(value)) {
            return "Null";   
        }
        return Converter.databaseValueConverter(value);
    }
}
