/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   * 
 *                           sub-project: JFlexSett                           *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package rjw.flex.settings;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

/**
 * Reads and saves data to the Settings File
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class SettingsFile {

    /**
     * Save the settings file
     */
    public static void save() {
        Properties properties = new Properties();
        
        Vector vSettings = Settings.getSettings();
        for (Enumeration enum = vSettings.elements(); enum.hasMoreElements(); ) {
            Setting setting = (Setting) enum.nextElement();
            properties.setProperty(setting.getKey(), setting.getValueAsString());
        }

        try {
            FileOutputStream fos = new FileOutputStream(Settings.getSettingsFile());
            properties.store(fos, "");
            fos.close();
        } catch (Exception exp) {
            exp.printStackTrace();    
        } 
    }

    /**
     * Loads the settings file
     */
    public static void load() {
        Properties properties = new Properties();
        
        try {
            FileInputStream fis = new FileInputStream(Settings.getSettingsFile());
            properties.load(fis);
            fis.close();
        } catch (Exception exp) {
            // Oh gee! Unable to load.. defaults will be used.
            // exp.printStackTrace();
        }
        
        Vector vSettings = Settings.getSettings();
        
        for (Enumeration enum = vSettings.elements(); enum.hasMoreElements(); ) {
            Setting setting = (Setting) enum.nextElement();
            if (properties.getProperty(setting.getKey()) != null) {
                String sValue = properties.getProperty(setting.getKey()).toString();
                Settings.setStringAsValue(setting.getKey(), sValue);
            }
        }        
    }
}
